// ==============================================================
//
// hst_ex.cpp
// Control module for hst_ex vessel class
//
// ==============================================================

#define STRICT
#define ORBITER_MODULE

#include "orbitersdk.h"
#include "gcAPI.h"

// ==============================================================
// Some vessel parameters
// ==============================================================

static double sunshield_dur = 0.015;
static double hga_dur = 0.03;
static double hga_rot_dur = 0.1;
static double fgs_door_dur = 0.07;
static double startrk_door_dur = 0.07;
static double pos_v2_door_dur = 0.07;
static double neg_v2_door_dur = 0.07;

static VECTOR3 beaconcol1 = _V(1.0,0.3,0.0);
static VECTOR3 beaconcol2 = _V(0.1,1.0,0.1);


#define GREEN RGB(125, 255, 125)

// ==============================================================
// Shuttle-PB class interface
// ==============================================================

class hst_ex: public VESSEL3 {
public:
	hst_ex (OBJHANDLE hVessel, int flightmodel);
	~hst_ex ();
	void clbkSetClassCaps (FILEHANDLE cfg);
	void clbkPreStep(double SimT, double SimDT, double MJD);
	//////void clbkPostStep (double simt, double simdt, double mjd);
	void DefineAnimations (void);
	void clbkSaveState (FILEHANDLE scn);
	void clbkLoadStateEx (FILEHANDLE scn, void *status);
	int clbkConsumeBufferedKey(DWORD key, bool down, char *kstate);
	int clbkConsumeDirectKey(char *kstate);
	bool clbkDrawHUD (int mode, const HUDPAINTSPEC *hps, oapi::Sketchpad *skp);
	void clbkPostCreation();

	void SetHGA(void);
	void StowHGA(void);
	void DoKillRot(void);
	void GoToTarget(void);
	void GoToRADec(void);
	bool EnterTarget(char *tname);
	bool EnterRA(char *name1);
	bool EnterDec(char *name2);
	void IncreaseFOV(void);
	void DecreaseFOV(void);
	void DoBeacons(void);
	void CheckVessels(void);
	void CheckVessels2(void);
	void SpawnWFC(void);
	void SpawnInst1(void);
	void SpawnInst2(void);
	void SpawnInst3(void);
	void SpawnInst4(void);
	void SpawnFGS1(void);
	void SpawnFGS2(void);

	void SpawnRSU1(void);
	void SpawnRSU2(void);
	void SpawnRSU3(void);

	void SpawnStbdSA(void);
	void SpawnPortSA(void);

	void SpawnStbdSA2(void);
	void SpawnPortSA2(void);

	void DoAutoSun(void);
	void DoRevertRot(void);

	BEACONLIGHTSPEC beacon[27];
	ANIMATIONCOMPONENT_HANDLE hst_ex_ch[48];

	ATTACHMENTHANDLE wfc_attach, fgs1_attach, fgs2_attach, wif1_attach, wif2_attach, scm_attach, grap1_attach, grap2_attach;
	ATTACHMENTHANDLE inst1_attach, inst2_attach, inst3_attach, inst4_attach, rsu1_attach, rsu2_attach, rsu3_attach;
	ATTACHMENTHANDLE sa_stbd_attach, sa_port_attach;
	MESHHANDLE hmesh1, hmesh2, hmesh3, hmesh4 , hmesh5, hmesh6;

private:
	UINT anim_sunsh, anim_hga_deploy, anim_hga_posy_alt, anim_hga_posy_az, anim_hga_negy_alt, anim_hga_negy_az;
	double sunsh_proc, hga_deploy_proc, hga_posy_alt_proc, hga_negy_alt_proc, hga_posy_az_proc, hga_negy_az_proc;
	
	UINT anim_fgs1, anim_fgs, anim_startrk, anim_posv2, anim_negv2;
	double fgs_proc, fgs1_proc, startrk_proc, posv2_proc, negv2_proc;
	UINT anim_fgs1b,anim_fgsb, anim_startrkb, anim_posv2b, anim_negv2b;
	double fgsb_proc, fgs1b_proc, startrkb_proc, posv2b_proc, negv2b_proc;

	UINT anim_sa1_port_elb, anim_sa1_port_rot, anim_sa1_port_scale, anim_sa2_port_rot, anim_sa2_port_deploy;
	double sa1_port_elb_proc, sa1_port_rot_proc, sa1_port_scale_proc;
	int sa1_port_elb_sts, sa1_port_scale_sts;

	UINT anim_sa1_stbd_elb, anim_sa1_stbd_rot, anim_sa1_stbd_scale, anim_sa2_stbd_rot, anim_sa2_stbd_deploy;
	double sa1_stbd_elb_proc, sa1_stbd_rot_proc, sa1_stbd_scale_proc;
	int sa1_stbd_elb_sts, sa1_stbd_scale_sts;

	UINT mesh1, mesh2, mesh3, mesh4, mesh5, mesh6, mesh7, mesh8, mesh9, mesh10, mesh11, mesh12;
	UINT mesh13, mesh14, mesh15, mesh16;

	int sunshield_status, hga_status;
	int door_status[10];
	int ap_status, dcr_fov, icr_fov, dobeacon_st;
	char refname[256];

	int wfc_status, inst1_status, inst2_status, inst3_status, inst4_status;
	int fgs1_status, fgs2_status, rsu1_status, rsu2_status, rsu3_status;
	int sa_stbd_status, sa_port_status;
	int sa_stbd_cfg, sa_port_cfg;


	double step_time;
	double tgt_ra, tgt_ra_rad;
	int tgt_ra_int, tgt_ra_hh, tgt_ra_mm, tgt_ra_ss;
	double tgt_dec, tgt_dec_rad;
	int tgt_dec_int, tgt_dec_hh, tgt_dec_mm, tgt_dec_ss;
	VECTOR3 beaconpos[27];
	VECTOR3 attp_pos[14];

	int att_pc_check;
	int cv1cv2_check;

	int autosun_status;

	int hst_cfg;
	int anot_status, anot2_status;
	double anot_timer, anot2_timer;

	char inst1_name[6], inst2_name[6], inst3_name[6], inst4_name[6];

	NOTEHANDLE hNote, hNote2;

	

	
};

hst_ex::hst_ex (OBJHANDLE hVessel, int flightmodel)
: VESSEL3 (hVessel, flightmodel)
{
    

}

hst_ex::~hst_ex ()
{
}




// ==============================================================
// Overloaded callback functions
// ==============================================================

// --------------------------------------------------------------
// Set the capabilities of the vessel class
// --------------------------------------------------------------
void hst_ex::clbkSetClassCaps (FILEHANDLE cfg)
{
	sunsh_proc = 0;
	hga_deploy_proc = 0;

	hga_posy_alt_proc = 0;
	hga_negy_alt_proc = 0;
	hga_posy_az_proc = 0;
	hga_negy_az_proc = 0;

	fgs_proc = 0;
	fgs1_proc = 0;
	startrk_proc = 0;
	posv2_proc = 0;
	negv2_proc = 0;

	fgsb_proc = 0;
	fgs1b_proc = 0;
	startrkb_proc = 0;
	posv2b_proc = 0;
	negv2b_proc = 0;

	sunshield_status = 0;
	hga_status = 0;

	autosun_status = 0;
	hst_cfg = 0;
	anot_status = 0;
	anot2_status = 0;
	anot_timer = 0;
	anot2_timer = 0;

	sa_stbd_cfg = 0;
	sa_port_cfg = 0;

	door_status[0] = 0;
	door_status[1] = 0;
	door_status[2] = 0;
	door_status[3] = 0;
	door_status[4] = 0;
	door_status[5] = 0;
	door_status[6] = 0;
	door_status[7] = 0;
	door_status[8] = 0;
	door_status[9] = 0;

	sa1_port_elb_proc = 1;
	sa1_port_rot_proc = 0;
	sa1_port_scale_proc = 1;

	sa1_port_elb_sts = 2;
	sa1_port_scale_sts = 2;

	sa1_stbd_elb_proc = 1;
	sa1_stbd_rot_proc = 0;
	sa1_stbd_scale_proc = 1;

	sa1_stbd_elb_sts = 2;
	sa1_stbd_scale_sts = 2;
	

	step_time = 0;
	ap_status = 0;
	dcr_fov = 0;
	icr_fov = 0;
	dobeacon_st = 0;

	att_pc_check = 0;
	cv1cv2_check = 0;

	wfc_status = 0;
	inst1_status = 0;
	inst2_status = 0;
	inst3_status = 0;
	inst4_status = 0;
	fgs1_status = 0;
	fgs2_status = 0;
	rsu1_status = 0;
	rsu2_status = 0;
	rsu3_status = 0;

	sa_stbd_status = 0;
	sa_port_status = 0;

	strcpy(inst1_name, "GHRS");
	strcpy(inst2_name, "FOS");
	strcpy(inst3_name, "FOC");
	strcpy(inst4_name, "HSP");
	

	tgt_ra = 0;
	tgt_ra_int = 0;
	tgt_ra_hh = 0;
	tgt_ra_mm = 0;
	tgt_ra_ss = 0;
	tgt_dec = 0;
	tgt_dec_int = 0;
	tgt_dec_hh = 0;
	tgt_dec_mm = 0;
	tgt_dec_ss = 0;

	tgt_ra_rad = 0;
	tgt_dec_rad = 0.0000001;
	

	strcpy(refname, "Moon");

	DefineAnimations();
	gcInitialize();

	// physical vessel parameters
	SetSize (10);
	SetEmptyMass (11300);
	SetPMI(_V(13,13,3));
	SetCrossSections (_V(40,80,20));
	SetCW(0.1,0.1,0.1,0.1);
	SetRotDrag (_V(1,1,1));
	SetTouchdownPoints ( _V(0,-2, 5), _V(-2,-2,-5), _V( 2,-2,-5));

	// camera parameters
	SetCameraOffset (_V(0,0,2));
	SetCameraRotationRange (0,0,0,0);

	// att point pos for grap checks
	attp_pos[0] = _V(0, -0.0924, -2.453);
	attp_pos[1] = _V(-0.625, 1.382, -2.418);
	attp_pos[2] = _V(-1.518, 1.4696, -3.168);
	attp_pos[3] = _V(0, 2.55, -1.963);

	attp_pos[4] = _V(-0.5, 0.472, -5.2);	// inst1
	attp_pos[5] = _V(-0.5, -0.472, -5.2);	// inst2
	attp_pos[6] = _V(0.5, -0.472, -5.2);	// inst3
	attp_pos[7] = _V(0.5, 0.472, -5.2);// inst4

	attp_pos[8] = _V(1.382, 0.625, -2.418); // fgs1

	attp_pos[9] = _V(0.689, -1.404, -3.022); // rsu1
	attp_pos[10] = _V(0, -1.316, -3.319); // rsu2
	attp_pos[11] = _V(-0.689, -1.404, -3.022); // rsu3

	attp_pos[12] = _V(1.775, 0, -0.24); // sa_stbd
	attp_pos[13] = _V(-1.773, 0, -0.24); // sa_port


	// wfc_attach, fgs2_attach, wif1_attach, wif2_attach, scm_attach, grap1_attach, grap2_attach;
	wfc_attach = CreateAttachment (false, attp_pos[0], _V(0,0,1), _V(0,1,0), "WFC");
	fgs2_attach = CreateAttachment (false, attp_pos[1], _V(0,1,0), _V(0,0,1), "FGS");
	//wif1_attach = CreateAttachment (false, attp_pos[2], _V(0,0,-1), _V(0,-1,0), "WIF1");
	wif2_attach = CreateAttachment (true, attp_pos[3], _V(0,1,0), _V(0,0,1), "XS");

	inst1_attach = CreateAttachment (false, attp_pos[4], _V(1,0,0), _V(0,1,0), "HI1");
	inst2_attach = CreateAttachment (false, attp_pos[5], _V(1,0,0), _V(0,-1,0), "HI2");
	inst3_attach = CreateAttachment (false, attp_pos[6], _V(-1,0,0), _V(0,-1,0), "HI3");
	inst4_attach = CreateAttachment (false, attp_pos[7], _V(-1,0,0), _V(0,1,0), "HI4");

	fgs1_attach = CreateAttachment (false, attp_pos[8], _V(1,0,0), _V(0,0,1), "FGS");

	rsu1_attach = CreateAttachment (false, attp_pos[9], _V(-0.7071, 0, 0.7071), _V(0.7071, 0, 0.7071), "RSU");
	rsu2_attach = CreateAttachment (false, attp_pos[10], _V(0,1,0), _V(-1,0,0), "RSU");
	rsu3_attach = CreateAttachment (false, attp_pos[11], _V(0.7071, 0, 0.7071), _V(0.7071, 0, -0.7071), "RSU");

	sa_stbd_attach = CreateAttachment (false, attp_pos[12], _V(1,0,0), _V(0,1,0), "SAS");
	sa_port_attach = CreateAttachment (false, attp_pos[13], _V(-1,0,0), _V(0,-1,0), "SAP");

	scm_attach = CreateAttachment (true, _V(0, 0, -5.683), _V(0,0,-1), _V(1,0,0), "SCM");
	grap1_attach = CreateAttachment (true, _V(-1.034, -1.4124, 0.99), _V(0,-1,0), _V(0,0,-1), "GS");
	grap2_attach = CreateAttachment (true, _V( 1.034, -1.4124, 0.99), _V(0,-1,0), _V(0,0,-1), "GS");

	if (gcEnabled() == true)
	{
	mesh1 = AddMesh("HST_EX/DX9/hst_ex");
	mesh2 = AddMesh("HST_EX/hst_wfc");
	mesh3 = AddMesh("HST_EX/hst_inst1");
	mesh4 = AddMesh("HST_EX/hst_inst2");
	mesh5 = AddMesh("HST_EX/hst_inst3");
	mesh6 = AddMesh("HST_EX/hst_inst4");
	mesh7 = AddMesh("HST_EX/hst_fgs1");
	mesh8 = AddMesh("HST_EX/hst_fgs2");
	mesh9 = AddMesh("HST_EX/hst_fgs3");

	mesh10 = AddMesh("HST_EX/hst_rsu1");
	mesh11 = AddMesh("HST_EX/hst_rsu2");
	mesh12 = AddMesh("HST_EX/hst_rsu3");

	mesh13 = AddMesh("HST_EX/DX9/hst_sa1_port");
	mesh14 = AddMesh("HST_EX/DX9/hst_sa1_stbd");

	mesh15 = AddMesh("HST_EX/DX9/hst_sa2_port");
	mesh16 = AddMesh("HST_EX/DX9/hst_sa2_stbd");
	}
	else
	{
	mesh1 = AddMesh("HST_EX/DX7/hst_ex");
	mesh2 = AddMesh("HST_EX/hst_wfc");
	mesh3 = AddMesh("HST_EX/hst_inst1");
	mesh4 = AddMesh("HST_EX/hst_inst2");
	mesh5 = AddMesh("HST_EX/hst_inst3");
	mesh6 = AddMesh("HST_EX/hst_inst4");
	mesh7 = AddMesh("HST_EX/hst_fgs1");
	mesh8 = AddMesh("HST_EX/hst_fgs2");
	mesh9 = AddMesh("HST_EX/hst_fgs3");

	mesh10 = AddMesh("HST_EX/hst_rsu1");
	mesh11 = AddMesh("HST_EX/hst_rsu2");
	mesh12 = AddMesh("HST_EX/hst_rsu3");

	mesh13 = AddMesh("HST_EX/DX7/hst_sa1_port");
	mesh14 = AddMesh("HST_EX/DX7/hst_sa1_stbd");

	mesh15 = AddMesh("HST_EX/DX7/hst_sa2_port");
	mesh16 = AddMesh("HST_EX/DX7/hst_sa2_stbd");
	}

	// DX7/DX9 common parts
	


	beaconpos[0] = _V(0.2728, -2.0603, -4.0656); // st door closed
	beaconpos[1] = _V(-0.0055, 2.0702, -2.4706); // fgs door closed
	beaconpos[3] = _V(2.0702, -0.0044, -4.0656); // V2+ door closed
	beaconpos[2] = _V(-2.079, -0.0044, -4.0656); // V2- door closed
	


	beaconpos[4] = _V(1.4872, 0, 0.2343); // +x sa
	beaconpos[5] = _V(-1.4839, 0, 0.2156); // -x sa


	beaconpos[7] = _V(1.4938, -2.4464, -4.0656); // st door +x open
	beaconpos[6] = _V(-1.7424, -2.9612, -4.0656); // st door -x open

	
	beaconpos[9] = _V(1.1825, 2.5839, -2.4706); // fgs door +x open
	beaconpos[8] = _V(-1.1935, 2.5839, -2.4706); // fgs door -x open

	
	beaconpos[12] = _V(2.6774, 1.6544, -4.0656); // V2+ door +y open
	beaconpos[13] = _V(2.6774, -1.6456, -4.0656); // V2+ door -y open

	
	beaconpos[11] = _V(-2.6862, 1.6544, -4.0656); // V2- door +y open
	beaconpos[10] = _V(-2.6862, -1.6456, -4.0656); // V2- door -y open


	beaconpos[14] = _V(0, -2.112, -2.7973); // wfc

	beaconpos[15] = _V(-1.03, 0.424, -2.869); // inst1
	beaconpos[16] = _V(-1.03, -0.424, -2.869); // inst2
	beaconpos[17] = _V(0.95, -0.424, -2.869); // inst3
	beaconpos[18] = _V(0.95, 0.424, -2.869); // inst4

	beaconpos[19] = _V(0, 1.925, -2.784); // fgs2
	beaconpos[20] = _V(1.925, 0, -2.784); // fgs1


	beaconpos[21] = _V(2.063, -0.004, -2.473); // fgs1 door closed
	beaconpos[22] = _V(2.562, 1.148, -2.473); // fgs1 door open
	beaconpos[23] = _V(2.562, -1.155, -2.473); // fgs1 door open

	beaconpos[24] = _V(0.439,-1.406,-2.921); // rsu1
	beaconpos[25] = _V(0.203,-1.314, -3.294); // rsu2
	beaconpos[26] = _V(-0.439,-1.406,-2.921); // rsu3


	///// add beacons
	for (int i = 0; i < 27; i++) 
	{
		beacon[i].shape = BEACONSHAPE_COMPACT;
		beacon[i].pos = &beaconpos[i];
		beacon[i].col = &beaconcol1;
		beacon[i].size = 0.1;
		beacon[i].falloff = 0.1;
		beacon[i].period = 1;
		beacon[i].duration = 0.75;
		beacon[i].tofs = 0;
		beacon[i].active = false;
		AddBeacon (&beacon[i]);
	}

	for (int i = 6; i < 14; i++) 
	{
		beacon[i].col = &beaconcol2;
	}

	for (int i = 22; i < 24; i++) 
	{
		beacon[i].col = &beaconcol2;
	}


}



// ==============================================================
// API callback interface
// ==============================================================

// --------------------------------------------------------------
// Vessel initialisation
// --------------------------------------------------------------
DLLCLBK VESSEL *ovcInit (OBJHANDLE hvessel, int flightmodel)
{
	return new hst_ex (hvessel, flightmodel);
}

// --------------------------------------------------------------
// Vessel cleanup
// --------------------------------------------------------------
DLLCLBK void ovcExit (VESSEL *vessel)
{
	if (vessel) delete (hst_ex*)vessel;
}

void hst_ex::DefineAnimations (void)
{
// 1. Sunshield

	anim_sunsh = CreateAnimation (0);

	static UINT SunshGrp[2] = {30,31};
	static MGROUP_ROTATE SunshAnim (0, SunshGrp, 2, _V(0.0, 1.4366, 7.26), _V(1,0,0), (float)(-105.25*RAD));	
	AddAnimationComponent (anim_sunsh, 0, 1, &SunshAnim);


// 2. Deploy/Retract HGA masts

	anim_hga_deploy = CreateAnimation (0);

	static UINT HGA1Grp1[3] = {12,13,14};
	static MGROUP_ROTATE HGA1Anim1 (0, HGA1Grp1, 3, _V(0, 1.7996, 0.363), _V(1,0,0), (float)(-93*RAD));
	hst_ex_ch[0] = AddAnimationComponent (anim_hga_deploy, 0, 1, &HGA1Anim1);

	static UINT HGA2Grp1[3] = {17,18,19};
	static MGROUP_ROTATE HGA2Anim1 (0, HGA2Grp1, 3, _V(0, -1.7424, 0.363), _V(1,0,0), (float)(93*RAD));
	hst_ex_ch[1] = AddAnimationComponent (anim_hga_deploy, 0, 1, &HGA2Anim1);


// 3. HGA dish rotations

	// dish azimuth
	anim_hga_posy_az = CreateAnimation (0);

	static UINT HGA1Grp2[1] = {16};
	static MGROUP_ROTATE HGA1Anim2 (0, HGA1Grp2, 1, _V(-0.011, 1.5884, 4.279), _V(0,-0.052336,0.99863), (float)(360*RAD));
	hst_ex_ch[2] = AddAnimationComponent (anim_hga_posy_az, 0, 1, &HGA1Anim2, hst_ex_ch[0]);


	anim_hga_negy_az = CreateAnimation (0);

	static UINT HGA2Grp2[1] = {21};
	static MGROUP_ROTATE HGA2Anim2 (0, HGA2Grp2, 1, _V(-0.011, -1.5356, 4.279), _V(0, -0.052336,-0.99863), (float)(360*RAD));
	hst_ex_ch[3] = AddAnimationComponent (anim_hga_negy_az, 0, 1, &HGA2Anim2, hst_ex_ch[1]);


	// dish altitude
	anim_hga_posy_alt = CreateAnimation (0);

	static UINT HGA1Grp3[1] = {15};
	static MGROUP_ROTATE HGA1Anim3 (0, HGA1Grp3, 1, _V(-0.011, 1.5884, 4.279), _V(1,0,0), (float)(90*RAD));
	AddAnimationComponent (anim_hga_posy_alt, 0, 1, &HGA1Anim3, hst_ex_ch[2]);


	anim_hga_negy_alt = CreateAnimation (0);

	static UINT HGA2Grp3[1] = {20};
	static MGROUP_ROTATE HGA2Anim3 (0, HGA2Grp3, 1, _V(-0.011, -1.5356, 4.279), _V(1,0,0), (float)(-90*RAD));
	AddAnimationComponent (anim_hga_negy_alt, 0, 1, &HGA2Anim3, hst_ex_ch[3]);



// 4. FGS doors

	anim_fgs = CreateAnimation (0);

	static UINT FGSGrp1[3] = {9,10,11};
	static MGROUP_ROTATE FGSAnim1 (0, FGSGrp1, 3, _V(-0.77, 1.9206, -2.486), _V(0, 0, 1), (float)(110*RAD));
	AddAnimationComponent (anim_fgs, 0, 1, &FGSAnim1);


	anim_fgsb = CreateAnimation (0);

	static UINT FGSGrp2[3] = {6,7,8};
	static MGROUP_ROTATE FGSAnim2 (0, FGSGrp2, 3, _V(0.759, 1.9206, -2.486), _V(0, 0, 1), (float)(-110*RAD));
	AddAnimationComponent (anim_fgsb, 0, 1, &FGSAnim2);

// 4B. FGS1 doors

	anim_fgs1 = CreateAnimation (0);

	static UINT FGS1Grp1[3] = {0,1,2};
	static MGROUP_ROTATE FGS1Anim1 (0, FGS1Grp1, 3, _V(1.922, 0.765, -2.486), _V(0, 0, 1), (float)(110*RAD));
	AddAnimationComponent (anim_fgs1, 0, 1, &FGS1Anim1);


	anim_fgs1b = CreateAnimation (0);

	static UINT FGS1Grp2[3] = {3,4,5};
	static MGROUP_ROTATE FGS1Anim2 (0, FGS1Grp2, 3, _V(1.922,-0.773, -2.486), _V(0, 0, 1), (float)(-110*RAD));
	AddAnimationComponent (anim_fgs1b, 0, 1, &FGS1Anim2);


// 5. Star Tracker Doors

	anim_startrk = CreateAnimation (0);

	static UINT STGrp1[4] = {26,27,28,29};
	static MGROUP_ROTATE STAnim1 (0, STGrp1, 4, _V(-1.045, -1.7974, -3.993), _V(0, 0, 1), (float)(-110*RAD));
	AddAnimationComponent (anim_startrk, 0, 1, &STAnim1);

	anim_startrkb = CreateAnimation (0);

	static UINT STGrp2[4] = {22,23,24,25};
	static MGROUP_ROTATE STAnim2 (0, STGrp2, 4, _V(1.034, -1.7974, -3.993), _V(0, 0, 1), (float)(110*RAD));
	AddAnimationComponent (anim_startrkb, 0, 1, &STAnim2);


// 6. Neg V2 door

	anim_negv2 = CreateAnimation (0);

	static UINT NegV2Grp1[3] = {32,33,34};
	static MGROUP_ROTATE NegV2Anim1 (0, NegV2Grp1, 3, _V(1.793,1.0296, -3.993), _V(0, 0, 1), (float)(110*RAD));
	AddAnimationComponent (anim_negv2, 0, 1, &NegV2Anim1);

	anim_negv2b = CreateAnimation (0);

	static UINT NegV2Grp2[3] = {35,36,37};
	static MGROUP_ROTATE NegV2Anim2 (0, NegV2Grp2, 3, _V(1.793,-1.0384, -3.993), _V(0, 0, 1), (float)(-110*RAD));
	AddAnimationComponent (anim_negv2b, 0, 1, &NegV2Anim2);


// 7. Pos V2 door

	anim_posv2 = CreateAnimation (0);

	static UINT PosV2Grp1[3] = {41,42,43};
	static MGROUP_ROTATE PosV2Anim1 (0, PosV2Grp1, 3, _V(-1.804, -1.0384, -3.993), _V(0, 0, 1), (float)(110*RAD));
	AddAnimationComponent (anim_posv2, 0, 1, &PosV2Anim1);

	anim_posv2b = CreateAnimation (0);

	static UINT PosV2Grp2[3] = {38,39,40};
	static MGROUP_ROTATE PosV2Anim2 (0, PosV2Grp2, 3, _V(-1.804, 1.0296, -3.993), _V(0, 0, 1), (float)(-110*RAD));
	AddAnimationComponent (anim_posv2b, 0, 1, &PosV2Anim2);



// 8. sa1 port scale
	anim_sa1_port_scale = CreateAnimation (0);

	static UINT SA1PortGrp1[2] = {5,6};
	static MGROUP_SCALE SA1PortAnim1 (12, SA1PortGrp1, 2, _V(0,0,0), _V(1, 0.04, 1));
	AddAnimationComponent (anim_sa1_port_scale, 0, 1, &SA1PortAnim1);

	static UINT SA1PortGrp2[1] = {3};
	static MGROUP_TRANSLATE SA1PortAnim2 (12, SA1PortGrp2, 1, _V(0, 5.88, 0));
	AddAnimationComponent (anim_sa1_port_scale, 0, 1, &SA1PortAnim2);

	static UINT SA1PortGrp3[1] = {4};
	static MGROUP_TRANSLATE SA1PortAnim3 (12, SA1PortGrp3, 1, _V(0, -5.88, 0));
	AddAnimationComponent (anim_sa1_port_scale, 0, 1, &SA1PortAnim3);


	/// elbow
	anim_sa1_port_elb = CreateAnimation (0);
	static UINT SA1PortGrp4[7] = {0,1,2,3,4,5,6};
	static MGROUP_ROTATE SA1PortAnim4 (12, SA1PortGrp4, 7, _V(-1.63, 0, 0), _V(0, 1, 0), (float)(90*RAD));
	AddAnimationComponent (anim_sa1_port_elb, 0, 1, &SA1PortAnim4);


	/// rotation
	anim_sa1_port_rot = CreateAnimation (0);
	static UINT SA1PortGrp5[7] = {0,1,2,3,4,5,6};
	static MGROUP_ROTATE SA1PortAnim5 (12, SA1PortGrp5, 7, _V(-1.63, 0, 0), _V(1, 0, 0), (float)(360*RAD));
	AddAnimationComponent (anim_sa1_port_rot, 0, 1, &SA1PortAnim5);

// 9. sa1 stbd scale
	anim_sa1_stbd_scale = CreateAnimation (0);

	static UINT SA1stbdGrp1[2] = {3,6};
	static MGROUP_SCALE SA1stbdAnim1 (13, SA1stbdGrp1, 2, _V(0,0,0), _V(1, 0.04, 1));
	AddAnimationComponent (anim_sa1_stbd_scale, 0, 1, &SA1stbdAnim1);

	static UINT SA1stbdGrp2[1] = {5};
	static MGROUP_TRANSLATE SA1stbdAnim2 (13, SA1stbdGrp2, 1, _V(0, 5.88, 0));
	AddAnimationComponent (anim_sa1_stbd_scale, 0, 1, &SA1stbdAnim2);

	static UINT SA1stbdGrp3[1] = {4};
	static MGROUP_TRANSLATE SA1stbdAnim3 (13, SA1stbdGrp3, 1, _V(0, -5.88, 0));
	AddAnimationComponent (anim_sa1_stbd_scale, 0, 1, &SA1stbdAnim3);

	/// elbow
	anim_sa1_stbd_elb = CreateAnimation (0);
	static UINT SA1stbdGrp4[7] = {0,1,2,3,4,5,6};
	static MGROUP_ROTATE SA1stbdAnim4 (13, SA1stbdGrp4, 7, _V(1.63, 0, 0), _V(0, -1, 0), (float)(90*RAD));
	AddAnimationComponent (anim_sa1_stbd_elb, 0, 1, &SA1stbdAnim4);

	/// rotation
	anim_sa1_stbd_rot = CreateAnimation (0);
	static UINT SA1stbdGrp5[7] = {0,1,2,3,4,5,6};
	static MGROUP_ROTATE SA1stbdAnim5 (13, SA1stbdGrp5, 7, _V(-1.63, 0, 0), _V(1, 0, 0), (float)(360*RAD));
	AddAnimationComponent (anim_sa1_stbd_rot, 0, 1, &SA1stbdAnim5);


// 10. SA2 PORT
	// rotation
	anim_sa2_port_rot = CreateAnimation (0);
	static UINT SA2PortGrp1[5] = {0,1,2,3,4};
	static MGROUP_ROTATE SA2PortAnim1 (14, SA2PortGrp1, 5, _V(-1.63, 0, 0), _V(1, 0, 0), (float)(360*RAD));
	AddAnimationComponent (anim_sa2_port_rot, 0, 1, &SA2PortAnim1);

	// deploy
	anim_sa2_port_deploy = CreateAnimation (0);
	static UINT SA2PortGrp2[2] = {3,4};
	static MGROUP_ROTATE SA2PortAnim2 (14, SA2PortGrp2, 2, _V(-1.63, 0, 0.037), _V(1, 0, 0), (float)(180*RAD));
	AddAnimationComponent (anim_sa2_port_deploy, 0, 1, &SA2PortAnim2);


// 11. SA2 STBD
	// rotation
	anim_sa2_stbd_rot = CreateAnimation (0);
	static UINT SA2stbdGrp1[5] = {0,1,2,3,4};
	static MGROUP_ROTATE SA2stbdAnim1 (15, SA2stbdGrp1, 5, _V(-1.63, 0, 0), _V(1, 0, 0), (float)(360*RAD));
	AddAnimationComponent (anim_sa2_stbd_rot, 0, 1, &SA2stbdAnim1);

	// deploy
	anim_sa2_stbd_deploy = CreateAnimation (0);
	static UINT SA2stbdGrp2[2] = {2,4};
	static MGROUP_ROTATE SA2stbdAnim2 (15, SA2stbdGrp2, 2, _V(-1.63, 0, 0.037), _V(1, 0, 0), (float)(180*RAD));
	AddAnimationComponent (anim_sa2_stbd_deploy, 0, 1, &SA2stbdAnim2);

}


void hst_ex::clbkSaveState (FILEHANDLE scn)
{
	char cbuf[256];

	for (int i = 0; i < (int)strlen(refname); i++)
	{
		if (refname[i] == ' ')
		{
			refname[i] = '+';
			break;
		}
	}

	// default vessel parameters
	VESSEL3::clbkSaveState(scn);

	// custom parameters
	sprintf (cbuf, "%d %0.4f", sunshield_status, sunsh_proc);
	oapiWriteScenario_string (scn, "SNSH", cbuf);
	sprintf (cbuf, "%d %d %d %0.4f %0.4f %0.4f", sa_port_status, sa1_port_elb_sts, sa1_port_scale_sts, sa1_port_elb_proc, sa1_port_rot_proc, sa1_port_scale_proc);
	oapiWriteScenario_string (scn, "SP1P", cbuf);
	sprintf (cbuf, "%d %d %d %0.4f %0.4f %0.4f", sa_stbd_status, sa1_stbd_elb_sts, sa1_stbd_scale_sts, sa1_stbd_elb_proc, sa1_stbd_rot_proc, sa1_stbd_scale_proc);
	oapiWriteScenario_string (scn, "SP1S", cbuf);
	sprintf (cbuf, "%d %d %d", hst_cfg, sa_port_cfg, sa_stbd_cfg);
	oapiWriteScenario_string (scn, "SAC", cbuf);
	sprintf (cbuf, "%d %0.4f", hga_status, hga_deploy_proc);
	oapiWriteScenario_string (scn, "HGAD", cbuf);
	sprintf (cbuf, "%0.4f %0.4f %0.4f %0.4f", hga_posy_az_proc, hga_posy_alt_proc, hga_negy_az_proc, hga_negy_alt_proc);
	oapiWriteScenario_string (scn, "HGAR", cbuf);
	sprintf (cbuf, "%d %0.4f %d %0.4f", door_status[2], fgs_proc, door_status[3], fgsb_proc);
	oapiWriteScenario_string (scn, "FG2", cbuf);
	sprintf (cbuf, "%d %0.4f %d %0.4f", door_status[8], fgs1_proc, door_status[9], fgs1b_proc);
	oapiWriteScenario_string (scn, "FG1", cbuf);
	sprintf (cbuf, "%d %0.4f %d %0.4f", door_status[0], startrk_proc, door_status[1], startrkb_proc);
	oapiWriteScenario_string (scn, "STR", cbuf);
	sprintf (cbuf, "%d %0.4f %d %0.4f", door_status[4], posv2_proc, door_status[5], posv2b_proc);
	oapiWriteScenario_string (scn, "PV2", cbuf);
	sprintf (cbuf, "%d %0.4f %d %0.4f", door_status[6], negv2_proc, door_status[7], negv2b_proc);
	oapiWriteScenario_string (scn, "NV2", cbuf);

	sprintf (cbuf, "%s %s %s %s", inst1_name, inst2_name, inst3_name, inst4_name);
	oapiWriteScenario_string (scn, "INM", cbuf);

	sprintf (cbuf, "%s", refname);
	oapiWriteScenario_string (scn, "TGN", cbuf);
	sprintf (cbuf, "%d %d", ap_status, autosun_status);
	oapiWriteScenario_string (scn, "AUTP", cbuf);
	sprintf (cbuf, "%d %d %d %d %d %d %d", wfc_status, inst1_status, inst2_status, inst3_status, inst4_status, fgs1_status, fgs2_status);
	oapiWriteScenario_string (scn, "MSHS", cbuf);
	sprintf (cbuf, "%d %d %d", rsu1_status, rsu2_status, rsu3_status);
	oapiWriteScenario_string (scn, "MMSH", cbuf);
	sprintf (cbuf, "%0.6f %0.6f %d %d %d %d %d %d", tgt_ra_rad, tgt_dec_rad, tgt_ra_hh, tgt_ra_mm, tgt_ra_ss, tgt_dec_hh, tgt_dec_mm, tgt_dec_ss);
	oapiWriteScenario_string (scn, "RADC", cbuf);

}
void hst_ex::clbkLoadStateEx (FILEHANDLE scn, void *vs)
{
    char *line;
	while (oapiReadScenario_nextline (scn, line)) {
		if (!strnicmp (line, "SNSH", 4)) {
			sscanf (line+4, "%d%lf", &sunshield_status, &sunsh_proc);
		} else if (!strnicmp (line, "SP1P", 4)) {
			sscanf (line+4, "%d%d%d%lf%lf%lf", &sa_port_status, &sa1_port_elb_sts, &sa1_port_scale_sts, &sa1_port_elb_proc, &sa1_port_rot_proc, &sa1_port_scale_proc);
		} else if (!strnicmp (line, "SP1S", 4)) {
			sscanf (line+4, "%d%d%d%lf%lf%lf", &sa_stbd_status, &sa1_stbd_elb_sts, &sa1_stbd_scale_sts, &sa1_stbd_elb_proc, &sa1_stbd_rot_proc, &sa1_stbd_scale_proc);
		} else if (!strnicmp (line, "SAC", 3)) {
			sscanf (line+3, "%d%d%d", &hst_cfg, &sa_port_cfg, &sa_stbd_cfg);
		} else if (!strnicmp (line, "HGAD", 4)) {
			sscanf (line+4, "%d%lf", &hga_status, &hga_deploy_proc);
		} else if (!strnicmp (line, "HGAR", 4)) {
			sscanf (line+4, "%lf%lf%lf%lf", &hga_posy_az_proc, &hga_posy_alt_proc, &hga_negy_az_proc, &hga_negy_alt_proc);
		} else if (!strnicmp (line, "FG2", 3)) {
			sscanf (line+3, "%d%lf%d%lf", &door_status[2], &fgs_proc, &door_status[3], &fgsb_proc);
		} else if (!strnicmp (line, "FG1", 3)) {
			sscanf (line+3, "%d%lf%d%lf", &door_status[8], &fgs1_proc, &door_status[9], &fgs1b_proc);
		} else if (!strnicmp (line, "STR", 3)) {
			sscanf (line+3, "%d%lf%d%lf", &door_status[0], &startrk_proc, &door_status[1], &startrkb_proc);
		} else if (!strnicmp (line, "PV2", 3)) {
			sscanf (line+3, "%d%lf%d%lf", &door_status[4], &posv2_proc, &door_status[5], &posv2b_proc);
		} else if (!strnicmp (line, "NV2", 3)) {
			sscanf (line+3, "%d%lf%d%lf", &door_status[6], &negv2_proc, &door_status[7], &negv2b_proc);
		} else if (!strnicmp (line, "TGN", 3)) {
			sscanf (line+3, "%s", &refname);
		} else if (!strnicmp (line, "INM", 3)) {
			sscanf (line+3, "%s%s%s%s", &inst1_name, &inst2_name, &inst3_name, &inst4_name);
		} else if (!strnicmp (line, "AUTP", 4)) {
			sscanf (line+4, "%d%d", &ap_status, &autosun_status);
		} else if (!strnicmp (line, "MSHS", 4)) {
			sscanf (line+4, "%d%d%d%d%d%d%d", &wfc_status, &inst1_status, &inst2_status, &inst3_status, &inst4_status, &fgs1_status, &fgs2_status);
		} else if (!strnicmp (line, "MMSH", 4)) {
			sscanf (line+4, "%d%d%d", &rsu1_status, &rsu2_status, &rsu3_status);
		} else if (!strnicmp (line, "RADC", 4)) {
			sscanf (line+4, "%lf%lf%d%d%d%d%d%d", &tgt_ra_rad, &tgt_dec_rad, &tgt_ra_hh, &tgt_ra_mm, &tgt_ra_ss, &tgt_dec_hh, &tgt_dec_mm, &tgt_dec_ss);
		}else {
            ParseScenarioLineEx (line, vs);
			// unrecognised option - pass to Orbiter's generic parser
        }
   }
	SetAnimation (anim_sunsh, sunsh_proc);
	SetAnimation (anim_hga_deploy, hga_deploy_proc);

	SetAnimation (anim_fgs, fgs_proc);
	SetAnimation (anim_fgs1, fgs1_proc);
	SetAnimation (anim_startrk, startrk_proc);
	SetAnimation (anim_posv2, posv2_proc);
	SetAnimation (anim_negv2, negv2_proc);

	SetAnimation (anim_fgsb, fgsb_proc);
	SetAnimation (anim_fgs1b, fgs1b_proc);
	SetAnimation (anim_startrkb, startrkb_proc);
	SetAnimation (anim_posv2b, posv2b_proc);
	SetAnimation (anim_negv2b, negv2b_proc);

	SetAnimation (anim_hga_posy_az, hga_posy_az_proc);
	SetAnimation (anim_hga_posy_alt, hga_posy_alt_proc);
	SetAnimation (anim_hga_negy_az, hga_negy_az_proc);
	SetAnimation (anim_hga_negy_alt, hga_negy_alt_proc);

	SetAnimation (anim_sa1_port_scale, sa1_port_scale_proc);
	SetAnimation (anim_sa1_port_elb, sa1_port_elb_proc);
	SetAnimation (anim_sa1_port_rot, sa1_port_rot_proc);

	SetAnimation (anim_sa1_stbd_scale, sa1_stbd_scale_proc);
	SetAnimation (anim_sa1_stbd_elb, sa1_stbd_elb_proc);
	SetAnimation (anim_sa1_stbd_rot, sa1_stbd_rot_proc);

	if (tgt_ra_rad == 0 && tgt_dec_rad == 0)tgt_dec_rad = 0.0000001;


	for (int i = 0; i < (int)strlen(refname); i++)
	{
		if (refname[i] == '+')
		{
			refname[i] = ' ';
			break;
		}
	}
}



int hst_ex::clbkConsumeBufferedKey(DWORD key, bool down, char *kstate) {

	bool cbEnterTarget(void *tid, char *tstr, void *tdata);
	bool cbEnterRA(void *id1, char *str1, void *data1);
	bool cbEnterDec(void *id2, char *str2, void *data2);


  if (!down) return 0; // only process keydown events

  if (KEYMOD_SHIFT (kstate)) 
  {	 
	switch (key)
	{
		case OAPI_KEY_B:
		{
			oapiOpenInputBox("Enter RA hhmmss", cbEnterRA, 0, 30, (void *)this);
		}
		return true;

		case OAPI_KEY_G: // increase fov
		icr_fov = 1;
		return 1;

	} 
  }


  else if (KEYMOD_CONTROL (kstate)) 
  { 
	switch (key)
	{
		case OAPI_KEY_B:
		{
			oapiOpenInputBox("Enter Dec ddmmss", cbEnterDec, 0, 30, (void *)this);
		}
		return true;
	} 
  }

	else { // unmodified keys
    switch (key) {
	case OAPI_KEY_K: // deploy sunshield and HGA
		if (sunshield_status == 0)sunshield_status = 1;
		else if (sunshield_status == 2)sunshield_status = 3;

		if (hga_status == 0)hga_status = 1;
		else if (hga_status == 2)hga_status = 3;
		return 1;


	case OAPI_KEY_J: // doors

		if (door_status[2] == 0)door_status[2] = 1;
		else if (door_status[2] == 2)door_status[2] = 3;

		if (door_status[0] == 0)door_status[0] = 1;
		else if (door_status[0] == 2)door_status[0] = 3;

		if (door_status[4] == 0)door_status[4] = 1;
		else if (door_status[4] == 2)door_status[4] = 3;

		if (door_status[6] == 0)door_status[6] = 1;
		else if (door_status[6] == 2)door_status[6] = 3;

		if (door_status[8] == 0)door_status[8] = 1;
		else if (door_status[8] == 2)door_status[8] = 3;

		if (door_status[9] == 0)door_status[9] = 1;
		else if (door_status[9] == 2)door_status[9] = 3;

		if (door_status[3] == 0)door_status[3] = 1;
		else if (door_status[3] == 2)door_status[3] = 3;

		if (door_status[1] == 0)door_status[1] = 1;
		else if (door_status[1] == 2)door_status[1] = 3;

		if (door_status[5] == 0)door_status[5] = 1;
		else if (door_status[5] == 2)door_status[5] = 3;

		if (door_status[7] == 0)door_status[7] = 1;
		else if (door_status[7] == 2)door_status[7] = 3;


		return 1;

	case OAPI_KEY_NUMPAD5: // do killrot
		if (ap_status == 0)ap_status = 1;
		else if (ap_status == 1)ap_status = 0;
		else if (ap_status == 2)ap_status = 1;
		else if (ap_status == 3)ap_status = 1;
		return 1;

	case OAPI_KEY_V: // goto target
		if (ap_status == 0)ap_status = 2;
		else if (ap_status == 1)ap_status = 2;
		else if (ap_status == 2)ap_status = 0;
		else if (ap_status == 3)ap_status = 2;
		return 1;

	case OAPI_KEY_B: // goto RA Dec
		if (ap_status == 0)ap_status = 3;
		else if (ap_status == 1)ap_status = 3;
		else if (ap_status == 2)ap_status = 3;
		else if (ap_status == 3)ap_status = 0;
		return 1;

	case OAPI_KEY_G: // decrease fov
		dcr_fov = 1;
		return 1;

		//////

	case OAPI_KEY_N: // autosun
		if ((sa1_stbd_elb_sts + sa1_stbd_scale_sts + sa1_port_elb_sts + sa1_port_scale_sts) == 0)
		{
		if (autosun_status == 0)autosun_status = 1;
		else if(autosun_status == 1)autosun_status = 0;
		else if(autosun_status == 2)autosun_status = 1;
		}
		return 1;

	case OAPI_KEY_M: // stow panels
		if ((sa1_port_elb_sts + sa1_stbd_elb_sts) == 0)
		{
		if (autosun_status == 0)autosun_status = 2;
		else if(autosun_status == 1)autosun_status = 2;
		else if(autosun_status == 2)autosun_status = 0;
		}
		return 1;

	case OAPI_KEY_F5: // unlatch solar panels 
		if (sa1_port_elb_sts == 2)sa1_port_elb_sts = 3;
		if (sa1_stbd_elb_sts == 2)sa1_stbd_elb_sts = 3;
		return 1;

	case OAPI_KEY_F6: // open stbd panels
		if ((sa1_stbd_elb_sts + autosun_status + sa1_stbd_rot_proc + sa_stbd_status) == 0 )
		{
		//if (sa1_stbd_scale_sts == 0 )sa1_stbd_scale_sts = 1;
		//else if (sa1_stbd_scale_sts == 2)sa1_stbd_scale_sts = 3;
			if (sa1_stbd_scale_sts == 2)sa1_stbd_scale_sts = 3;
		}
		return 1;

	case OAPI_KEY_F7: // open port panels
		if ((sa1_port_elb_sts + autosun_status + sa1_port_rot_proc + sa_port_status) == 0 )
		{
		//if (sa1_port_scale_sts == 0)sa1_port_scale_sts = 1;
		//else if (sa1_port_scale_sts == 2)sa1_port_scale_sts = 3;
			if (sa1_port_scale_sts == 2)sa1_port_scale_sts = 3;
		}
		return 1;

	case OAPI_KEY_E:
		{
			oapiOpenInputBox("Enter Target Name", cbEnterTarget, 0, 30, (void *)this);
		}
		return true;


	}
  }
  return 0;
}



bool cbEnterRA(void *id1, char *str1, void *data1)
{
	return (((hst_ex *)data1)->EnterRA(str1));
}

bool hst_ex::EnterRA(char *Name1)
{
	if (strlen(Name1) == 6)
	{
	sscanf(Name1,"%lf", &tgt_ra);

	tgt_ra_int = (int) (tgt_ra);
	if (tgt_ra_int == tgt_ra)
		{
		int tgt_ra_hh_chk = (int) (tgt_ra*0.0001);
		int tgt_ra_mm_chk = (int) ((tgt_ra*0.01)-(tgt_ra_hh_chk*100));
		int tgt_ra_ss_chk = (int) (tgt_ra - ((tgt_ra_mm_chk*100) + (tgt_ra_hh_chk*10000)));

		if (tgt_ra_hh_chk > 23) return false;
		else if (tgt_ra_hh_chk < 0) return false;

		if (tgt_ra_mm_chk > 59) return false;
		else if (tgt_ra_mm_chk < 0) return false;

		if (tgt_ra_ss_chk > 59) return false;
		else if (tgt_ra_ss_chk < 0) return false;

		tgt_ra_hh = tgt_ra_hh_chk;
		tgt_ra_mm = tgt_ra_mm_chk;
		tgt_ra_ss = tgt_ra_ss_chk;

		tgt_ra_rad = RAD*( (15*tgt_ra_hh) + (0.25*tgt_ra_mm) + (0.004166667*tgt_ra_ss) );
		if (tgt_ra_rad == 0 && tgt_dec_rad == 0)tgt_dec_rad = 0.0000001;

		return true;
		}
	
	else return false;
	}

	return false;
}



bool cbEnterDec(void *id2, char *str2, void *data2)
{
	return (((hst_ex *)data2)->EnterDec(str2));
}

bool hst_ex::EnterDec(char *Name2)
{
	if (strlen(Name2) > 5 && strlen(Name2) < 8)
	{
	sscanf(Name2,"%lf", &tgt_dec);

	tgt_dec_int = (int) (tgt_dec);
	if (tgt_dec_int == tgt_dec)
		{
		int tgt_dec_hh_chk = (int) (tgt_dec*0.0001);
		int tgt_dec_mm_chk = (int) ((tgt_dec*0.01)-(tgt_dec_hh_chk*100));
		int tgt_dec_ss_chk = (int) (tgt_dec - ((tgt_dec_mm_chk*100) + (tgt_dec_hh_chk*10000)));

		if (tgt_dec_hh_chk > 90) return false;
		else if (tgt_dec_hh_chk < -90) return false;

		if (tgt_dec_mm_chk > 59) return false;
		else if (tgt_dec_mm_chk < -59) return false;

		if (tgt_dec_ss_chk > 59) return false;
		else if (tgt_dec_ss_chk < -59) return false;

		tgt_dec_hh = tgt_dec_hh_chk;
		tgt_dec_mm = tgt_dec_mm_chk;
		tgt_dec_ss = tgt_dec_ss_chk;

		tgt_dec_rad = RAD*( (tgt_dec_hh) + (0.01666667*tgt_dec_mm) + (0.0002777778*tgt_dec_ss) );
		if (tgt_dec_rad == 0 && tgt_dec_rad == 0)tgt_dec_rad = 0.0000001;

		return true;
		}
	
	else return false;
	}

	return false;
}




bool cbEnterTarget(void *tid, char *tstr, void *tdata)
{
	return (((hst_ex *)tdata)->EnterTarget(tstr));
}

bool hst_ex::EnterTarget(char *tname)
{
OBJHANDLE MyObj = oapiGetObjectByName(tname);

	if (MyObj)
	{
	strcpy(refname,tname);
	return true;
	}

	return false;

}


int hst_ex::clbkConsumeDirectKey(char *kstate)
{
		VECTOR3 avel = _V(0,0,0);
		GetAngularVel(avel);
		double dt = oapiGetSimStep();
		double drot = RAD*0.1*dt;

if (ap_status == 0)
{
if (KEYDOWN (kstate, OAPI_KEY_NUMPAD2))
		{
		if (avel.x < RAD)
			{
			avel.x = avel.x+drot;
			if (avel.x > RAD) avel.x = RAD;
			}
		}

if (KEYDOWN (kstate, OAPI_KEY_NUMPAD8))
		{
		if (avel.x > -RAD)
			{
			avel.x = avel.x-drot;
			if (avel.x < -RAD) avel.x = -RAD;
			}
		}

if (KEYDOWN (kstate, OAPI_KEY_NUMPAD1))
		{
		if (avel.y < RAD)
			{
			avel.y = avel.y+drot;
			if (avel.y > RAD) avel.y = RAD;
			}
		}

if (KEYDOWN (kstate, OAPI_KEY_NUMPAD3))
		{
		if (avel.y > -RAD)
			{
			avel.y = avel.y-drot;
			if (avel.y < -RAD) avel.y = -RAD;
			}
		}

if (KEYDOWN (kstate, OAPI_KEY_NUMPAD6))
		{
		if (avel.z < RAD)
			{			
			avel.z = avel.z+drot;
			if (avel.z > RAD) avel.z = RAD;
			}
		}

if (KEYDOWN (kstate, OAPI_KEY_NUMPAD4))
		{
		if (avel.z > -RAD)
			{
			avel.z = avel.z-drot;
			if (avel.z < -RAD) avel.z = -RAD;
			}		
		}

		SetAngularVel(avel);
}

return 0;
}

void hst_ex::DecreaseFOV(void)
{
if (oapiCameraInternal() == true)
	{
	double fov = oapiCameraAperture ();
	fov = fov*0.75;
	oapiCameraSetAperture(fov);
	oapiCameraAttach(GetHandle(), 0);
	}

}

void hst_ex::IncreaseFOV(void)
{
if (oapiCameraInternal() == true)
	{
	double fov = oapiCameraAperture ();
	fov = fov*1.5;
	if (fov > (45*RAD))fov = (45*RAD);
	oapiCameraSetAperture(fov);
	oapiCameraAttach(GetHandle(), 0);
	}

}

void hst_ex::clbkPostCreation()
{
if (wfc_status > 0)SetMeshVisibilityMode(mesh2,MESHVIS_NEVER);
if (inst1_status > 0)SetMeshVisibilityMode(mesh3,MESHVIS_NEVER);
if (inst2_status > 0)SetMeshVisibilityMode(mesh4,MESHVIS_NEVER);
if (inst3_status > 0)SetMeshVisibilityMode(mesh5,MESHVIS_NEVER);
if (inst4_status > 0)SetMeshVisibilityMode(mesh6,MESHVIS_NEVER);
if (fgs1_status > 0)SetMeshVisibilityMode(mesh7,MESHVIS_NEVER);
if (fgs2_status > 0)SetMeshVisibilityMode(mesh8,MESHVIS_NEVER);

if (rsu1_status > 0)SetMeshVisibilityMode(mesh10,MESHVIS_NEVER);
if (rsu2_status > 0)SetMeshVisibilityMode(mesh11,MESHVIS_NEVER);
if (rsu3_status > 0)SetMeshVisibilityMode(mesh12,MESHVIS_NEVER);

if (sa_stbd_status > 0)
{
SetMeshVisibilityMode(mesh14,MESHVIS_NEVER);
SetMeshVisibilityMode(mesh16,MESHVIS_NEVER);
}
if (sa_port_status > 0)
{
SetMeshVisibilityMode(mesh13,MESHVIS_NEVER);
SetMeshVisibilityMode(mesh15,MESHVIS_NEVER);
}

if (sa_stbd_cfg == 0)SetMeshVisibilityMode(mesh16,MESHVIS_NEVER);
else SetMeshVisibilityMode(mesh14,MESHVIS_NEVER);

if (sa_port_cfg == 0)SetMeshVisibilityMode(mesh15,MESHVIS_NEVER);
else 
{
SetMeshVisibilityMode(mesh13,MESHVIS_NEVER);
hst_cfg = 1;
}

}

void hst_ex::clbkPreStep(double SimT, double SimDT, double MJD)
{

if (oapiGetShowGrapplePoints () == true)
{
DoBeacons();
dobeacon_st = 1;
}

else if(dobeacon_st == 1)
{
dobeacon_st = 0;
for (int i = 0; i < 27; i++) 
	{
	beacon[i].active = false;
	}
}
/////////////

if(anot_status > 0)
{

if (anot_status == 1)
	{
char imstr[256];
strcpy(imstr, "RELEASED");
VECTOR3 col = _V(0.8,0.7,0.1);
hNote = oapiCreateAnnotation(true, 1, col);
oapiAnnotationSetPos(hNote, 0.468, 0.0, 0.66, 0.05);
oapiAnnotationSetText(hNote, imstr);
anot_status = 2;
	}

if(anot_status == 2)
{
anot_timer = anot_timer + oapiGetSysStep();
if (anot_timer > 2)
	{
	anot_status = 3;
	anot_timer = 0;
	}

}

if(anot_status == 3)
	{
	anot_status = 0;
	oapiDelAnnotation (hNote);
	}


}
///////////

if(anot2_status > 0)
{

if (anot2_status == 1)
	{
char imstr[256];
strcpy(imstr, "SECURED");
VECTOR3 col = _V(0.8,0.7,0.1);
hNote2 = oapiCreateAnnotation(true, 1, col);
oapiAnnotationSetPos(hNote2, 0.468, 0.0, 0.66, 0.05);
oapiAnnotationSetText(hNote2, imstr);
anot2_status = 2;
	}

if(anot2_status == 2)
{
anot2_timer = anot2_timer + oapiGetSysStep();
if (anot2_timer > 2)
	{
	anot2_status = 3;
	anot2_timer = 0;
	}

}

if(anot2_status == 3)
	{
	anot2_status = 0;
	oapiDelAnnotation (hNote2);
	}


}



//////}


//////void hst_ex::clbkPostStep (double simt, double simdt, double mjd)
//////{
step_time = SimDT;///////
double simdt = SimDT;

if (hga_status == 2) SetHGA();
if (hga_status == 3) StowHGA();

if (ap_status == 1) DoKillRot();
else if (ap_status == 2) GoToTarget();
else if (ap_status == 3) GoToRADec();

if (dcr_fov == 1)
{
DecreaseFOV();
dcr_fov = 0;
}

if (icr_fov == 1)
{
IncreaseFOV();
icr_fov = 0;
}

if (cv1cv2_check < 2)CheckVessels();
else if (cv1cv2_check > 1)CheckVessels2();


if(att_pc_check == 0)att_pc_check = 1;
else if(att_pc_check == 1)att_pc_check = 0;

if (autosun_status == 1) DoAutoSun();
else if (autosun_status == 2) DoRevertRot();

if (sa_stbd_cfg == 0) SetAnimation (anim_sa1_stbd_rot, sa1_stbd_rot_proc);
else SetAnimation (anim_sa2_stbd_rot, sa1_stbd_rot_proc);
if (sa_port_cfg == 0) SetAnimation (anim_sa1_port_rot, sa1_port_rot_proc);
else SetAnimation (anim_sa2_port_rot, sa1_port_rot_proc);

//sprintf(oapiDebugString(), "%d %0.4f %0.4f", (sa1_stbd_elb_sts + sa1_stbd_scale_sts + sa1_port_elb_sts + sa1_port_scale_sts),sa1_port_rot_proc,sa1_stbd_rot_proc);

////////////////// ANIMATIONS ///////////////
//// sunshield
if (sunshield_status > 0)
{
double da = simdt * sunshield_dur;
if (sunshield_status == 1) sunsh_proc = sunsh_proc + da;
else if (sunshield_status == 3) sunsh_proc = sunsh_proc - da;

if (sunsh_proc > 1)
	{
	if (sunshield_status == 1) sunshield_status = 2;
	sunsh_proc = 1;
	}

else if (sunsh_proc < 0)
	{
	if (sunshield_status == 3) sunshield_status = 0;
	sunsh_proc = 0;
	}
SetAnimation (anim_sunsh, sunsh_proc);
}



//// HGA deploy
if (hga_status > 0)
{
double da = simdt * hga_dur;
if (hga_status == 1) hga_deploy_proc = hga_deploy_proc + da;
else if (hga_status == 3) hga_deploy_proc = hga_deploy_proc - da;

if (hga_deploy_proc > 1)
	{
	if (hga_status == 1) hga_status = 2;
	hga_deploy_proc = 1;
	}

else if (hga_deploy_proc < 0)
	{
	if (hga_status == 3) hga_status = 0;
	hga_deploy_proc = 0;
	}
SetAnimation (anim_hga_deploy, hga_deploy_proc);
}


//// FGS doors a
if (door_status[2] > 0)
{
double da = simdt * fgs_door_dur;
if (door_status[2] == 1) fgs_proc = fgs_proc + da;
else if (door_status[2] == 3) fgs_proc = fgs_proc - da;

if (fgs_proc > 1)
	{
	if (door_status[2] == 1) door_status[2] = 2;
	fgs_proc = 1;
	}

else if (fgs_proc < 0)
	{
	if (door_status[2] == 3) door_status[2] = 0;
	fgs_proc = 0;
	}
SetAnimation (anim_fgs, fgs_proc);
}


//// FGS1 doors a1
if (door_status[8] > 0)
{
double da = simdt * fgs_door_dur;
if (door_status[8] == 1) fgs1_proc = fgs1_proc + da;
else if (door_status[8] == 3) fgs1_proc = fgs1_proc - da;

if (fgs1_proc > 1)
	{
	if (door_status[8] == 1) door_status[8] = 2;
	fgs1_proc = 1;
	}

else if (fgs1_proc < 0)
	{
	if (door_status[8] == 3) door_status[8] = 0;
	fgs1_proc = 0;
	}
SetAnimation (anim_fgs1, fgs1_proc);
}


//// FGS doors b
if (door_status[3] > 0)
{
double da = simdt * fgs_door_dur;
if (door_status[3] == 1) fgsb_proc = fgsb_proc + da;
else if (door_status[3] == 3) fgsb_proc = fgsb_proc - da;

if (fgsb_proc > 1)
	{
	if (door_status[3] == 1) door_status[3] = 2;
	fgsb_proc = 1;
	}

else if (fgsb_proc < 0)
	{
	if (door_status[3] == 3) door_status[3] = 0;
	fgsb_proc = 0;
	}
SetAnimation (anim_fgsb, fgsb_proc);
}


//// FGS1 doors 1b
if (door_status[9] > 0)
{
double da = simdt * fgs_door_dur;
if (door_status[9] == 1) fgs1b_proc = fgs1b_proc + da;
else if (door_status[9] == 3) fgs1b_proc = fgs1b_proc - da;

if (fgs1b_proc > 1)
	{
	if (door_status[9] == 1) door_status[9] = 2;
	fgs1b_proc = 1;
	}

else if (fgs1b_proc < 0)
	{
	if (door_status[9] == 3) door_status[9] = 0;
	fgs1b_proc = 0;
	}
SetAnimation (anim_fgs1b, fgs1b_proc);
}






//// star tracker doors a
if (door_status[0] > 0)
{
double da = simdt * startrk_door_dur;
if (door_status[0] == 1) startrk_proc = startrk_proc + da;
else if (door_status[0] == 3) startrk_proc = startrk_proc - da;

if (startrk_proc > 1)
	{
	if (door_status[0] == 1) door_status[0] = 2;
	startrk_proc = 1;
	}

else if (startrk_proc < 0)
	{
	if (door_status[0] == 3) door_status[0] = 0;
	startrk_proc = 0;
	}
SetAnimation (anim_startrk, startrk_proc);
}


//// star tracker doors b
if (door_status[1] > 0)
{
double da = simdt * startrk_door_dur;
if (door_status[1] == 1) startrkb_proc = startrkb_proc + da;
else if (door_status[1] == 3) startrkb_proc = startrkb_proc - da;

if (startrkb_proc > 1)
	{
	if (door_status[1] == 1) door_status[1] = 2;
	startrkb_proc = 1;
	}

else if (startrkb_proc < 0)
	{
	if (door_status[1] == 3) door_status[1] = 0;
	startrkb_proc = 0;
	}
SetAnimation (anim_startrkb, startrkb_proc);
}




//// Pos V2 doors a
if (door_status[4] > 0)
{
double da = simdt * pos_v2_door_dur;
if (door_status[4] == 1) posv2_proc = posv2_proc + da;
else if (door_status[4] == 3) posv2_proc = posv2_proc - da;

if (posv2_proc > 1)
	{
	if (door_status[4] == 1) door_status[4] = 2;
	posv2_proc = 1;
	}

else if (posv2_proc < 0)
	{
	if (door_status[4] == 3) door_status[4] = 0;
	posv2_proc = 0;
	}
SetAnimation (anim_posv2, posv2_proc);
}


//// Pos V2 doors b
if (door_status[5] > 0)
{
double da = simdt * pos_v2_door_dur;
if (door_status[5] == 1) posv2b_proc = posv2b_proc + da;
else if (door_status[5] == 3) posv2b_proc = posv2b_proc - da;

if (posv2b_proc > 1)
	{
	if (door_status[5] == 1) door_status[5] = 2;
	posv2b_proc = 1;
	}

else if (posv2b_proc < 0)
	{
	if (door_status[5] == 3) door_status[5] = 0;
	posv2b_proc = 0;
	}
SetAnimation (anim_posv2b, posv2b_proc);
}




//// Neg V2 doors a
if (door_status[6] > 0)
{
double da = simdt * neg_v2_door_dur;
if (door_status[6] == 1) negv2_proc = negv2_proc + da;
else if (door_status[6] == 3) negv2_proc = negv2_proc - da;

if (negv2_proc > 1)
	{
	if (door_status[6] == 1) door_status[6] = 2;
	negv2_proc = 1;
	}

else if (negv2_proc < 0)
	{
	if (door_status[6] == 3) door_status[6] = 0;
	negv2_proc = 0;
	}
SetAnimation (anim_negv2, negv2_proc);
}


//// Neg V2 doors b
if (door_status[7] > 0)
{
double da = simdt * neg_v2_door_dur;
if (door_status[7] == 1) negv2b_proc = negv2b_proc + da;
else if (door_status[7] == 3) negv2b_proc = negv2b_proc - da;

if (negv2b_proc > 1)
	{
	if (door_status[7] == 1) door_status[7] = 2;
	negv2b_proc = 1;
	}

else if (negv2b_proc < 0)
	{
	if (door_status[7] == 3) door_status[7] = 0;
	negv2b_proc = 0;
	}
SetAnimation (anim_negv2b, negv2b_proc);
}

//// sa1 port scale
if (sa1_port_scale_sts > 0)
{
if (sa_port_cfg == 1)
{
if (sa1_port_scale_sts == 1) sa1_port_scale_sts = 3;
else if (sa1_port_scale_sts == 2) sa1_port_scale_sts = 3;
}

double da = simdt * sunshield_dur;
if (sa1_port_scale_sts == 1) sa1_port_scale_proc = sa1_port_scale_proc + da;
else if (sa1_port_scale_sts == 3) sa1_port_scale_proc = sa1_port_scale_proc - da;

double sa1_stop = 1;
if (hst_cfg == 0) sa1_stop = 0.5;

if (sa1_port_scale_proc > sa1_stop && sa1_port_scale_sts < 2)
	{
	if (sa1_port_scale_sts == 1) sa1_port_scale_sts = 2;
	sa1_port_scale_proc = sa1_stop;
	}

else if (sa1_port_scale_proc < 0)
	{
	if (sa1_port_scale_sts == 3) sa1_port_scale_sts = 0;
	sa1_port_scale_proc = 0;
	}
if (sa_port_cfg == 0)SetAnimation (anim_sa1_port_scale, sa1_port_scale_proc);
else SetAnimation (anim_sa2_port_deploy, sa1_port_scale_proc);
}

//// sa1 port elbow
if (sa1_port_elb_sts > 0)
{
double da = simdt * neg_v2_door_dur;
if (sa1_port_elb_sts == 1) sa1_port_elb_proc = sa1_port_elb_proc + da;
else if (sa1_port_elb_sts == 3) sa1_port_elb_proc = sa1_port_elb_proc - da;

if (sa1_port_elb_proc > 1)
	{
	if (sa1_port_elb_sts == 1) sa1_port_elb_sts = 2;
	sa1_port_elb_proc = 1;
	}

else if (sa1_port_elb_proc < 0)
	{
	if (sa1_port_elb_sts == 3) sa1_port_elb_sts = 0;
	sa1_port_elb_proc = 0;
	}
SetAnimation (anim_sa1_port_elb, sa1_port_elb_proc);
}

//// sa1 stbd scale
if (sa1_stbd_scale_sts > 0)
{
if (sa_stbd_cfg == 1)
{
if (sa1_stbd_scale_sts == 1) sa1_stbd_scale_sts = 3;
else if (sa1_stbd_scale_sts == 2) sa1_stbd_scale_sts = 3;
}

double da = simdt * sunshield_dur;
if (sa1_stbd_scale_sts == 1) sa1_stbd_scale_proc = sa1_stbd_scale_proc + da;
else if (sa1_stbd_scale_sts == 3) sa1_stbd_scale_proc = sa1_stbd_scale_proc - da;

if (sa1_stbd_scale_proc > 1)
	{
	if (sa1_stbd_scale_sts == 1) sa1_stbd_scale_sts = 2;
	sa1_stbd_scale_proc = 1;
	}

else if (sa1_stbd_scale_proc < 0)
	{
	if (sa1_stbd_scale_sts == 3) sa1_stbd_scale_sts = 0;
	sa1_stbd_scale_proc = 0;
	}
if (sa_stbd_cfg == 0) SetAnimation (anim_sa1_stbd_scale, sa1_stbd_scale_proc);
else SetAnimation (anim_sa2_stbd_deploy, sa1_stbd_scale_proc);
}


//// sa1 stbd elbow
if (sa1_stbd_elb_sts > 0)
{
double da = simdt * neg_v2_door_dur;
if (sa1_stbd_elb_sts == 1) sa1_stbd_elb_proc = sa1_stbd_elb_proc + da;
else if (sa1_stbd_elb_sts == 3) sa1_stbd_elb_proc = sa1_stbd_elb_proc - da;

if (sa1_stbd_elb_proc > 1)
	{
	if (sa1_stbd_elb_sts == 1) sa1_stbd_elb_sts = 2;
	sa1_stbd_elb_proc = 1;
	}

else if (sa1_stbd_elb_proc < 0)
	{
	if (sa1_stbd_elb_sts == 3) sa1_stbd_elb_sts = 0;
	sa1_stbd_elb_proc = 0;
	}
SetAnimation (anim_sa1_stbd_elb, sa1_stbd_elb_proc);
}




}


void hst_ex::SetHGA (void)
{
	char cbuf[7]; 
	strcpy(cbuf,"Earth");
	OBJHANDLE hEarth = oapiGetObjectByName(cbuf);

	double earth_obl = oapiGetPlanetObliquity (hEarth);
	//sprintf(oapiDebugString(), "earth_obl %0.5f", DEG*earth_obl);
	
	VECTOR3 earthglobpos = _V(0,0,0);
	oapiGetGlobalPos(hEarth, &earthglobpos);
	VECTOR3 earthlocalpos = _V(0,0,0);
	Global2Local(earthglobpos, earthlocalpos);
	double earth_az = atan2(-earthlocalpos.x, -earthlocalpos.z); // 0deg along -z axis
	if (earth_az < 0)earth_az = earth_az+(2*PI);

	double earth_az_mag = length( _V(earthlocalpos.x, 0, earthlocalpos.z));

	int earth_hemi_sw = 0;
	if (earthlocalpos.y < 0)
	{
		earth_hemi_sw = 1;
		earthlocalpos.y = -earthlocalpos.y;
	}
	else earth_hemi_sw = 0;
	double earth_alt = atan2(earthlocalpos.y, earth_az_mag);

	double hga1_az = 2*PI*hga_posy_az_proc;
	double hga1_alt = 0.5*PI*hga_posy_alt_proc;
	double hga2_az = 2*PI*hga_negy_az_proc;
	double hga2_alt = 0.5*PI*hga_negy_alt_proc;


	if (earth_hemi_sw == 0)
	{
		double ang_diff = earth_az - hga1_az;
		

		double da = step_time*hga_rot_dur;
		double drotaz = da*2*PI;

		double abs_angdiff = ang_diff;
		if (abs_angdiff < 0) abs_angdiff = -abs_angdiff;

		if (ang_diff < 0)ang_diff = ang_diff+(2*PI);

			if ( abs_angdiff > drotaz)
			{
			if (ang_diff < PI && ang_diff > 0) hga_posy_az_proc = hga_posy_az_proc + da;
			else if (ang_diff > PI)hga_posy_az_proc = hga_posy_az_proc - da;
			else if (ang_diff > -PI && ang_diff < 0) hga_posy_az_proc = hga_posy_az_proc - da;
			else if (ang_diff < -PI) hga_posy_az_proc = hga_posy_az_proc + da;

			if (hga_posy_az_proc > 1) hga_posy_az_proc = hga_posy_az_proc - ((int) (hga_posy_az_proc));
			if (hga_posy_az_proc < 0) hga_posy_az_proc = 1 + hga_posy_az_proc - ((int) (hga_posy_az_proc));
			}
			else hga_posy_az_proc = earth_az/(2*PI);



		///// alt
		ang_diff = earth_alt - hga1_alt;

		abs_angdiff = ang_diff;
		if (abs_angdiff < 0) abs_angdiff = -abs_angdiff;

		double drotalt = da*0.5*PI;
		if (abs_angdiff > drotalt)
		{
		if (ang_diff > 0)hga_posy_alt_proc = hga_posy_alt_proc + da;
		else hga_posy_alt_proc = hga_posy_alt_proc - da;
		if (hga_posy_alt_proc < 0) hga_posy_alt_proc = 0;
		if (hga_posy_alt_proc > 1) hga_posy_alt_proc = 1;
		}
		else hga_posy_alt_proc = earth_alt/(0.5*PI);
		
		/////////////////////////////////////////////////////////////
		int num_ves = oapiGetVesselCount();
		int tdrs_count = 0;
		double tdrs_min_dist = 1000000000;
		OBJHANDLE tdrs_tgt;
		for (int i = 0; i < num_ves; i++) 
		{
		OBJHANDLE hVessel = oapiGetVesselByIndex(i);
		if (hVessel == GetHandle()) continue;

		char ves_name[256];
		oapiGetObjectName(hVessel,ves_name,4);
		if (strncmp (ves_name, "TDRS", 4))continue;
		

		VECTOR3 tdrs_pos;
		oapiGetGlobalPos(hVessel,&tdrs_pos);
		VECTOR3 tdrs_pos_loc;
		Global2Local(tdrs_pos,tdrs_pos_loc);

		if (tdrs_pos_loc.y > 0)continue;
		else tdrs_count = tdrs_count+1;

			if (length(tdrs_pos_loc) < tdrs_min_dist)
			{
			tdrs_min_dist = length(tdrs_pos_loc);
			tdrs_tgt = hVessel;
			}
			else tdrs_count = tdrs_count-1;

		}

		if (tdrs_count > 0)
		{
		VECTOR3 tdrs_pos;
		oapiGetGlobalPos(tdrs_tgt,&tdrs_pos);
		VECTOR3 tdrs_pos_loc;
		Global2Local(tdrs_pos,tdrs_pos_loc);


	double tdrs_az = atan2(-tdrs_pos_loc.x, -tdrs_pos_loc.z); // 0deg along -z axis
	if (tdrs_az < 0)tdrs_az = tdrs_az+(2*PI);

	double tdrs_az_mag = length( _V(tdrs_pos_loc.x, 0, tdrs_pos_loc.z));

	if (tdrs_pos_loc.y < 0)
	{
		tdrs_pos_loc.y = -tdrs_pos_loc.y;
	}
	double tdrs_alt = atan2(tdrs_pos_loc.y, tdrs_az_mag);

	ang_diff = tdrs_az - hga2_az;
		

		da = step_time*hga_rot_dur;
		drotaz = da*2*PI;

		abs_angdiff = ang_diff;
		if (abs_angdiff < 0) abs_angdiff = -abs_angdiff;

		if (ang_diff < 0)ang_diff = ang_diff+(2*PI);

			if ( abs_angdiff > drotaz)
			{
			if (ang_diff < PI && ang_diff > 0) hga_negy_az_proc = hga_negy_az_proc + da;
			else if (ang_diff > PI)hga_negy_az_proc = hga_negy_az_proc - da;
			else if (ang_diff > -PI && ang_diff < 0) hga_negy_az_proc = hga_negy_az_proc - da;
			else if (ang_diff < -PI) hga_negy_az_proc = hga_negy_az_proc + da;

			if (hga_negy_az_proc > 1) hga_negy_az_proc = hga_negy_az_proc - ((int) (hga_negy_az_proc));
			if (hga_negy_az_proc < 0) hga_negy_az_proc = 1 + hga_negy_az_proc - ((int) (hga_negy_az_proc));
			}
			else hga_negy_az_proc = tdrs_az/(2*PI);


		///// alt
		ang_diff = tdrs_alt - hga2_alt;

		abs_angdiff = ang_diff;
		if (abs_angdiff < 0) abs_angdiff = -abs_angdiff;

		drotalt = da*0.5*PI;
		if (abs_angdiff > drotalt)
		{
		if (ang_diff > 0)hga_negy_alt_proc = hga_negy_alt_proc + da;
		else hga_negy_alt_proc = hga_negy_alt_proc - da;
		if (hga_negy_alt_proc < 0) hga_negy_alt_proc = 0;
		if (hga_negy_alt_proc > 1) hga_negy_alt_proc = 1;
		}
		else hga_negy_alt_proc = tdrs_alt/(0.5*PI);


		}

	}
	////////////////////////////////////////////////////
	else
	{
		double ang_diff = earth_az - hga2_az;
		

		double da = step_time*hga_rot_dur;
		double drotaz = da*2*PI;

		double abs_angdiff = ang_diff;
		if (abs_angdiff < 0) abs_angdiff = -abs_angdiff;

		if (ang_diff < 0)ang_diff = ang_diff+(2*PI);

			if ( abs_angdiff > drotaz)
			{
			if (ang_diff < PI && ang_diff > 0) hga_negy_az_proc = hga_negy_az_proc + da;
			else if (ang_diff > PI)hga_negy_az_proc = hga_negy_az_proc - da;
			else if (ang_diff > -PI && ang_diff < 0) hga_negy_az_proc = hga_negy_az_proc - da;
			else if (ang_diff < -PI) hga_negy_az_proc = hga_negy_az_proc + da;

			if (hga_negy_az_proc > 1) hga_negy_az_proc = hga_negy_az_proc - ((int) (hga_negy_az_proc));
			if (hga_negy_az_proc < 0) hga_negy_az_proc = 1 + hga_negy_az_proc - ((int) (hga_negy_az_proc));
			}
			else hga_negy_az_proc = earth_az/(2*PI);


		///// alt
		ang_diff = earth_alt - hga2_alt;

		abs_angdiff = ang_diff;
		if (abs_angdiff < 0) abs_angdiff = -abs_angdiff;

		double drotalt = da*0.5*PI;
		if (abs_angdiff > drotalt)
		{
		if (ang_diff > 0)hga_negy_alt_proc = hga_negy_alt_proc + da;
		else hga_negy_alt_proc = hga_negy_alt_proc - da;
		if (hga_negy_alt_proc < 0) hga_negy_alt_proc = 0;
		if (hga_negy_alt_proc > 1) hga_negy_alt_proc = 1;
		}
		else hga_negy_alt_proc = earth_alt/(0.5*PI);
	/////////////////////////////////////////////////////
		int num_ves = oapiGetVesselCount();
		int tdrs_count = 0;
		double tdrs_min_dist = 1000000000;
		OBJHANDLE tdrs_tgt;
		for (int i = 0; i < num_ves; i++) 
		{
		OBJHANDLE hVessel = oapiGetVesselByIndex(i);
		if (hVessel == GetHandle()) continue;

		char ves_name[256];
		oapiGetObjectName(hVessel,ves_name,4);
		if (strncmp (ves_name, "TDRS", 4))continue;
		

		VECTOR3 tdrs_pos;
		oapiGetGlobalPos(hVessel,&tdrs_pos);
		VECTOR3 tdrs_pos_loc;
		Global2Local(tdrs_pos,tdrs_pos_loc);

		if (tdrs_pos_loc.y < 0)continue;
		else tdrs_count = tdrs_count+1;

			if (length(tdrs_pos_loc) < tdrs_min_dist)
			{
			tdrs_min_dist = length(tdrs_pos_loc);
			tdrs_tgt = hVessel;
			}
			else tdrs_count = tdrs_count-1;
		}

		if (tdrs_count > 0)
		{
		VECTOR3 tdrs_pos;
		oapiGetGlobalPos(tdrs_tgt,&tdrs_pos);
		VECTOR3 tdrs_pos_loc;
		Global2Local(tdrs_pos,tdrs_pos_loc);

		double tdrs_az = atan2(-tdrs_pos_loc.x, -tdrs_pos_loc.z); // 0deg along -z axis
	if (tdrs_az < 0)tdrs_az = tdrs_az+(2*PI);

	double tdrs_az_mag = length( _V(tdrs_pos_loc.x, 0, tdrs_pos_loc.z));

	if (tdrs_pos_loc.y < 0)
	{
		tdrs_pos_loc.y = -tdrs_pos_loc.y;
	}
	double tdrs_alt = atan2(tdrs_pos_loc.y, tdrs_az_mag);

		ang_diff = tdrs_az - hga1_az;
		

		da = step_time*hga_rot_dur;
		drotaz = da*2*PI;

		abs_angdiff = ang_diff;
		if (abs_angdiff < 0) abs_angdiff = -abs_angdiff;

		if (ang_diff < 0)ang_diff = ang_diff+(2*PI);

			if ( abs_angdiff > drotaz)
			{
			if (ang_diff < PI && ang_diff > 0) hga_posy_az_proc = hga_posy_az_proc + da;
			else if (ang_diff > PI)hga_posy_az_proc = hga_posy_az_proc - da;
			else if (ang_diff > -PI && ang_diff < 0) hga_posy_az_proc = hga_posy_az_proc - da;
			else if (ang_diff < -PI) hga_posy_az_proc = hga_posy_az_proc + da;

			if (hga_posy_az_proc > 1) hga_posy_az_proc = hga_posy_az_proc - ((int) (hga_posy_az_proc));
			if (hga_posy_az_proc < 0) hga_posy_az_proc = 1 + hga_posy_az_proc - ((int) (hga_posy_az_proc));
			}
			else hga_posy_az_proc = tdrs_az/(2*PI);



		///// alt
		ang_diff = tdrs_alt - hga1_alt;

		abs_angdiff = ang_diff;
		if (abs_angdiff < 0) abs_angdiff = -abs_angdiff;

		double drotalt = da*0.5*PI;
		if (abs_angdiff > drotalt)
		{
		if (ang_diff > 0)hga_posy_alt_proc = hga_posy_alt_proc + da;
		else hga_posy_alt_proc = hga_posy_alt_proc - da;
		if (hga_posy_alt_proc < 0) hga_posy_alt_proc = 0;
		if (hga_posy_alt_proc > 1) hga_posy_alt_proc = 1;
		}
		else hga_posy_alt_proc = tdrs_alt/(0.5*PI);


		}

	}

	SetAnimation (anim_hga_posy_az, hga_posy_az_proc);
	SetAnimation (anim_hga_posy_alt, hga_posy_alt_proc);
	SetAnimation (anim_hga_negy_az, hga_negy_az_proc);
	SetAnimation (anim_hga_negy_alt, hga_negy_alt_proc);

}



void hst_ex::StowHGA (void)
{
double da = step_time*hga_rot_dur;

if (hga_posy_az_proc > 0)hga_posy_az_proc = hga_posy_az_proc - da;
if (hga_posy_az_proc < 0)hga_posy_az_proc = 0;

if (hga_posy_alt_proc > 0)hga_posy_alt_proc = hga_posy_alt_proc - da;
if (hga_posy_alt_proc < 0)hga_posy_alt_proc = 0;

if (hga_negy_az_proc > 0)hga_negy_az_proc = hga_negy_az_proc - da;
if (hga_negy_az_proc < 0)hga_negy_az_proc = 0;

if (hga_negy_alt_proc > 0)hga_negy_alt_proc = hga_negy_alt_proc - da;
if (hga_negy_alt_proc < 0)hga_negy_alt_proc = 0;

	SetAnimation (anim_hga_posy_az, hga_posy_az_proc);
	SetAnimation (anim_hga_posy_alt, hga_posy_alt_proc);
	SetAnimation (anim_hga_negy_az, hga_negy_az_proc);
	SetAnimation (anim_hga_negy_alt, hga_negy_alt_proc);


}

void hst_ex::DoKillRot (void)
{
		VECTOR3 avel = _V(0,0,0);
		GetAngularVel(avel);
		double dt = oapiGetSimStep();
		double drot = RAD*0.1*dt;

		if (avel.x > 0)
		{
			avel.x = avel.x-drot;
			if (avel.x < 0)avel.x = 0;
		}

		if (avel.x < 0)
		{
			avel.x = avel.x+drot;
			if (avel.x > 0)avel.x = 0;
		}


		if (avel.y > 0)
		{
			avel.y = avel.y-drot;
			if (avel.y < 0)avel.y = 0;
		}

		if (avel.y < 0)
		{
			avel.y = avel.y+drot;
			if (avel.y > 0)avel.y = 0;
		}


		if (avel.z > 0)
		{
			avel.z = avel.z-drot;
			if (avel.z < 0)avel.z = 0;
		}

		if (avel.z < 0)
		{
			avel.z = avel.z+drot;
			if (avel.z > 0)avel.z = 0;
		}

		SetAngularVel(avel);
}

void hst_ex::GoToTarget(void)
{
OBJHANDLE hTarget = oapiGetObjectByName(refname);
VECTOR3 tgt_pos_glob = _V(0,0,0);
oapiGetGlobalPos(hTarget, &tgt_pos_glob);
VECTOR3 tgt_pos_loc = _V(0,0,0);
Global2Local(tgt_pos_glob, tgt_pos_loc);
normalise(tgt_pos_loc);

VECTOR3 rot_axis_loc = crossp(tgt_pos_loc, _V(0,0,1));
double ang_sep = acos(dotp(tgt_pos_loc, _V(0,0,1)));


VECTOR3 rot_axis_glob;
GlobalRot(rot_axis_loc, rot_axis_glob);
normalise(rot_axis_glob);
rot_axis_glob.x = -rot_axis_glob.x;
rot_axis_glob.y = -rot_axis_glob.y;
rot_axis_glob.z = -rot_axis_glob.z;



VECTOR3 hst_x_loc = _V(1,0,0);
VECTOR3 hst_y_loc = _V(0,1,0);
VECTOR3 hst_z_loc = _V(0,0,1);

VECTOR3 hst_x_glob = _V(0,0,0);
VECTOR3 hst_y_glob = _V(0,0,0);
VECTOR3 hst_z_glob = _V(0,0,0);

GlobalRot(hst_x_loc, hst_x_glob);
GlobalRot(hst_y_loc, hst_y_glob);
GlobalRot(hst_z_loc, hst_z_glob);

	double rot_angle = step_time*1*RAD;

if (rot_angle < ang_sep)
	{
	double ct = cos(rot_angle);
	double st = sin(rot_angle);

	double ux = rot_axis_glob.x;
	double uy = rot_axis_glob.y;
	double uz = rot_axis_glob.z;
	double ux2 = ux*ux;
	double uy2 = uy*uy;
	double uz2 = uz*uz;

	MATRIX3 RG1;

	RG1.m11 = ux2+(1-ux2)*ct;
	RG1.m12 = ux*uy*(1-ct)-uz*st;
	RG1.m13 = ux*uz*(1-ct)+uy*st;
	RG1.m21 = ux*uy*(1-ct)+uz*st;
	RG1.m22 = uy2+(1-uy2)*ct;
	RG1.m23 = uy*uz*(1-ct)-ux*st;
	RG1.m31 = ux*uz*(1-ct)-uy*st;
	RG1.m32 = uy*uz*(1-ct)+ux*st;
	RG1.m33 = uz2+(1-uz2)*ct;

	VECTOR3 xright = mul(RG1,hst_x_glob);
	VECTOR3 yup = mul(RG1,hst_y_glob);
	VECTOR3 zforward = mul(RG1,hst_z_glob);

	double gamma = atan2( xright.x, yup.x);
	double beta = -1*(asin(zforward.x));
	double alpha = atan2(zforward.y, zforward.z);
	SetGlobalOrientation (_V(alpha, beta, (PI/2)-gamma));
	SetAngularVel (_V(0,0,0));

	}

else
	{
	double ct = cos(ang_sep);
	double st = sin(ang_sep);

	double ux = rot_axis_glob.x;
	double uy = rot_axis_glob.y;
	double uz = rot_axis_glob.z;
	double ux2 = ux*ux;
	double uy2 = uy*uy;
	double uz2 = uz*uz;

	MATRIX3 RG1;

	RG1.m11 = ux2+(1-ux2)*ct;
	RG1.m12 = ux*uy*(1-ct)-uz*st;
	RG1.m13 = ux*uz*(1-ct)+uy*st;
	RG1.m21 = ux*uy*(1-ct)+uz*st;
	RG1.m22 = uy2+(1-uy2)*ct;
	RG1.m23 = uy*uz*(1-ct)-ux*st;
	RG1.m31 = ux*uz*(1-ct)-uy*st;
	RG1.m32 = uy*uz*(1-ct)+ux*st;
	RG1.m33 = uz2+(1-uz2)*ct;

	VECTOR3 xright = mul(RG1,hst_x_glob);
	VECTOR3 yup = mul(RG1,hst_y_glob);
	VECTOR3 zforward = mul(RG1,hst_z_glob);

	double gamma = atan2( xright.x, yup.x);
	double beta = -1*(asin(zforward.x));
	double alpha = atan2(zforward.y, zforward.z);
	SetGlobalOrientation (_V(alpha, beta, (PI/2)-gamma));
	SetAngularVel (_V(0,0,0));

	}


}


bool hst_ex::clbkDrawHUD (int mode, const HUDPAINTSPEC *hps, oapi::Sketchpad *skp)
{
	// draw the default HUD
	VESSEL3::clbkDrawHUD (mode, hps, skp);

	oapiSetDefNavDisplay(0);
	oapiSetDefRCSDisplay(0);

	int s = hps->H;
	double d = (s*0.00130208);
	int sw = ((hps->W));
	int lw =  (int)(16*sw/1024);
	int lwoffset = sw - (18*lw);
	int hlw = (int) (lw/2);


	int roxl = 0;
	int royl = 0;

	double ds = s;
	double dsw = sw;
	double sc_ratio = ds/dsw;


	if (sc_ratio < 0.7284)
	{
	roxl = (lw*10);
	royl = (int) (-88*d);
	}



	int wd = (int)(152*d);
	int wc = (int)(168*d);
	int w0 = (int)(184*d);
	int w1 = (int) (200*d);
	int w2 = (int)(216*d);
	int w3 = (int)(232*d);
	int w4 = (int)(248*d);
	int w5 = (int)(264*d);
	int w6 = (int)(280*d);
	int w7 = (int)(296*d);
	int w8 = (int)(312*d);
	int w9 = (int)(328*d);
	int w10 = (int)(344*d);
	int w11 = (int)(360*d);
	int w12 = (int)(376*d);
	int w13 = (int)(392*d);
	int w14 = (int)(408*d);
	int w15 = (int)(424*d);


	{
	skp->SetTextColor(0x0066FF66);
	char abuf[256];
	sprintf(abuf,"[E] = Enter Target Name:");
	skp->Text ((10+roxl), (wd+royl), abuf,strlen(abuf));
	}

	{
	skp->SetTextColor(0x0066FFFF);
	char abuf[256];
	sprintf(abuf,"                        %s", refname);
	skp->Text ((10+roxl), (wd+royl), abuf,strlen(abuf));
	}

	if (ap_status == 2)
	{
	skp->SetTextColor(0x0066FFFF);
	char abuf[256];
	sprintf(abuf,"[V] = GoTo Target On/Off");
	skp->Text ((10+roxl), (wc+royl), abuf,strlen(abuf));
	}
	else
	{
	skp->SetTextColor(0x0066FF66);
	char abuf[256];
	sprintf(abuf,"[V] = GoTo Target On/Off");
	skp->Text ((10+roxl), (wc+royl), abuf,strlen(abuf));
	}

	{
	skp->SetTextColor(0x0066FF66);
	char abuf[256];
	sprintf(abuf,"[G]/[Shift+G] = +/-Zoom FOV %0.2f", (oapiCameraAperture()*2*DEG));
	skp->Text ((10+roxl), (w1+royl), abuf,strlen(abuf));
	}
	


	if (ap_status == 1)
	{
	skp->SetTextColor(0x0066FFFF);
	char abuf[256];
	sprintf(abuf,"[NumPad5] = KillRot On/Off");
	skp->Text ((10+roxl), (w3+royl), abuf,strlen(abuf));
	}
	else
	{
	skp->SetTextColor(0x0066FF66);
	char abuf[256];
	sprintf(abuf,"[NumPad5] = KillRot On/Off");
	skp->Text ((10+roxl), (w3+royl), abuf,strlen(abuf));
	}

	if (ap_status == 3)
	{
	skp->SetTextColor(0x0066FFFF);
	char abuf[256];
	sprintf(abuf,"[B] = GoTo RA Dec On/Off");
	skp->Text ((10+roxl), (w5+royl), abuf,strlen(abuf));
	}
	else
	{
	skp->SetTextColor(0x0066FF66);
	char abuf[256];
	sprintf(abuf,"[B] = GoTo RA Dec On/Off");
	skp->Text ((10+roxl), (w5+royl), abuf,strlen(abuf));
	}

	{
	skp->SetTextColor(0x0066FF66);
	char abuf[256];
	sprintf(abuf,"[Shift+B] = Enter Target RA");
	skp->Text ((10+roxl), (w6+royl), abuf,strlen(abuf));
	}

	{
	skp->SetTextColor(0x0066FF66);
	char abuf[256];
	sprintf(abuf,"[Ctrl+B] = Enter Target Dec");
	skp->Text ((10+roxl), (w7+royl), abuf,strlen(abuf));
	}


	{
	char xbuf[256];
	char ybuf[256];
	char zbuf[256];
	char pbuf[256];
	char qbuf[256];
	char rbuf[256];

	sprintf(pbuf, "%d", tgt_ra_hh); 
	if (tgt_ra_hh < 10)
	{
	sprintf(xbuf, "0%s", pbuf);
	strcpy(pbuf, xbuf);
	}
	sprintf(qbuf, "%d", tgt_ra_mm); 
	if (tgt_ra_mm < 10)
	{
	sprintf(ybuf, "0%s", qbuf);
	strcpy(qbuf, ybuf);
	}
	sprintf(rbuf, "%d", tgt_ra_ss); 
	if (tgt_ra_ss < 10)
	{
	sprintf(zbuf, "0%s", rbuf);
	strcpy(rbuf, zbuf);
	}

	skp->SetTextColor(0x0066FF66);
	char abuf[256];
	sprintf(abuf,"Target RA :  %sh %sm %ss", pbuf, qbuf, rbuf);
	skp->Text ((10+roxl), (w8+royl), abuf,strlen(abuf));
	}

	{
	char xbuf[256];
	char ybuf[256];
	char zbuf[256];
	char pbuf[256];
	char qbuf[256];
	char rbuf[256];

	sprintf(pbuf, "%d", max(tgt_dec_hh, -tgt_dec_hh)); 
	if (tgt_dec_hh < 10 && tgt_dec_hh > -10)
	{
	sprintf(xbuf, "0%s", pbuf);
	strcpy(pbuf, xbuf);
	}
	sprintf(qbuf, "%d", max(tgt_dec_mm, -tgt_dec_mm)); 
	if (tgt_dec_mm < 10 && tgt_dec_mm > -10)
	{
	sprintf(ybuf, "0%s", qbuf);
	strcpy(qbuf, ybuf);
	}
	sprintf(rbuf, "%d", max(tgt_dec_ss, -tgt_dec_ss)); 
	if (tgt_dec_ss < 10 && tgt_dec_ss > -10)
	{
	sprintf(zbuf, "0%s", rbuf);
	strcpy(rbuf, zbuf);
	}

	skp->SetTextColor(0x0066FF66);
	char abuf[256];
	if (tgt_dec_hh < 0)
	sprintf(abuf,"Target Dec: -%s %s' %s''", pbuf, qbuf, rbuf);
	else sprintf(abuf,"Target Dec: +%s %s' %s''", pbuf, qbuf, rbuf);
	skp->Text ((10+roxl), (w9+royl), abuf,strlen(abuf));
	}

	if (sa1_stbd_elb_sts == 2 && sa_stbd_cfg == 0 && sa_port_cfg == 0)
	{
	skp->SetTextColor(0x0066FF66);
	char abuf[256];
	sprintf(abuf,"F5 = Deploy Array Arms");
	skp->Text ((10+roxl), (w11+royl), abuf,strlen(abuf));
	}

	if (sa1_stbd_elb_sts == 0 && sa1_stbd_scale_sts == 2 && sa_stbd_status == 0 && sa_stbd_cfg == 0)
	{
	skp->SetTextColor(0x0066FF66);
	char abuf[256];
	sprintf(abuf,"F6 = Open Stbd Array");
	skp->Text ((10+roxl), (w11+royl), abuf,strlen(abuf));
	}

	if (sa1_port_elb_sts == 0 && sa1_port_scale_sts == 2 && sa_port_status == 0 && sa_port_cfg == 0)
	{
	skp->SetTextColor(0x0066FF66);
	char abuf[256];
	sprintf(abuf,"F7 = Open Port Array");
	skp->Text ((10+roxl), (w12+royl), abuf,strlen(abuf));
	}

	if (sa1_stbd_scale_sts == 0 && sa1_port_scale_sts == 0)
	{
	if (autosun_status == 1) skp->SetTextColor(0x0066FFFF);
	else skp->SetTextColor(0x0066FF66);
	char abuf[256];
	sprintf(abuf,"N = Arrays Sun Tracking On/Off");
	skp->Text ((10+roxl), (w11+royl), abuf,strlen(abuf));
	}

	if ((sa1_port_elb_sts + sa1_stbd_elb_sts + sa1_stbd_scale_sts) == 0)
	{
	if (autosun_status == 2) skp->SetTextColor(0x0066FFFF);
	else skp->SetTextColor(0x0066FF66);
	char abuf[256];
	sprintf(abuf,"M = Stow Arrays");
	skp->Text ((10+roxl), (w13+royl), abuf,strlen(abuf));
	}
	else if ((sa1_port_elb_sts + sa1_stbd_elb_sts + sa1_port_scale_sts) == 0)
	{
	if (autosun_status == 2) skp->SetTextColor(0x0066FFFF);
	else skp->SetTextColor(0x0066FF66);
	char abuf[256];
	sprintf(abuf,"M = Stow Arrays");
	skp->Text ((10+roxl), (w13+royl), abuf,strlen(abuf));
	}

	{
	skp->SetTextColor(0x0066FF66);
	char abuf[256];
	sprintf(abuf,"K = Deploy/Stow HGA+Sunshield");
	skp->Text ((10+roxl), (w14+royl), abuf,strlen(abuf));
	}
	

	return true;
}



void hst_ex::GoToRADec(void){

	double dec_angle = tgt_dec_rad;
	double ra_angle = tgt_ra_rad;

	////// target +z axis vector
	VECTOR3 vessel_z = _V(0,0,1);


	

	////// create rotation matrix for rotation by declination angle
	MATRIX3 RG1;

	double ct = cos(-dec_angle);
	double st = sin(-dec_angle);

	double ux = 1;
	double uy = 0;
	double uz = 0;
	double ux2 = 1;
	double uy2 = 0;
	double uz2 = 0;

	RG1.m11 = ux2+(1-ux2)*ct;
	RG1.m12 = ux*uy*(1-ct)-uz*st;
	RG1.m13 = ux*uz*(1-ct)+uy*st;
	RG1.m21 = ux*uy*(1-ct)+uz*st;
	RG1.m22 = uy2+(1-uy2)*ct;
	RG1.m23 = uy*uz*(1-ct)-ux*st;
	RG1.m31 = ux*uz*(1-ct)-uy*st;
	RG1.m32 = uy*uz*(1-ct)+ux*st;
	RG1.m33 = uz2+(1-uz2)*ct;

	////// target vector after rotation by declination angle
	vessel_z = mul(RG1,vessel_z);



	////// create rotation matrix for rotation by right ascension angle
	MATRIX3 RG2;

	ct = cos((PI/2)-ra_angle);	////// 90 deg offset
	st = sin((PI/2)-ra_angle);	////// 90 deg offset

	ux = 0;
	uy = 1;
	uz = 0;
	ux2 = 0;
	uy2 = 1;
	uz2 = 0;

	RG2.m11 = ux2+(1-ux2)*ct;
	RG2.m12 = ux*uy*(1-ct)-uz*st;
	RG2.m13 = ux*uz*(1-ct)+uy*st;
	RG2.m21 = ux*uy*(1-ct)+uz*st;
	RG2.m22 = uy2+(1-uy2)*ct;
	RG2.m23 = uy*uz*(1-ct)-ux*st;
	RG2.m31 = ux*uz*(1-ct)-uy*st;
	RG2.m32 = uy*uz*(1-ct)+ux*st;
	RG2.m33 = uz2+(1-uz2)*ct;


	////// target vector after rotation by right ascension angle
	vessel_z = mul(RG2,vessel_z);



	////// create rotation matrix for rotation into ecliptic frame
	MATRIX3 RG3;

	double ecliptic_inc = 23.439291*RAD;

	ct = cos(ecliptic_inc);
	st = sin(ecliptic_inc);

	ux = 1;
	uy = 0;
	uz = 0;
	ux2 = 1;
	uy2 = 0;
	uz2 = 0;

	RG3.m11 = ux2+(1-ux2)*ct;
	RG3.m12 = ux*uy*(1-ct)-uz*st;
	RG3.m13 = ux*uz*(1-ct)+uy*st;
	RG3.m21 = ux*uy*(1-ct)+uz*st;
	RG3.m22 = uy2+(1-uy2)*ct;
	RG3.m23 = uy*uz*(1-ct)-ux*st;
	RG3.m31 = ux*uz*(1-ct)-uy*st;
	RG3.m32 = uy*uz*(1-ct)+ux*st;
	RG3.m33 = uz2+(1-uz2)*ct;

	////// target vector after rotation into ecliptic frame
	vessel_z = mul(RG3,vessel_z);

VECTOR3 hst_pos_glob = _V(0,0,0);
GetGlobalPos(hst_pos_glob);
VECTOR3 tgt_pos_glob = vessel_z + hst_pos_glob;
VECTOR3 tgt_pos_loc = _V(0,0,0);
Global2Local(tgt_pos_glob, tgt_pos_loc);
normalise(tgt_pos_loc);

VECTOR3 rot_axis_loc = crossp(tgt_pos_loc, _V(0,0,1));
double ang_sep = acos(dotp(tgt_pos_loc, _V(0,0,1)));


VECTOR3 rot_axis_glob;
GlobalRot(rot_axis_loc, rot_axis_glob);
normalise(rot_axis_glob);
rot_axis_glob.x = -rot_axis_glob.x;
rot_axis_glob.y = -rot_axis_glob.y;
rot_axis_glob.z = -rot_axis_glob.z;



VECTOR3 hst_x_loc = _V(1,0,0);
VECTOR3 hst_y_loc = _V(0,1,0);
VECTOR3 hst_z_loc = _V(0,0,1);

VECTOR3 hst_x_glob = _V(0,0,0);
VECTOR3 hst_y_glob = _V(0,0,0);
VECTOR3 hst_z_glob = _V(0,0,0);

GlobalRot(hst_x_loc, hst_x_glob);
GlobalRot(hst_y_loc, hst_y_glob);
GlobalRot(hst_z_loc, hst_z_glob);

	double rot_angle = step_time*1*RAD;

if (rot_angle < ang_sep)
	{
	double ct = cos(rot_angle);
	double st = sin(rot_angle);

	double ux = rot_axis_glob.x;
	double uy = rot_axis_glob.y;
	double uz = rot_axis_glob.z;
	double ux2 = ux*ux;
	double uy2 = uy*uy;
	double uz2 = uz*uz;

	MATRIX3 RG1;

	RG1.m11 = ux2+(1-ux2)*ct;
	RG1.m12 = ux*uy*(1-ct)-uz*st;
	RG1.m13 = ux*uz*(1-ct)+uy*st;
	RG1.m21 = ux*uy*(1-ct)+uz*st;
	RG1.m22 = uy2+(1-uy2)*ct;
	RG1.m23 = uy*uz*(1-ct)-ux*st;
	RG1.m31 = ux*uz*(1-ct)-uy*st;
	RG1.m32 = uy*uz*(1-ct)+ux*st;
	RG1.m33 = uz2+(1-uz2)*ct;

	VECTOR3 xright = mul(RG1,hst_x_glob);
	VECTOR3 yup = mul(RG1,hst_y_glob);
	VECTOR3 zforward = mul(RG1,hst_z_glob);

	double gamma = atan2( xright.x, yup.x);
	double beta = -1*(asin(zforward.x));
	double alpha = atan2(zforward.y, zforward.z);
	SetGlobalOrientation (_V(alpha, beta, (PI/2)-gamma));
	SetAngularVel (_V(0,0,0));

	}

else
	{
	double ct = cos(ang_sep);
	double st = sin(ang_sep);

	double ux = rot_axis_glob.x;
	double uy = rot_axis_glob.y;
	double uz = rot_axis_glob.z;
	double ux2 = ux*ux;
	double uy2 = uy*uy;
	double uz2 = uz*uz;

	MATRIX3 RG1;

	RG1.m11 = ux2+(1-ux2)*ct;
	RG1.m12 = ux*uy*(1-ct)-uz*st;
	RG1.m13 = ux*uz*(1-ct)+uy*st;
	RG1.m21 = ux*uy*(1-ct)+uz*st;
	RG1.m22 = uy2+(1-uy2)*ct;
	RG1.m23 = uy*uz*(1-ct)-ux*st;
	RG1.m31 = ux*uz*(1-ct)-uy*st;
	RG1.m32 = uy*uz*(1-ct)+ux*st;
	RG1.m33 = uz2+(1-uz2)*ct;

	VECTOR3 xright = mul(RG1,hst_x_glob);
	VECTOR3 yup = mul(RG1,hst_y_glob);
	VECTOR3 zforward = mul(RG1,hst_z_glob);

	double gamma = atan2( xright.x, yup.x);
	double beta = -1*(asin(zforward.x));
	double alpha = atan2(zforward.y, zforward.z);
	SetGlobalOrientation (_V(alpha, beta, (PI/2)-gamma));
	SetAngularVel (_V(0,0,0));

	}
	


}

void hst_ex::DoBeacons(void)
{
////// doors closed
if (door_status[1] == 0 && door_status[0] == 0) beacon[0].active = true;
else beacon[0].active = false;

if (door_status[3] == 0 && door_status[2] == 0) beacon[1].active = true;
else beacon[1].active = false;

if (door_status[5] == 0 && door_status[4] == 0) beacon[2].active = true;
else beacon[2].active = false;

if (door_status[7] == 0 && door_status[6] == 0) beacon[3].active = true;
else beacon[3].active = false;

if (door_status[9] == 0 && door_status[8] == 0) beacon[21].active = true;
else beacon[21].active = false;

///// doors open

if (door_status[1] == 2) beacon[7].active = true;
else beacon[7].active = false;
if (door_status[0] == 2) beacon[6].active = true;
else beacon[6].active = false;

if (door_status[3] == 2) beacon[9].active = true;
else beacon[9].active = false;
if (door_status[2] == 2) beacon[8].active = true;
else beacon[8].active = false;

if (door_status[4] == 2) beacon[10].active = true;
else beacon[10].active = false;
if (door_status[5] == 2) beacon[11].active = true;
else beacon[11].active = false;

if (door_status[6] == 2) beacon[12].active = true;
else beacon[12].active = false;
if (door_status[7] == 2) beacon[13].active = true;
else beacon[13].active = false;

if (door_status[8] == 2) beacon[22].active = true;
else beacon[22].active = false;
if (door_status[9] == 2) beacon[23].active = true;
else beacon[23].active = false;


///// replacement units

/// wfc
beacon[14].active = true;
if (wfc_status == 0)beacon[14].col = &beaconcol1;
else beacon[14].col = &beaconcol2;

/// inst1 thru inst4
beacon[15].active = true;
if (inst1_status == 0)beacon[15].col = &beaconcol1;
else beacon[15].col = &beaconcol2;

beacon[16].active = true;
if (inst2_status == 0)beacon[16].col = &beaconcol1;
else beacon[16].col = &beaconcol2;

beacon[17].active = true;
if (inst3_status == 0)beacon[17].col = &beaconcol1;
else beacon[17].col = &beaconcol2;

beacon[18].active = true;
if (inst4_status == 0)beacon[18].col = &beaconcol1;
else beacon[18].col = &beaconcol2;

/// fgs1
beacon[20].active = true;
if (fgs1_status == 0)beacon[20].col = &beaconcol1;
else beacon[20].col = &beaconcol2;

/// fgs2
beacon[19].active = true;
if (fgs2_status == 0)beacon[19].col = &beaconcol1;
else beacon[19].col = &beaconcol2;

/// rsu1
beacon[24].active = true;
if (rsu1_status == 0)beacon[24].col = &beaconcol1;
else beacon[24].col = &beaconcol2;

/// rsu2
beacon[25].active = true;
if (rsu2_status == 0)beacon[25].col = &beaconcol1;
else beacon[25].col = &beaconcol2;

/// rsu3
beacon[26].active = true;
if (rsu3_status == 0)beacon[26].col = &beaconcol1;
else beacon[26].col = &beaconcol2;

/// sa stbd
if (sa_stbd_cfg == 0)
{
if ((sa1_stbd_elb_sts + sa1_stbd_rot_proc) == 0 && sa1_stbd_scale_sts == 2)  beacon[4].active = true;
else beacon[4].active = false;
}
else
{
if ((sa1_stbd_elb_sts + sa1_stbd_rot_proc + sa1_stbd_scale_sts) == 0)  beacon[4].active = true;
else beacon[4].active = false;
}

if (sa_stbd_status == 0)beacon[4].col = &beaconcol1;
else beacon[4].col = &beaconcol2;

/// sa port
if (sa_port_cfg == 0)
{
if ((sa1_port_elb_sts + sa1_port_rot_proc) == 0 && sa1_port_scale_sts == 2)  beacon[5].active = true;
else beacon[5].active = false;
}
else
{
if ((sa1_port_elb_sts + sa1_port_rot_proc + sa1_port_scale_sts) == 0)  beacon[5].active = true;
else beacon[5].active = false;
}
if (sa_port_status == 0)beacon[5].col = &beaconcol1;
else beacon[5].col = &beaconcol2;


}
void hst_ex::CheckVessels(void)
{
if (cv1cv2_check == 0)cv1cv2_check = 1;
else if (cv1cv2_check == 1)cv1cv2_check = 2;


VECTOR3 pos = _V(0,0,0);
VECTOR3 dir = _V(0,0,0);
VECTOR3 rot = _V(0,0,0);
VECTOR3 att_glob = _V(0,0,0);

////
VECTOR3 hst_drp_glob[24];
for (int i = 0; i < 24; i++)
{
Local2Global(beaconpos[i], hst_drp_glob[i]);
}

///// grap points
VECTOR3 hst_attp_glob[9];
for (int i = 0; i < 9; i++)
{
Local2Global(attp_pos[i], hst_attp_glob[i]);
}


int num_ves = oapiGetVesselCount();
for (int i = 0; i < num_ves; i++) 
{
VECTOR3 hst_pos_glob = _V(0,0,0);
GetGlobalPos(hst_pos_glob);
OBJHANDLE hVessel = oapiGetVesselByIndex(i);
if (hVessel == GetHandle()) continue;

VECTOR3 ves_pos_glob = _V(0,0,0);
oapiGetGlobalPos(hVessel, &ves_pos_glob);
if (dist(ves_pos_glob, hst_pos_glob)> 20)continue;

VESSEL *v = oapiGetVesselInterface (hVessel);
DWORD nAttach;
if (att_pc_check == 0)nAttach = v->AttachmentCount (false);
else nAttach = v->AttachmentCount (true);

if (nAttach < 1)continue;


for (DWORD j = 0; j < nAttach; j++)
{
ATTACHMENTHANDLE hAtt;
if(att_pc_check == 0) hAtt = v->GetAttachmentHandle (false, j);
else hAtt = v->GetAttachmentHandle (true, j);
v->GetAttachmentParams (hAtt, pos, dir, rot);
v->Local2Global (pos, att_glob);


//////// doors /////////
int di = 0;
for (di = 0; di < 4; di++)
	{
	if (dist(hst_drp_glob[di], att_glob) < 0.1)
		{
		if (door_status[(2*di)] == 0 && door_status[((2*di)+1)] == 0)
			{
			door_status[(2*di)] = 1;
			door_status[((2*di)+1)] = 1;
			return;
			}

		}
	}


for (di = 6; di < 14; di++)
{
	if (dist(hst_drp_glob[di], att_glob) < 0.1)
	{
	if (door_status[(di-6)] == 2)
		{
		door_status[(di-6)] = 3;
		return;
		}
	}
}

////// fgs1 doors
	
	if (dist(hst_drp_glob[21], att_glob) < 0.1)
		{
		if (door_status[8] == 0 && door_status[9] == 0)
			{
			door_status[8] = 1;
			door_status[9] = 1;
			return;
			}

		}

	if (dist(hst_drp_glob[22], att_glob) < 0.1)
		{
		if (door_status[8] == 2)
			{
			door_status[8] = 3;
			return;
			}

		}

	if (dist(hst_drp_glob[23], att_glob) < 0.1)
		{
		if (door_status[9] == 2)
			{
			door_status[9] = 3;
			return;
			}

		}


	

/////// wfc
	if (dist(hst_drp_glob[14], att_glob) < 0.1)
	{
	if (wfc_status == 0)
		{
		wfc_status = 1;
		SetMeshVisibilityMode(mesh2,MESHVIS_NEVER);
		SpawnWFC();
		anot_status = 1;
		return;
		}
	}

	

	if (wfc_status == 1)
	{
		if (GetAttachmentStatus(wfc_attach) == NULL)
		{
		wfc_status = 2;
		return;
		}
	}



	else if (wfc_status == 2)
	{
	//sprintf(oapiDebugString(), "length %0.2f", dist(hst_attp_glob[0], att_glob));
	if (dist(hst_attp_glob[0], att_glob) < 0.2)
		{
		const char *id = v->GetAttachmentId (hAtt);
		if (strncmp (id, "WFC", 3)); // attachment point not compatible
		else 
			{
			int atchk = 0;
			DWORD natt = v->AttachmentCount (true);
			for (DWORD k = 0; k < natt; k++)
			{
			ATTACHMENTHANDLE atth = v->GetAttachmentHandle (true, k);
			if (!v->GetAttachmentStatus(atth) == NULL) atchk = 1;
			}

			if (atchk == 0)
			{
			oapiDeleteVessel(hVessel);
			SetMeshVisibilityMode(mesh2,MESHVIS_EXTERNAL);
			wfc_status = 0;
			anot2_status = 1;
			return;
			}

			}
		}
	}
////////////// inst1

	if (dist(hst_drp_glob[15], att_glob) < 0.1)
	{
	if (inst1_status == 0)
		{
		inst1_status = 1;
		SetMeshVisibilityMode(mesh3,MESHVIS_NEVER);
		SpawnInst1();
		anot_status = 1;
		return;
		}
	}

	

	if (inst1_status == 1)
	{
		if (GetAttachmentStatus(inst1_attach) == NULL)
		{
		inst1_status = 2;
		return;
		}
	}



	else if (inst1_status == 2)
	{
	if (dist(hst_attp_glob[4], att_glob) < 0.2)
		{
		const char *id = v->GetAttachmentId (hAtt);
		if (strncmp (id, "IN1", 3)); // attachment point not compatible
		else 
			{
			int atchk = 0;
			DWORD natt = v->AttachmentCount (true);
			for (DWORD k = 0; k < natt; k++)
			{
			ATTACHMENTHANDLE atth = v->GetAttachmentHandle (true, k);
			if (!v->GetAttachmentStatus(atth) == NULL) atchk = 1;
			}

			if (atchk == 0)
			{
			oapiGetObjectName(hVessel, inst1_name, 6);
			oapiDeleteVessel(hVessel);
			SetMeshVisibilityMode(mesh3,MESHVIS_EXTERNAL);
			inst1_status = 0;
			anot2_status = 1;
			return;
			}

			}
		}
	}


////////////// inst2
	if (dist(hst_drp_glob[16], att_glob) < 0.1)
	{
	if (inst2_status == 0)
		{
		inst2_status = 1;
		SetMeshVisibilityMode(mesh4,MESHVIS_NEVER);
		SpawnInst2();
		anot_status = 1;
		return;
		}
	}

	

	if (inst2_status == 1)
	{
		if (GetAttachmentStatus(inst2_attach) == NULL)
		{
		inst2_status = 2;
		return;
		}
	}



	else if (inst2_status == 2)
	{
	if (dist(hst_attp_glob[5], att_glob) < 0.2)
		{
		const char *id = v->GetAttachmentId (hAtt);
		if (strncmp (id, "IN2", 3)); // attachment point not compatible
		else 
			{
			int atchk = 0;
			DWORD natt = v->AttachmentCount (true);
			for (DWORD k = 0; k < natt; k++)
			{
			ATTACHMENTHANDLE atth = v->GetAttachmentHandle (true, k);
			if (!v->GetAttachmentStatus(atth) == NULL) atchk = 1;
			}

			if (atchk == 0)
			{
			oapiGetObjectName(hVessel, inst2_name, 6);
			oapiDeleteVessel(hVessel);
			SetMeshVisibilityMode(mesh4,MESHVIS_EXTERNAL);
			inst2_status = 0;
			anot2_status = 1;
			return;
			}

			}
		}
	}

////////////// inst3
	if (dist(hst_drp_glob[17], att_glob) < 0.1)
	{
	if (inst3_status == 0)
		{
		inst3_status = 1;
		SetMeshVisibilityMode(mesh5,MESHVIS_NEVER);
		SpawnInst3();
		anot_status = 1;
		return;
		}
	}

	

	if (inst3_status == 1)
	{
		if (GetAttachmentStatus(inst3_attach) == NULL)
		{
		inst3_status = 2;
		return;
		}
	}



	else if (inst3_status == 2)
	{
	if (dist(hst_attp_glob[6], att_glob) < 0.2)
		{
		const char *id = v->GetAttachmentId (hAtt);
		if (strncmp (id, "IN3", 3)); // attachment point not compatible
		else 
			{
			int atchk = 0;
			DWORD natt = v->AttachmentCount (true);
			for (DWORD k = 0; k < natt; k++)
			{
			ATTACHMENTHANDLE atth = v->GetAttachmentHandle (true, k);
			if (!v->GetAttachmentStatus(atth) == NULL) atchk = 1;
			}

			if (atchk == 0)
			{
			oapiGetObjectName(hVessel, inst3_name, 6);
			oapiDeleteVessel(hVessel);
			SetMeshVisibilityMode(mesh5,MESHVIS_EXTERNAL);
			inst3_status = 0;
			anot2_status = 1;
			return;
			}

			}
		}
	}

////////////// inst4
	if (dist(hst_drp_glob[18], att_glob) < 0.1)
	{
	if (inst4_status == 0)
		{
		inst4_status = 1;
		SetMeshVisibilityMode(mesh6,MESHVIS_NEVER);
		SpawnInst4();
		anot_status = 1;
		return;
		}
	}

	

	if (inst4_status == 1)
	{
		if (GetAttachmentStatus(inst4_attach) == NULL)
		{
		inst4_status = 2;
		return;
		}
	}



	else if (inst4_status == 2)
	{
	if (dist(hst_attp_glob[7], att_glob) < 0.2)
		{
		const char *id = v->GetAttachmentId (hAtt);
		if (strncmp (id, "IN4", 3)); // attachment point not compatible
		else 
			{
			int atchk = 0;
			DWORD natt = v->AttachmentCount (true);
			for (DWORD k = 0; k < natt; k++)
			{
			ATTACHMENTHANDLE atth = v->GetAttachmentHandle (true, k);
			if (!v->GetAttachmentStatus(atth) == NULL) atchk = 1;
			}

			if (atchk == 0)
			{
			oapiGetObjectName(hVessel, inst4_name, 6);
			oapiDeleteVessel(hVessel);
			SetMeshVisibilityMode(mesh6,MESHVIS_EXTERNAL);
			inst4_status = 0;
			anot2_status = 1;
			return;
			}

			}
		}
	}


	////////////////////// fgs1
	if (dist(hst_drp_glob[20], att_glob) < 0.1)
	{
	if (fgs1_status == 0)
		{
		fgs1_status = 1;
		SetMeshVisibilityMode(mesh7,MESHVIS_NEVER);
		SpawnFGS1();
		anot_status = 1;
		return;
		}
	}

	

	if (fgs1_status == 1)
	{
		if (GetAttachmentStatus(fgs1_attach) == NULL)
		{
		fgs1_status = 2;
		return;
		}
	}



	else if (fgs1_status == 2)
	{
	if (dist(hst_attp_glob[8], att_glob) < 0.2)
		{
		const char *id = v->GetAttachmentId (hAtt);
		if (strncmp (id, "FGS", 3)); // attachment point not compatible
		else 
			{
			int atchk = 0;
			DWORD natt = v->AttachmentCount (true);
			for (DWORD k = 0; k < natt; k++)
			{
			ATTACHMENTHANDLE atth = v->GetAttachmentHandle (true, k);
			if (!v->GetAttachmentStatus(atth) == NULL) atchk = 1;
			}

			if (atchk == 0)
			{
			oapiDeleteVessel(hVessel);
			SetMeshVisibilityMode(mesh7,MESHVIS_EXTERNAL);
			fgs1_status = 0;
			anot2_status = 1;
			return;
			}

			}
		}
	}






	///////////////// fgs2
	if (dist(hst_drp_glob[19], att_glob) < 0.1)
	{
	if (fgs2_status == 0)
		{
		fgs2_status = 1;
		SetMeshVisibilityMode(mesh8,MESHVIS_NEVER);
		SpawnFGS2();
		anot_status = 1;
		return;
		}
	}

	

	if (fgs2_status == 1)
	{
		if (GetAttachmentStatus(fgs2_attach) == NULL)
		{
		fgs2_status = 2;
		return;
		}
	}



	else if (fgs2_status == 2)
	{
	if (dist(hst_attp_glob[1], att_glob) < 0.2)
		{
		const char *id = v->GetAttachmentId (hAtt);
		if (strncmp (id, "FGS", 3)); // attachment point not compatible
		else 
			{
			int atchk = 0;
			DWORD natt = v->AttachmentCount (true);
			for (DWORD k = 0; k < natt; k++)
			{
			ATTACHMENTHANDLE atth = v->GetAttachmentHandle (true, k);
			if (!v->GetAttachmentStatus(atth) == NULL) atchk = 1;
			}

			if (atchk == 0)
			{
			oapiDeleteVessel(hVessel);
			SetMeshVisibilityMode(mesh8,MESHVIS_EXTERNAL);
			fgs2_status = 0;
			anot2_status = 1;
			return;
			}

			}
		}
	}


}

}




}


void hst_ex::SpawnWFC(void)
{
VESSELSTATUS vs;
  GetStatus(vs);
  OBJHANDLE hWFC = oapiCreateVessel ("WFC", "HST_EX/wfc", vs);
  VESSEL *v = oapiGetVesselInterface (hWFC);
  ATTACHMENTHANDLE hAttWFC = v->GetAttachmentHandle(true,0);
  AttachChild(hWFC, wfc_attach, hAttWFC);
}


void hst_ex::SpawnInst1(void)
{
VESSELSTATUS vs;
  GetStatus(vs);
  OBJHANDLE hWFC = oapiCreateVessel (inst1_name, "HST_EX/inst1", vs);
  VESSEL *v = oapiGetVesselInterface (hWFC);
  ATTACHMENTHANDLE hAttWFC = v->GetAttachmentHandle(true,0);
  AttachChild(hWFC, inst1_attach, hAttWFC);
}

void hst_ex::SpawnInst2(void)
{
VESSELSTATUS vs;
  GetStatus(vs);
  OBJHANDLE hWFC = oapiCreateVessel (inst2_name, "HST_EX/inst2", vs);
  VESSEL *v = oapiGetVesselInterface (hWFC);
  ATTACHMENTHANDLE hAttWFC = v->GetAttachmentHandle(true,0);
  AttachChild(hWFC, inst2_attach, hAttWFC);
}

void hst_ex::SpawnInst3(void)
{
VESSELSTATUS vs;
  GetStatus(vs);
  OBJHANDLE hWFC = oapiCreateVessel (inst3_name, "HST_EX/inst3", vs);
  VESSEL *v = oapiGetVesselInterface (hWFC);
  ATTACHMENTHANDLE hAttWFC = v->GetAttachmentHandle(true,0);
  AttachChild(hWFC, inst3_attach, hAttWFC);
}

void hst_ex::SpawnInst4(void)
{
VESSELSTATUS vs;
  GetStatus(vs);
  OBJHANDLE hWFC = oapiCreateVessel (inst4_name, "HST_EX/inst4", vs);
  VESSEL *v = oapiGetVesselInterface (hWFC);
  ATTACHMENTHANDLE hAttWFC = v->GetAttachmentHandle(true,0);
  AttachChild(hWFC, inst4_attach, hAttWFC);
}


void hst_ex::SpawnFGS1(void)
{
VESSELSTATUS vs;
  GetStatus(vs);
  OBJHANDLE hWFC = oapiCreateVessel ("FGS1", "HST_EX/fgs1", vs);
  VESSEL *v = oapiGetVesselInterface (hWFC);
  ATTACHMENTHANDLE hAttWFC = v->GetAttachmentHandle(true,0);
  AttachChild(hWFC, fgs1_attach, hAttWFC);
}


void hst_ex::SpawnFGS2(void)
{
VESSELSTATUS vs;
  GetStatus(vs);
  OBJHANDLE hWFC = oapiCreateVessel ("FGS2", "HST_EX/fgs2", vs);
  VESSEL *v = oapiGetVesselInterface (hWFC);
  ATTACHMENTHANDLE hAttWFC = v->GetAttachmentHandle(true,0);
  AttachChild(hWFC, fgs2_attach, hAttWFC);
}




void hst_ex::CheckVessels2(void)
{
if (cv1cv2_check == 2)cv1cv2_check = 3;
else if (cv1cv2_check == 3)cv1cv2_check = 0;

VECTOR3 pos = _V(0,0,0);
VECTOR3 dir = _V(0,0,0);
VECTOR3 rot = _V(0,0,0);
VECTOR3 att_glob = _V(0,0,0);

//// beacons
VECTOR3 hst_drp_glob[5];
for (int i = 24; i < 27; i++)
{
Local2Global(beaconpos[i], hst_drp_glob[(i-24)]);
}

// solps
for (int i = 4; i < 6; i++)
{
Local2Global(beaconpos[i], hst_drp_glob[(i-1)]);
}




///// grap points
VECTOR3 hst_attp_glob[5];
for (int i = 9; i < 14; i++)
{
Local2Global(attp_pos[i], hst_attp_glob[(i-9)]);
}


int num_ves = oapiGetVesselCount();
for (int i = 0; i < num_ves; i++) 
{
VECTOR3 hst_pos_glob = _V(0,0,0);
GetGlobalPos(hst_pos_glob);
OBJHANDLE hVessel = oapiGetVesselByIndex(i);
if (hVessel == GetHandle()) continue;

VECTOR3 ves_pos_glob = _V(0,0,0);
oapiGetGlobalPos(hVessel, &ves_pos_glob);
if (dist(ves_pos_glob, hst_pos_glob)> 20)continue;

VESSEL *v = oapiGetVesselInterface (hVessel);
DWORD nAttach;
if (att_pc_check == 0)nAttach = v->AttachmentCount (false);
else nAttach = v->AttachmentCount (true);

if (nAttach < 1)continue;


for (DWORD j = 0; j < nAttach; j++)
{
ATTACHMENTHANDLE hAtt;
if(att_pc_check == 0) hAtt = v->GetAttachmentHandle (false, j);
else hAtt = v->GetAttachmentHandle (true, j);
v->GetAttachmentParams (hAtt, pos, dir, rot);
v->Local2Global (pos, att_glob);



/////// rsu1
	if (dist(hst_drp_glob[0], att_glob) < 0.1)
	{
	if (rsu1_status == 0)
		{
		rsu1_status = 1;
		SetMeshVisibilityMode(mesh10,MESHVIS_NEVER);
		SpawnRSU1();
		anot_status = 1;
		return;
		}
	}

	

	if (rsu1_status == 1)
	{
		if (GetAttachmentStatus(rsu1_attach) == NULL)
		{
		rsu1_status = 2;
		return;
		}
	}



	else if (rsu1_status == 2)
	{
	if (dist(hst_attp_glob[0], att_glob) < 0.2)
		{
		const char *id = v->GetAttachmentId (hAtt);
		if (strncmp (id, "RSU", 3)); // attachment point not compatible
		else 
			{
			int atchk = 0;
			DWORD natt = v->AttachmentCount (true);
			for (DWORD k = 0; k < natt; k++)
			{
			ATTACHMENTHANDLE atth = v->GetAttachmentHandle (true, k);
			if (!v->GetAttachmentStatus(atth) == NULL) atchk = 1;
			}

			if (atchk == 0)
			{
			oapiDeleteVessel(hVessel);
			SetMeshVisibilityMode(mesh10,MESHVIS_EXTERNAL);
			rsu1_status = 0;
			anot2_status = 1;
			return;
			}

			}
		}
	}





/////// rsu2
	if (dist(hst_drp_glob[1], att_glob) < 0.1)
	{
	if (rsu2_status == 0)
		{
		rsu2_status = 1;
		SetMeshVisibilityMode(mesh11,MESHVIS_NEVER);
		SpawnRSU2();
		anot_status = 1;
		return;
		}
	}

	

	if (rsu2_status == 1)
	{
		if (GetAttachmentStatus(rsu2_attach) == NULL)
		{
		rsu2_status = 2;
		return;
		}
	}



	else if (rsu2_status == 2)
	{
	if (dist(hst_attp_glob[1], att_glob) < 0.2)
		{
		const char *id = v->GetAttachmentId (hAtt);
		if (strncmp (id, "RSU", 3)); // attachment point not compatible
		else 
			{
			int atchk = 0;
			DWORD natt = v->AttachmentCount (true);
			for (DWORD k = 0; k < natt; k++)
			{
			ATTACHMENTHANDLE atth = v->GetAttachmentHandle (true, k);
			if (!v->GetAttachmentStatus(atth) == NULL) atchk = 1;
			}

			if (atchk == 0)
			{
			oapiDeleteVessel(hVessel);
			SetMeshVisibilityMode(mesh11,MESHVIS_EXTERNAL);
			rsu2_status = 0;
			anot2_status = 1;
			return;
			}

			}
		}
	}



/////// rsu3
	if (dist(hst_drp_glob[2], att_glob) < 0.1)
	{
	if (rsu3_status == 0)
		{
		rsu3_status = 1;
		SetMeshVisibilityMode(mesh12,MESHVIS_NEVER);
		SpawnRSU3();
		anot_status = 1;
		return;
		}
	}

	

	if (rsu3_status == 1)
	{
		if (GetAttachmentStatus(rsu3_attach) == NULL)
		{
		rsu3_status = 2;
		return;
		}
	}



	else if (rsu3_status == 2)
	{
	if (dist(hst_attp_glob[2], att_glob) < 0.2)
		{
		const char *id = v->GetAttachmentId (hAtt);
		if (strncmp (id, "RSU", 3)); // attachment point not compatible
		else 
			{
			int atchk = 0;
			DWORD natt = v->AttachmentCount (true);
			for (DWORD k = 0; k < natt; k++)
			{
			ATTACHMENTHANDLE atth = v->GetAttachmentHandle (true, k);
			if (!v->GetAttachmentStatus(atth) == NULL) atchk = 1;
			}

			if (atchk == 0)
			{
			oapiDeleteVessel(hVessel);
			SetMeshVisibilityMode(mesh12,MESHVIS_EXTERNAL);
			rsu3_status = 0;
			anot2_status = 1;
			return;
			}

			}
		}
	}



	/////// stbd solar array
	int sts_chk = 1;
	if (sa_stbd_cfg == 0 && sa1_stbd_scale_sts == 2) sts_chk = 0;
	else if (sa_stbd_cfg == 1 && (sa1_stbd_scale_sts + sa1_stbd_rot_proc) == 0) sts_chk = 0;

	if (sa1_stbd_elb_sts == 0 && sts_chk == 0)
	{
	if (dist(hst_drp_glob[3], att_glob) < 0.1)
		{
	if (sa_stbd_status == 0)
			{
		sa_stbd_status = 1;
		if (sa_stbd_cfg == 0)
		{
		SetMeshVisibilityMode(mesh14,MESHVIS_NEVER);
		SpawnStbdSA();
		anot_status = 1;
		return;
		}
		else
		{
		SetMeshVisibilityMode(mesh16,MESHVIS_NEVER);
		SpawnStbdSA2();
		anot_status = 1;
		return;
		}
			}
		}

	

	if (sa_stbd_status == 1)
	{
		if (GetAttachmentStatus(sa_stbd_attach) == NULL)
		{
		sa_stbd_status = 2;
		return;
		}
	}



	else if (sa_stbd_status == 2)
	{
	if (dist(hst_attp_glob[3], att_glob) < 0.2)
		{
		const char *id = v->GetAttachmentId (hAtt);
		if (strncmp (id, "S1S", 3) == 0)
			{
			int atchk = 0;
			DWORD natt = v->AttachmentCount (true);
			for (DWORD k = 0; k < natt; k++)
			{
			ATTACHMENTHANDLE atth = v->GetAttachmentHandle (true, k);
			if (!v->GetAttachmentStatus(atth) == NULL) atchk = 1;
			}

			if (atchk == 0)
			{
			oapiDeleteVessel(hVessel);
			SetMeshVisibilityMode(mesh14,MESHVIS_EXTERNAL);
			sa_stbd_status = 0;
			sa_stbd_cfg = 0;
			anot2_status = 1;
			return;
			}

			}
		
		else if (strncmp (id, "S2S", 3) == 0)
			{
			int atchk = 0;
			DWORD natt = v->AttachmentCount (true);
			for (DWORD k = 0; k < natt; k++)
			{
			ATTACHMENTHANDLE atth = v->GetAttachmentHandle (true, k);
			if (!v->GetAttachmentStatus(atth) == NULL) atchk = 1;
			}

			if (atchk == 0)
			{
			oapiDeleteVessel(hVessel);
			SetMeshVisibilityMode(mesh16,MESHVIS_EXTERNAL);
			sa_stbd_status = 0;
			sa_stbd_cfg = 1;
			anot2_status = 1;
			return;
			}

			}

		}
	}

	}

	/////// Port solar array
	if (sa_port_cfg == 0 && sa1_port_scale_sts == 2) sts_chk = 0;
	else if (sa_port_cfg == 1 && (sa1_port_scale_sts + sa1_port_rot_proc) == 0) sts_chk = 0;
	else sts_chk = 1;

	if (sa1_port_elb_sts == 0 && sts_chk == 0)
	{
	if (dist(hst_drp_glob[4], att_glob) < 0.1)
	{
	if (sa_port_status == 0)
		{
		sa_port_status = 1;
		if(sa_port_cfg == 0)
		{
		SetMeshVisibilityMode(mesh13,MESHVIS_NEVER);
		SpawnPortSA();
		hst_cfg = 1;
		anot_status = 1;
		return;
		}
		else
		{
		SetMeshVisibilityMode(mesh15,MESHVIS_NEVER);
		SpawnPortSA2();
		anot_status = 1;
		return;
		}
		}
	}

	

	if (sa_port_status == 1)
	{
		if (GetAttachmentStatus(sa_port_attach) == NULL)
		{
		sa_port_status = 2;
		return;
		}
	}



	else if (sa_port_status == 2)
	{
	if (dist(hst_attp_glob[4], att_glob) < 0.2)
		{
		const char *id = v->GetAttachmentId (hAtt);
		if (strncmp (id, "S1P", 3) == 0)
			{
			int atchk = 0;
			DWORD natt = v->AttachmentCount (true);
			for (DWORD k = 0; k < natt; k++)
			{
			ATTACHMENTHANDLE atth = v->GetAttachmentHandle (true, k);
			if (!v->GetAttachmentStatus(atth) == NULL) atchk = 1;
			}

			if (atchk == 0)
			{
			oapiDeleteVessel(hVessel);
			SetMeshVisibilityMode(mesh13,MESHVIS_EXTERNAL);
			sa_port_status = 0;
			sa_port_cfg = 0;
			anot2_status = 1;
			return;
			}

			}

		else if (strncmp (id, "S2P", 3) == 0)
			{
			int atchk = 0;
			DWORD natt = v->AttachmentCount (true);
			for (DWORD k = 0; k < natt; k++)
			{
			ATTACHMENTHANDLE atth = v->GetAttachmentHandle (true, k);
			if (!v->GetAttachmentStatus(atth) == NULL) atchk = 1;
			}

			if (atchk == 0)
			{
			oapiDeleteVessel(hVessel);
			SetMeshVisibilityMode(mesh15,MESHVIS_EXTERNAL);
			sa_port_status = 0;
			sa_port_cfg = 1;
			anot2_status = 1;
			return;
			}

			}

		}
	}


}




}

}




}

void hst_ex::SpawnRSU1(void)
{
VESSELSTATUS vs;
  GetStatus(vs);
  OBJHANDLE hWFC = oapiCreateVessel ("RSU1", "HST_EX/rsu1", vs);
  VESSEL *v = oapiGetVesselInterface (hWFC);
  ATTACHMENTHANDLE hAttWFC = v->GetAttachmentHandle(true,0);
  AttachChild(hWFC, rsu1_attach, hAttWFC);
}

void hst_ex::SpawnRSU2(void)
{
VESSELSTATUS vs;
  GetStatus(vs);
  OBJHANDLE hWFC = oapiCreateVessel ("RSU2", "HST_EX/rsu2", vs);
  VESSEL *v = oapiGetVesselInterface (hWFC);
  ATTACHMENTHANDLE hAttWFC = v->GetAttachmentHandle(true,0);
  AttachChild(hWFC, rsu2_attach, hAttWFC);
}

void hst_ex::SpawnRSU3(void)
{
VESSELSTATUS vs;
  GetStatus(vs);
  OBJHANDLE hWFC = oapiCreateVessel ("RSU3", "HST_EX/rsu3", vs);
  VESSEL *v = oapiGetVesselInterface (hWFC);
  ATTACHMENTHANDLE hAttWFC = v->GetAttachmentHandle(true,0);
  AttachChild(hWFC, rsu3_attach, hAttWFC);
}

void hst_ex::SpawnStbdSA(void)
{
VESSELSTATUS vs;
  GetStatus(vs);
  OBJHANDLE hWFC = oapiCreateVessel ("HSTArray_Stbd", "HST_EX/sa1_stbd", vs);
  VESSEL *v = oapiGetVesselInterface (hWFC);
  ATTACHMENTHANDLE hAttWFC = v->GetAttachmentHandle(true,0);
  AttachChild(hWFC, sa_stbd_attach, hAttWFC);
}

void hst_ex::SpawnStbdSA2(void)
{
VESSELSTATUS vs;
  GetStatus(vs);
  OBJHANDLE hWFC = oapiCreateVessel ("HSTArray_Stbd", "HST_EX/sa2_stbd", vs);
  VESSEL *v = oapiGetVesselInterface (hWFC);
  ATTACHMENTHANDLE hAttWFC = v->GetAttachmentHandle(true,0);
  AttachChild(hWFC, sa_stbd_attach, hAttWFC);
}


void hst_ex::SpawnPortSA(void)
{
VESSELSTATUS vs;
  GetStatus(vs);
  OBJHANDLE hWFC;
  if (sa1_port_scale_proc == 0.5)
  {
	hWFC = oapiCreateVessel ("HSTArray_Port", "HST_EX/sa1_port_50", vs);
	sa1_port_scale_proc = 1;
	SetAnimation (anim_sa1_port_scale, sa1_port_scale_proc);
  }
  else hWFC = oapiCreateVessel ("HSTArray_Port", "HST_EX/sa1_port", vs);
  VESSEL *v = oapiGetVesselInterface (hWFC);
  ATTACHMENTHANDLE hAttWFC = v->GetAttachmentHandle(true,0);
  AttachChild(hWFC, sa_port_attach, hAttWFC);
}


void hst_ex::SpawnPortSA2(void)
{
VESSELSTATUS vs;
  GetStatus(vs);
  OBJHANDLE hWFC;
  hWFC = oapiCreateVessel ("HSTArray_Port", "HST_EX/sa2_port", vs);
  VESSEL *v = oapiGetVesselInterface (hWFC);
  ATTACHMENTHANDLE hAttWFC = v->GetAttachmentHandle(true,0);
  AttachChild(hWFC, sa_port_attach, hAttWFC);
}


void hst_ex::DoAutoSun(void)
{
	
	char cbuf[7];
	strcpy(cbuf, "Sun");
	OBJHANDLE hSun = oapiGetObjectByName(cbuf);
	VECTOR3 sun_pos_glob;
	oapiGetGlobalPos(hSun, &sun_pos_glob);
	VECTOR3 sunpos_relves;
	Global2Local(sun_pos_glob,sunpos_relves);
	double sun_angle = atan2(-sunpos_relves.y, sunpos_relves.z);
	sun_angle = sun_angle/(2*PI);
	if (sun_angle < 0)sun_angle = sun_angle + 1;

	double panel_angle_diff = sun_angle - sa1_stbd_rot_proc;
	int rotmode = 1;
	if (panel_angle_diff < 0) 
	{
		rotmode = -1;
		panel_angle_diff = -panel_angle_diff;
	}
	
	double da = step_time * 0.017;
	
	if (sa_stbd_status == 0)
	{
	if (panel_angle_diff < da)
	{
	sa1_stbd_rot_proc = sun_angle;
	}
	else if (panel_angle_diff < 0.5)sa1_stbd_rot_proc = sa1_stbd_rot_proc+(da*rotmode);
	else sa1_stbd_rot_proc = sa1_stbd_rot_proc-(da*rotmode);

	if (sa1_stbd_rot_proc > 1) sa1_stbd_rot_proc = sa1_stbd_rot_proc - ((int) (sa1_stbd_rot_proc));
	else if (sa1_stbd_rot_proc < 0) sa1_stbd_rot_proc = sa1_stbd_rot_proc - ((int) (sa1_stbd_rot_proc)) + 1;
	}
	else sa1_stbd_rot_proc = 0;



	panel_angle_diff = sun_angle - sa1_port_rot_proc;
	rotmode = 1;
	if (panel_angle_diff < 0) 
	{
		rotmode = -1;
		panel_angle_diff = -panel_angle_diff;
	}
	
	if (sa_port_status == 0)
	{
	if (panel_angle_diff < da)
	{
	sa1_port_rot_proc = sun_angle;
	}
	else if (panel_angle_diff < 0.5)sa1_port_rot_proc = sa1_port_rot_proc+(da*rotmode);
	else sa1_port_rot_proc = sa1_port_rot_proc-(da*rotmode);

	if (sa1_port_rot_proc > 1) sa1_port_rot_proc = sa1_port_rot_proc - ((int) (sa1_port_rot_proc));
	else if (sa1_port_rot_proc < 0) sa1_port_rot_proc = sa1_port_rot_proc - ((int) (sa1_port_rot_proc)) + 1;
	}
	else sa1_port_rot_proc = 0;

	 
	

}

void hst_ex::DoRevertRot(void)
{
double da = step_time * 0.017;

if (sa1_stbd_rot_proc > 0.5) sa1_stbd_rot_proc = sa1_stbd_rot_proc + da;
else sa1_stbd_rot_proc = sa1_stbd_rot_proc - da;

if (sa1_stbd_rot_proc > 1) sa1_stbd_rot_proc = 0;
else if (sa1_stbd_rot_proc < 0) sa1_stbd_rot_proc = 0;

if (sa1_stbd_rot_proc == 0)
{
if (sa1_stbd_scale_sts == 0)sa1_stbd_scale_sts = 1;
if (sa1_port_scale_sts == 0)sa1_port_scale_sts = 1;
autosun_status = 0;
}

sa1_port_rot_proc = sa1_stbd_rot_proc;

if (sa_stbd_status > 0) sa1_stbd_rot_proc = 0;
if (sa_port_status > 0) sa1_port_rot_proc = 0;


}