// ==============================================================
//                 ORBITER MODULE: hst_ex_ev1
//
// hst_ex_ev1.cpp
// Control module for hst_ex_ev1 vessel class
//
// ==============================================================

#define STRICT
#define ORBITER_MODULE

#include "orbitersdk.h"
#include "OrbiterSoundSDK40.h"

// ==============================================================
// Some vessel parameters
// ==============================================================

static UINT LampGrp[2];
static VECTOR3 beacon_col = _V(1,1,1);
static VECTOR3 beacon_pos = _V(-0.189,0.6903,0.1575);

#define BRTH	1

class hst_ex_ev1: public VESSEL3 {
public:
	hst_ex_ev1 (OBJHANDLE hVessel, int flightmodel)
		: VESSEL3 (hVessel, flightmodel) {}

	~hst_ex_ev1();
	void clbkSetClassCaps (FILEHANDLE cfg);
	void clbkPreStep(double SimT, double SimDT, double MJD);
	void clbkPostStep (double simt, double simdt, double mjd);
	void clbkPostCreation();
	void clbkSaveState (FILEHANDLE scn);
	void clbkLoadStateEx (FILEHANDLE scn, void *status);
	int clbkConsumeBufferedKey(DWORD key, bool down, char *kstate);
	int clbkConsumeDirectKey(char *kstate);
	bool clbkDrawHUD (int mode, const HUDPAINTSPEC *hps, oapi::Sketchpad *skp);
	void clbkVisualCreated (VISHANDLE vis, int refcount);
	bool clbkLoadGenericCockpit ();
	bool clbkLoadVC (int id);
	int MyID;	// this is your unique Id, it will identify your ship in OrbiterSound.
	

private:
	void DefineAnimations (void);
	int light_status;
	SpotLight *spotlight1;
	BEACONLIGHTSPEC spot_beacon;


	void DoIncAttX(void);
	void DoDecAttX(void);
	void DoIncAttY(void);
	void DoDecAttY(void);
	void DoIncAttZ(void);
	void DoDecAttZ(void);

	void DoAttDirX(void);
	void DoAttDirZ(void);
	void DoDecAttDirX(void);
	void DoDecAttDirZ(void);
	void DoIncAttRot(void);
	void DoDecAttRot(void);

	void DoIncLampAz(void);
	void DoDecLampAz(void);
	void DoIncLampAlt(void);
	void DoDecLampAlt(void);

	void RotHandUp(void);
	void RotHandDown(void);

	void TwistRight(void);
	void TwistLeft(void);

	void SetParentAtt(void);
	void GetParentAtt(void);

	void SetBeaconLightFOV(void);
	void Grapple(void);
	void EndEVA(void);
	void DoHolder(void);

	ATTACHMENTHANDLE att1, att2;
	ANIMATIONCOMPONENT_HANDLE hAnimComp, hAnimComp2, hAnimComp3;

	VECTOR3 att_pos, att_dir, att_rot;
	VECTOR3 scn_att_pos, scn_att_dir, scn_att_rot;
	VECTOR3 new_beacon_pos;
	VECTOR3 arm_tip[4];
	VECTOR3 los[3];

	double delta_pos, delta_angle;
	double waist_proc, arm_proc, hand_proc, twist_proc;

	MESHHANDLE mesh1, mesh2;

	VECTOR3 current_x, current_y, current_z;

	ANIMATIONCOMPONENT_HANDLE LightAnimComp[3];
	MGROUP_TRANSFORM *LightMgrpTran[7];
	UINT anim_waist, anim_arm, anim_hand, anim_att, anim_twist, anim_los, anim_hldr;

	int scn_sw;
	int move_mode;
	int hldr_sw;
	int sound_status;
	double life_support;
	int cam_status;
};
// Define animation sequences for moving parts
// --------------------------------------------------------------



// ==============================================================
// hst_ex_ev1 class interface
// ==============================================================

// ==============================================================
// Overloaded callback functions
// ==============================================================

// --------------------------------------------------------------
// Set the capabilities of the vessel class
// --------------------------------------------------------------
void hst_ex_ev1::clbkSetClassCaps (FILEHANDLE cfg)
{



	att_pos = _V(0,0,0);
	att_dir = _V(0,1,0);
	att_rot = _V(0,0,1);

	scn_att_pos = _V(0,0,0);
	scn_att_dir = _V(0,0,0);
	scn_att_rot = _V(0,0,0);

	arm_tip[0] = _V(0.2223,-0.2097,0.36);
	arm_tip[1] = _V(0.2223,-0.9168,1.0671);
	arm_tip[2] = _V(0.2223,-0.2097,1.7742);
	arm_tip[3] = _V(0.156,-0.352,0.552);

	los[0] = _V(0,0.6,0);
	los[1] = _V(0,0.6,1);
	los[2] = _V(-0.189,0.6903,0.1575);


	delta_pos = 0;
	delta_angle = 0;

	waist_proc = 0;
	arm_proc = 0;
	hand_proc = 0.5;
	twist_proc = 0.5;

	current_x = _V(1,0,0);
	current_y = _V(0,1,0);
	current_z = _V(0,0,1);
	new_beacon_pos = _V(-0.189,0.6903,0.1575);

	scn_sw = 0;
	light_status = 0;
	move_mode = 0;
	hldr_sw = 0;

	sound_status = 0;
	life_support = 28800;
	cam_status = 0;

	SetSize(2);
	SetEmptyMass(100);
	SetEnableFocus(true);
	SetTouchdownPoints(_V(0,-1.2, 1), _V(-1,-1.2,-1), _V(1,-1.2,-1));
	SetPMI(_V(2,1,2));
	SetCrossSections(_V(0.5, 0.25, 1));
	SetCW(1,1,1,1);
	SetRotDrag(_V(1,1,1));
	SetCameraOffset(_V(0,0.6,0));




		spot_beacon.shape = BEACONSHAPE_STAR;
		spot_beacon.pos = &beacon_pos;
		spot_beacon.col = &beacon_col;
		spot_beacon.size = 0.30;
		spot_beacon.falloff = 0.3;
		spot_beacon.period = 0;
		spot_beacon.duration = 0;
		spot_beacon.tofs = 0;
		spot_beacon.active = false;
		AddBeacon (&spot_beacon);



	COLOUR4 col_d = {1,1,0.9,0};
	COLOUR4 col_s = {1,1,0.9,0};
	COLOUR4 col_a = {0,0,0,0};
	
	spotlight1 = (SpotLight*)AddSpotLight(beacon_pos, _V(0,0,1),100, 1, 0, 5e-5, (45*RAD), (60*RAD), col_d, col_s, col_a);
	spotlight1->Activate(false);

	SetVisibilityLimit(1e-4, -1);


	att1 = CreateAttachment(true,_V(0,0,0),_V(0,-1,0),_V(0,0,1),"GS",false);
	att2 = CreateAttachment(false,arm_tip[0],arm_tip[1]-arm_tip[0],arm_tip[2]-arm_tip[1],"GS",true);


	mesh1 = oapiLoadMesh ("HST_EX/hst_astro");
	AddMesh(mesh1);
	SetMeshVisibilityMode (0, MESHVIS_ALWAYS);
	mesh2 = oapiLoadMesh ("HST_EX/hst_astro_hldr");
	AddMesh(mesh2);
	SetMeshVisibilityMode (1, MESHVIS_NEVER);

	DefineAnimations();

}
void hst_ex_ev1::clbkVisualCreated (VISHANDLE vis, int refcount)
{

}

bool hst_ex_ev1::clbkLoadGenericCockpit ()
{
SetBeaconLightFOV();
return true;
}

bool hst_ex_ev1::clbkLoadVC (int id)
{
SetBeaconLightFOV();
return true;
}

void hst_ex_ev1::DefineAnimations (void)
{

anim_twist = CreateAnimation (0.5);
anim_waist = CreateAnimation (0);
anim_los = CreateAnimation (0);
anim_arm = CreateAnimation (0);
anim_hand = CreateAnimation (0.5);
anim_att = CreateAnimation (0.5);
anim_hldr = CreateAnimation (0.5);

///// 0. twist
static UINT LampGrp0[1] = {0};
LightMgrpTran[5] = new MGROUP_ROTATE(0,LampGrp0,0,_V(0,-0.111,-0.02),_V(0,0.9848,0.1736), (float) (60*RAD));
hAnimComp3 = AddAnimationComponent (anim_twist, 0, 1, LightMgrpTran[5]);


////// 1. waist

static UINT LampGrp1[2] = {3,4};
LightMgrpTran[0] = new MGROUP_ROTATE(0,LampGrp1,2,_V(0,-0.22176,-0.16632),_V(1,0,0), (float) (30*RAD));
hAnimComp = AddAnimationComponent (anim_waist, 0, 1, LightMgrpTran[0],hAnimComp3);

///// 2 arm
static UINT LampGrp2[2] = {1,5};
LightMgrpTran[1] = new MGROUP_ROTATE(0,LampGrp2,2,_V(0.24156,0.32076,0.04752),_V(-1,0,0), (float) (90*RAD));
hAnimComp2 = AddAnimationComponent (anim_arm, 0, 1, LightMgrpTran[1], hAnimComp);

//// 3 hand
static UINT LampGrp3[1] = {2};
LightMgrpTran[2] = new MGROUP_ROTATE(0,LampGrp3,1,_V(0.3114,-0.0747,0.2268),_V(-0.025,-0.64944,0.76), (float) (180*RAD));
AddAnimationComponent (anim_hand, 0, 1, LightMgrpTran[2], hAnimComp2);

//// 4 att point
LightMgrpTran[3] = new MGROUP_ROTATE (LOCALVERTEXLIST, MAKEGROUPARRAY(arm_tip), 4,_V(0.3114,-0.0747,0.2268), _V(-0.025,-0.64944,0.76), (float)(180*RAD));
AddAnimationComponent (anim_att, 0, 1, LightMgrpTran[3], hAnimComp2);

//// 5 line of sight
LightMgrpTran[4] = new MGROUP_ROTATE (LOCALVERTEXLIST, MAKEGROUPARRAY(los), 3,_V(0,-0.22176,-0.16632),_V(1,0,0), (float) (30*RAD));
AddAnimationComponent (anim_los, 0, 1, LightMgrpTran[4], hAnimComp3);

//// 6 hldr
static UINT LampGrp4[1] = {0};
LightMgrpTran[6] = new MGROUP_ROTATE (1,LampGrp4,1,_V(0.3114,-0.0747,0.2268), _V(-0.025,-0.64944,0.76), (float)(180*RAD));
AddAnimationComponent (anim_hldr, 0, 1, LightMgrpTran[6], hAnimComp2);

}


hst_ex_ev1::~hst_ex_ev1 ()
{
int i;
for (i = 0; i < 7; i++)
	{
	delete LightMgrpTran[i];
	}

}

// ==============================================================
// API callback interface
// ==============================================================

// --------------------------------------------------------------
// Vessel initialisation
// --------------------------------------------------------------
DLLCLBK VESSEL *ovcInit (OBJHANDLE hvessel, int flightmodel)
{
	return new hst_ex_ev1 (hvessel, flightmodel);
}
// --------------------------------------------------------------
// Vessel cleanup
// --------------------------------------------------------------
DLLCLBK void ovcExit (VESSEL *vessel)
{
	if (vessel) delete (hst_ex_ev1*)vessel;
}
// ==============================================================
// Overloaded clbkPostCreation callback functions
// ==============================================================

void hst_ex_ev1::clbkPostCreation ()
{
if(light_status == 1)
	{
	spot_beacon.active = true;
	spotlight1->Activate(true);
	}

if (hldr_sw == 1)SetMeshVisibilityMode (1, MESHVIS_ALWAYS);

///////////// sound
	MyID=ConnectToOrbiterSoundDLL(GetHandle());
	RequestLoadVesselWave(MyID,BRTH,"Sound\\_CustomVesselsSounds\\HST_EX\\breathing.wav",INTERNAL_ONLY);
	SoundOptionOnOff(MyID,PLAYCABINAIRCONDITIONING,FALSE);
	SoundOptionOnOff(MyID,PLAYCABINRANDOMAMBIANCE,FALSE);

	//SetBeaconLightFOV();
}



void hst_ex_ev1::clbkSaveState (FILEHANDLE scn)
{
	char cbuf[256];

	GetParentAtt();

	// default vessel parameters
	VESSEL3::clbkSaveState (scn);

	// custom parameters

	sprintf (cbuf, "%0.4f %0.4f %0.4f %0.4f %0.4f %0.4f %0.4f %0.4f %0.4f", scn_att_pos.x, scn_att_pos.y, scn_att_pos.z, scn_att_dir.x, scn_att_dir.y, scn_att_dir.z, scn_att_rot.x, scn_att_rot.y, scn_att_rot.z);
	oapiWriteScenario_string (scn, "PAT", cbuf);

	//sprintf (cbuf, "%0.4f %0.4f %0.4f %0.4f %0.4f %0.4f %0.4f %0.4f %0.4f", los[0].x, los[0].y, los[0].z, los[1].x, los[1].y, los[1].z, los[2].x, los[2].y, los[2].z);
	//oapiWriteScenario_string (scn, "LOS", cbuf);

	sprintf (cbuf, "%0.4f %0.4f %0.4f %0.4f", hand_proc, arm_proc, waist_proc, twist_proc);
	oapiWriteScenario_string (scn, "ANM", cbuf);

	sprintf (cbuf, "%d %d %d %d %0.4f", light_status, move_mode, hldr_sw, sound_status, life_support);
	oapiWriteScenario_string (scn, "TCH", cbuf);


}
void hst_ex_ev1::clbkLoadStateEx (FILEHANDLE scn, void *vs)
{
    char *line;
	while (oapiReadScenario_nextline (scn, line)) {
		if (!strnicmp (line, "PAT", 3)) {
			sscanf (line+3, "%lf%lf%lf%lf%lf%lf%lf%lf%lf", &scn_att_pos.x, &scn_att_pos.y, &scn_att_pos.z, &scn_att_dir.x, &scn_att_dir.y, &scn_att_dir.z, &scn_att_rot.x, &scn_att_rot.y, &scn_att_rot.z);
		//}else if (!strnicmp (line, "LOS", 3)) {
		//	sscanf (line+3, "%lf%lf%lf%lf%lf%lf%lf%lf%lf", &scn_los[0].x, &scn_los[0].y, &scn_los[0].z, &scn_los[1].x, &scn_los[1].y, &scn_los[1].z, &scn_los[2].x, &scn_los[2].y, &scn_los[2].z);
		}else if (!strnicmp (line, "ANM", 3)) {
			sscanf (line+3, "%lf%lf%lf%lf", &hand_proc, &arm_proc, &waist_proc, &twist_proc);
		}else if (!strnicmp (line, "TCH", 3)) {
			sscanf (line+3, "%d%d%d%d%lf", &light_status, &move_mode, &hldr_sw, &sound_status, &life_support);
		}else {
            ParseScenarioLineEx (line, vs);
			// unrecognised option - pass to Orbiter's generic parser
        }
   }
	SetAnimation(anim_twist, twist_proc);
	SetAnimation(anim_waist, waist_proc);
	SetAnimation(anim_los,waist_proc);
	SetAnimation(anim_arm, arm_proc);
	SetAnimation(anim_hand, hand_proc);
	SetAnimation(anim_att, hand_proc);
	SetAnimation(anim_hldr, hand_proc);
	//SetBeaconLightFOV();

	if (light_status == 1)
	{
	spotlight1->Activate(true);
	spot_beacon.active = true;
	}
	else
	{
	spotlight1->Activate(false);
	spot_beacon.active = false;
	}


}

int hst_ex_ev1::clbkConsumeDirectKey(char *kstate)
{

	
if (KEYMOD_ALT (kstate)) 
{
 return 0;
}
else if (KEYMOD_CONTROL (kstate)) 
{
	return 0;
}
else if (KEYMOD_SHIFT (kstate)) 
{

if (KEYDOWN (kstate, OAPI_KEY_DOWN))
	{
	DoIncLampAlt();
	}
else if (KEYDOWN (kstate, OAPI_KEY_UP))
	{
	DoDecLampAlt();
	}

if (KEYDOWN (kstate, OAPI_KEY_RIGHT))
	{
	TwistRight();
	}
else if (KEYDOWN (kstate, OAPI_KEY_LEFT))
	{
	TwistLeft();
	}

}


else
{

if (KEYDOWN (kstate, OAPI_KEY_NUMPAD1))
	{
	if (move_mode == 0)DoDecAttX();
	else DoDecAttDirZ();
	}
if (KEYDOWN (kstate, OAPI_KEY_NUMPAD2))
	{
	if (move_mode == 0) DoIncAttY();
	else DoAttDirX();
	}
if (KEYDOWN (kstate, OAPI_KEY_NUMPAD3))
	{
	if (move_mode == 0)DoIncAttX();
	else DoAttDirZ();
	}

if (KEYDOWN (kstate, OAPI_KEY_NUMPAD4))
	{
	if (move_mode == 1)DoIncAttRot();
	}

if (KEYDOWN (kstate, OAPI_KEY_NUMPAD6))
	{
	if (move_mode == 0)DoIncAttZ();
	else DoDecAttRot();
	}

if (KEYDOWN (kstate, OAPI_KEY_NUMPAD8))
	{
	if (move_mode == 0) DoDecAttY();
	else DoDecAttDirX();
	}

if (KEYDOWN (kstate, OAPI_KEY_NUMPAD9))
	{
	if (move_mode == 0)DoDecAttZ();
	}

if (KEYDOWN (kstate, OAPI_KEY_RIGHT))
	{
	RotHandUp();
	}
else if (KEYDOWN (kstate, OAPI_KEY_LEFT))
	{
	RotHandDown();
	}


if (KEYDOWN (kstate, OAPI_KEY_UP))
	{
	DoIncLampAz();
	}
else if (KEYDOWN (kstate, OAPI_KEY_DOWN))
	{
	DoDecLampAz();
	}

}

return 0;
}


int hst_ex_ev1::clbkConsumeBufferedKey(DWORD key, bool down, char *kstate) {


  if (!down) return 0; // only process keydown events

  if (KEYMOD_ALT (kstate)) 
   {
	   return 0;
   }
  else if (KEYMOD_CONTROL (kstate)) 
   {
	   return 0;
   }
  else if (KEYMOD_SHIFT (kstate)) {
	  switch (key)
	  {
		
	  }
	  
  } else { // unmodified keys
    switch (key) {
	case OAPI_KEY_B: // spotlight on/off

		if(light_status == 0)
		{
			spotlight1->Activate(true);
			spot_beacon.active = true;
			light_status = 1;
		}

		else if(light_status == 1)
		{
			spotlight1->Activate(false);
			spot_beacon.active = false;
			light_status = 0;
		}
		return 1;

		case OAPI_KEY_G: /// grasp / release
		Grapple();
		return 1;

		case OAPI_KEY_DIVIDE: /// rot lin switch
		if(move_mode == 0)move_mode = 1;
		else if (move_mode == 1)move_mode = 0;
		return 1;

		case OAPI_KEY_V: /// exit
		EndEVA();
		return 1;

		case OAPI_KEY_N: /// release holder
		DoHolder();
		return 1;

		case OAPI_KEY_F5: // show att points
		if (oapiGetShowGrapplePoints () == true) oapiSetShowGrapplePoints (false);
		else oapiSetShowGrapplePoints (true);
		return 1;

		case OAPI_KEY_M: /// sound on off
		if(sound_status == 0)sound_status = 1;
		else if(sound_status == 1)sound_status = 0;
		return 1;

	}
  }
  return 0;
}





void hst_ex_ev1::clbkPreStep(double SimT, double SimDT, double MJD) 
{
	delta_pos = 0.5*SimDT;
	delta_angle = 0.05*SimDT;

	if (scn_sw == 0)
	{
	//sprintf(oapiDebugString(), "scn_att_dir %0.4f %0.4f %0.4f", scn_att_dir.x, scn_att_dir.y, scn_att_dir.z);
	if(length(scn_att_dir) > 0 && length(scn_att_rot) > 0)SetParentAtt();
	scn_sw = 1;
	}

	if (hldr_sw == 0)SetAttachmentParams(att2, arm_tip[0], arm_tip[1]-arm_tip[0], arm_tip[2]-arm_tip[1]);
	else SetAttachmentParams(att2, arm_tip[3], arm_tip[1]-arm_tip[0], arm_tip[2]-arm_tip[1]);


	if(sound_status == 0)PlayVesselWave(MyID, BRTH, LOOP, 130, 0);
	else StopVesselWave(MyID,BRTH);

	life_support = life_support - SimDT;
	if(life_support < 0)oapiDeleteVessel(GetHandle());

	//if(oapiCameraMode() == CAM_COCKPIT)SetBeaconLightFOV();
	if(cam_status == 1)
	{
		SetBeaconLightFOV();
		cam_status = 0;
	}
}


void hst_ex_ev1::clbkPostStep (double simt, double simdt, double mjd)
{
	if (scn_sw == 1)
	{
		SetBeaconLightFOV();
		scn_sw = 2;
	}
	else if (scn_sw == 2)
	{
		SetBeaconLightFOV();
		scn_sw = 3;
	}
	
}


bool hst_ex_ev1::clbkDrawHUD (int mode, const HUDPAINTSPEC *hps, oapi::Sketchpad *skp)
{
	// draw the default HUD
	VESSEL3::clbkDrawHUD (mode, hps, skp);

	oapiSetDefNavDisplay(0);
	oapiSetDefRCSDisplay(0);

	int s = hps->H;
	double d = (s*0.00130208);
	int sw = ((hps->W));
	int lw =  (int)(16*sw/1024);
	int lwoffset = sw - (18*lw);
	int hlw = (int) (lw/2);


	int roxl = 0;
	int royl = 0;

	double ds = s;
	double dsw = sw;
	double sc_ratio = ds/dsw;


	if (sc_ratio < 0.7284)
	{
	roxl = (lw*10);
	royl = (int) (-88*d);
	}



	int wd = (int)(152*d);
	int wc = (int)(168*d);
	int w0 = (int)(184*d);
	int w1 = (int) (200*d);
	int w2 = (int)(216*d);
	int w3 = (int)(232*d);
	int w4 = (int)(248*d);
	int w5 = (int)(264*d);
	int w6 = (int)(280*d);
	int w7 = (int)(296*d);
	int w8 = (int)(312*d);
	int w9 = (int)(328*d);
	int w10 = (int)(344*d);
	int w11 = (int)(360*d);
	int w12 = (int)(376*d);
	int w13 = (int)(392*d);
	int w14 = (int)(408*d);
	int w15 = (int)(424*d);


	{
	skp->SetTextColor(0x0066FF66);
	char abuf[256];
	sprintf(abuf,"[B] = Spotlight On/Off");
	skp->Text ((10+roxl), (wc+royl), abuf,strlen(abuf));
	}

	

	if (GetAttachmentStatus(att2) == NULL)
	{
	skp->SetTextColor(0x0066FF66);
	char abuf[256];
	sprintf(abuf,"[G] = Grapple");
	skp->Text ((10+roxl), (w0+royl), abuf,strlen(abuf));
	}
	else
	{
	skp->SetTextColor(0x0066FF66);
	char abuf[256];
	sprintf(abuf,"[G] = Release");
	skp->Text ((10+roxl), (w0+royl), abuf,strlen(abuf));
	}

	if (move_mode == 0)
	{
	skp->SetTextColor(0x0066FF66);
	char abuf[256];
	sprintf(abuf,"Rot/Lin Mode = Linear");
	skp->Text ((10+roxl), (w1+royl), abuf,strlen(abuf));
	}
	else
	{
	skp->SetTextColor(0x0066FF66);
	char abuf[256];
	sprintf(abuf,"Rot/Lin Mode = Rotation");
	skp->Text ((10+roxl), (w1+royl), abuf,strlen(abuf));
	}

	if (hldr_sw == 1)
	{
	skp->SetTextColor(0x0066FF66);
	char abuf[256];
	sprintf(abuf,"[N] = Release Holder");
	skp->Text ((10+roxl), (w2+royl), abuf,strlen(abuf));
	}

	{
	skp->SetTextColor(0x0066FF66);
	char abuf[256];
	sprintf(abuf,"[V] = End EVA(at Airlock)");
	skp->Text ((10+roxl), (w4+royl), abuf,strlen(abuf));
	}

	{
	skp->SetTextColor(0x0066FF66);
	char abuf[256];
	sprintf(abuf,"[F5] = Show/Hide Attachment Points");
	skp->Text ((10+roxl), (w3+royl), abuf,strlen(abuf));
	}

	{
	skp->SetTextColor(0x0066FF66);
	char abuf[256];
	sprintf(abuf,"[M] = Sound On/Off");
	skp->Text ((10+roxl), (w5+royl), abuf,strlen(abuf));
	}

	{
	int ls_hrs = (int) (life_support/3600);
	int ls_min = (int) ((life_support - (ls_hrs*3600))/60);
	int total_hrs_min = (ls_hrs*3600)+(ls_min*60);
	int ls_sec = (int) (life_support - total_hrs_min);

	skp->SetTextColor(0x0066FF66);
	if(life_support < 300) skp->SetTextColor(0x006666FF);
	else if(life_support < 600) skp->SetTextColor(0x0066FFFF);
	char abuf[256];
	sprintf(abuf,"Life Support: %dhrs %dmin %dsec", ls_hrs, ls_min, ls_sec);
	skp->Text ((10+roxl), (w6+royl), abuf,strlen(abuf));
	}

	return true;
}

void hst_ex_ev1::DoIncAttX()
{
OBJHANDLE hPVes = GetAttachmentStatus(att1);
if (hPVes)
	{
	VESSEL *v = oapiGetVesselInterface (hPVes);

			DWORD att_count = v->AttachmentCount (false);
			for (int i = 0; i < att_count; i++)
			{
			ATTACHMENTHANDLE hParentAttach = v->GetAttachmentHandle(false, i);
			OBJHANDLE hChildVessel = v->GetAttachmentStatus(hParentAttach);
			if (hChildVessel == GetHandle()) 
				{
				VECTOR3 pves_x;
				VECTOR3 glob_x;
				VECTOR3 pves_pos;
				v->GetGlobalPos(pves_pos);
				GlobalRot(_V(1,0,0),glob_x);
				glob_x = glob_x + pves_pos;
				v->Global2Local(glob_x,pves_x);


				v->GetAttachmentParams(hParentAttach,att_pos,att_dir,att_rot);
				att_pos.x = att_pos.x + pves_x.x*delta_pos;
				att_pos.y = att_pos.y + pves_x.y*delta_pos;
				att_pos.z = att_pos.z + pves_x.z*delta_pos;
				v->SetAttachmentParams(hParentAttach,att_pos,att_dir,att_rot);
				return;
				}
			}

	}
}
void hst_ex_ev1::DoDecAttX()
{
OBJHANDLE hPVes = GetAttachmentStatus(att1);
if (hPVes)
	{
	VESSEL *v = oapiGetVesselInterface (hPVes);

			DWORD att_count = v->AttachmentCount (false);
			for (int i = 0; i < att_count; i++)
			{
			ATTACHMENTHANDLE hParentAttach = v->GetAttachmentHandle(false, i);
			OBJHANDLE hChildVessel = v->GetAttachmentStatus(hParentAttach);
			if (hChildVessel == GetHandle()) 
				{	
				VECTOR3 pves_x;
				VECTOR3 glob_x;
				VECTOR3 pves_pos;
				v->GetGlobalPos(pves_pos);
				GlobalRot(_V(-1,0,0),glob_x);
				glob_x = glob_x + pves_pos;
				v->Global2Local(glob_x,pves_x);


				v->GetAttachmentParams(hParentAttach,att_pos,att_dir,att_rot);
				att_pos.x = att_pos.x + pves_x.x*delta_pos;
				att_pos.y = att_pos.y + pves_x.y*delta_pos;
				att_pos.z = att_pos.z + pves_x.z*delta_pos;
				v->SetAttachmentParams(hParentAttach,att_pos,att_dir,att_rot);
				return;
				}
			}

	}
}

void hst_ex_ev1::DoIncAttY()
{
OBJHANDLE hPVes = GetAttachmentStatus(att1);
if (hPVes)
	{
	VESSEL *v = oapiGetVesselInterface (hPVes);

			DWORD att_count = v->AttachmentCount (false);
			for (int i = 0; i < att_count; i++)
			{
			ATTACHMENTHANDLE hParentAttach = v->GetAttachmentHandle(false, i);
			OBJHANDLE hChildVessel = v->GetAttachmentStatus(hParentAttach);
			if (hChildVessel == GetHandle()) 
				{	
				VECTOR3 pves_x;
				VECTOR3 glob_x;
				VECTOR3 pves_pos;
				v->GetGlobalPos(pves_pos);
				GlobalRot(_V(0,1,0),glob_x);
				glob_x = glob_x + pves_pos;
				v->Global2Local(glob_x,pves_x);


				v->GetAttachmentParams(hParentAttach,att_pos,att_dir,att_rot);
				att_pos.x = att_pos.x + pves_x.x*delta_pos;
				att_pos.y = att_pos.y + pves_x.y*delta_pos;
				att_pos.z = att_pos.z + pves_x.z*delta_pos;
				v->SetAttachmentParams(hParentAttach,att_pos,att_dir,att_rot);
				return;
				}
			}

	}
}
void hst_ex_ev1::DoDecAttY()
{
OBJHANDLE hPVes = GetAttachmentStatus(att1);
if (hPVes)
	{
	VESSEL *v = oapiGetVesselInterface (hPVes);

			DWORD att_count = v->AttachmentCount (false);
			for (int i = 0; i < att_count; i++)
			{
			ATTACHMENTHANDLE hParentAttach = v->GetAttachmentHandle(false, i);
			OBJHANDLE hChildVessel = v->GetAttachmentStatus(hParentAttach);
			if (hChildVessel == GetHandle()) 
				{	
				VECTOR3 pves_x;
				VECTOR3 glob_x;
				VECTOR3 pves_pos;
				v->GetGlobalPos(pves_pos);
				GlobalRot(_V(0,-1,0),glob_x);
				glob_x = glob_x + pves_pos;
				v->Global2Local(glob_x,pves_x);


				v->GetAttachmentParams(hParentAttach,att_pos,att_dir,att_rot);
				att_pos.x = att_pos.x + pves_x.x*delta_pos;
				att_pos.y = att_pos.y + pves_x.y*delta_pos;
				att_pos.z = att_pos.z + pves_x.z*delta_pos;
				v->SetAttachmentParams(hParentAttach,att_pos,att_dir,att_rot);
				return;
				}
			}

	}
}

void hst_ex_ev1::DoIncAttZ()
{
OBJHANDLE hPVes = GetAttachmentStatus(att1);
if (hPVes)
	{
	VESSEL *v = oapiGetVesselInterface (hPVes);

			DWORD att_count = v->AttachmentCount (false);
			for (int i = 0; i < att_count; i++)
			{
			ATTACHMENTHANDLE hParentAttach = v->GetAttachmentHandle(false, i);
			OBJHANDLE hChildVessel = v->GetAttachmentStatus(hParentAttach);
			if (hChildVessel == GetHandle()) 
				{	
				VECTOR3 pves_x;
				VECTOR3 glob_x;
				VECTOR3 pves_pos;
				v->GetGlobalPos(pves_pos);
				GlobalRot(_V(0,0,1),glob_x);
				glob_x = glob_x + pves_pos;
				v->Global2Local(glob_x,pves_x);


				v->GetAttachmentParams(hParentAttach,att_pos,att_dir,att_rot);
				att_pos.x = att_pos.x + pves_x.x*delta_pos;
				att_pos.y = att_pos.y + pves_x.y*delta_pos;
				att_pos.z = att_pos.z + pves_x.z*delta_pos;
				v->SetAttachmentParams(hParentAttach,att_pos,att_dir,att_rot);
				return;
				}
			}

	}
}
void hst_ex_ev1::DoDecAttZ()
{
OBJHANDLE hPVes = GetAttachmentStatus(att1);
if (hPVes)
	{
	VESSEL *v = oapiGetVesselInterface (hPVes);

			DWORD att_count = v->AttachmentCount (false);
			for (int i = 0; i < att_count; i++)
			{
			ATTACHMENTHANDLE hParentAttach = v->GetAttachmentHandle(false, i);
			OBJHANDLE hChildVessel = v->GetAttachmentStatus(hParentAttach);
			if (hChildVessel == GetHandle()) 
				{	
				VECTOR3 pves_x;
				VECTOR3 glob_x;
				VECTOR3 pves_pos;
				v->GetGlobalPos(pves_pos);
				GlobalRot(_V(0,0,-1),glob_x);
				glob_x = glob_x + pves_pos;
				v->Global2Local(glob_x,pves_x);


				v->GetAttachmentParams(hParentAttach,att_pos,att_dir,att_rot);
				att_pos.x = att_pos.x + pves_x.x*delta_pos;
				att_pos.y = att_pos.y + pves_x.y*delta_pos;
				att_pos.z = att_pos.z + pves_x.z*delta_pos;
				v->SetAttachmentParams(hParentAttach,att_pos,att_dir,att_rot);
				return;
				}
			}

	}
}

void hst_ex_ev1::DoIncLampAz()
{
arm_proc = arm_proc + (delta_angle*3);
if (arm_proc > 1) arm_proc = 1;
SetAnimation(anim_arm,arm_proc);
}

void hst_ex_ev1::DoDecLampAz()
{
arm_proc = arm_proc - (delta_angle*3);
if (arm_proc < 0) arm_proc = 0;
SetAnimation(anim_arm,arm_proc);
}

void hst_ex_ev1::DoIncLampAlt()
{
waist_proc = waist_proc + (delta_angle*3);
if (waist_proc > 1) waist_proc = 1;
SetAnimation(anim_waist,waist_proc);
SetAnimation(anim_los,waist_proc);
//SetBeaconLightFOV();
cam_status = 1;
}

void hst_ex_ev1::DoDecLampAlt()
{
waist_proc = waist_proc - (delta_angle*3);
if (waist_proc < 0) waist_proc = 0;
SetAnimation(anim_waist,waist_proc);
SetAnimation(anim_los,waist_proc);
//SetBeaconLightFOV();
cam_status = 1;
}

void hst_ex_ev1::TwistRight()
{
twist_proc = twist_proc + (delta_angle*3);
if (twist_proc > 1) twist_proc = 1;
SetAnimation(anim_twist,twist_proc);
//SetBeaconLightFOV();
cam_status = 1;
}

void hst_ex_ev1::TwistLeft()
{
twist_proc = twist_proc - (delta_angle*3);
if (twist_proc < 0) twist_proc = 0;
SetAnimation(anim_twist,twist_proc);
//SetBeaconLightFOV();
cam_status = 1;
}

void hst_ex_ev1::RotHandDown()
{
hand_proc = hand_proc + (delta_angle*3);
if (hand_proc > 1) hand_proc = 1;
SetAnimation(anim_hand,hand_proc);
SetAnimation(anim_att,hand_proc);
SetAnimation(anim_hldr,hand_proc);
}

void hst_ex_ev1::RotHandUp()
{
hand_proc = hand_proc - (delta_angle*3);
if (hand_proc < 0) hand_proc = 0;
SetAnimation(anim_hand,hand_proc);
SetAnimation(anim_att,hand_proc);
SetAnimation(anim_hldr,hand_proc);
}

void hst_ex_ev1::SetBeaconLightFOV()
{

new_beacon_pos = los[2];
VECTOR3 new_dir = los[1] - los[0];

spot_beacon.pos = &new_beacon_pos;
spotlight1->SetPosition(new_beacon_pos);
spotlight1->SetDirection(new_dir);




//if(oapiCameraMode() == CAM_COCKPIT)
//{
SetCameraDefaultDirection(new_dir, 0);
oapiCameraSetCockpitDir(0,0);
//}
SetCameraOffset(los[0]);
if(oapiGetFocusObject () == GetHandle())
	{
	if(oapiCameraMode() == CAM_COCKPIT)oapiCameraAttach(GetHandle(), 2);
	}
}


void hst_ex_ev1::DoAttDirX()
{
OBJHANDLE hPVes = GetAttachmentStatus(att1);
if (hPVes)
	{
	VESSEL *v = oapiGetVesselInterface (hPVes);

			DWORD att_count = v->AttachmentCount (false);
			for (int i = 0; i < att_count; i++)
			{
			ATTACHMENTHANDLE hParentAttach = v->GetAttachmentHandle(false, i);
			OBJHANDLE hChildVessel = v->GetAttachmentStatus(hParentAttach);
			if (hChildVessel == GetHandle()) 
				{	
				v->GetAttachmentParams(hParentAttach,att_pos,att_dir,att_rot);

	VECTOR3 x_right = crossp(att_dir, att_rot);
	normalise(x_right);
				
	double dir_x_angle = -2*PI*delta_angle;
	
	double ct = cos(dir_x_angle);
	double st = sin(dir_x_angle);

	double ux = x_right.x;
	double uy = x_right.y;
	double uz = x_right.z;
	double ux2 = ux*ux;
	double uy2 = uy*uy;
	double uz2 = uz*uz;

	MATRIX3 RG1;

	RG1.m11 = ux2+(1-ux2)*ct;
	RG1.m12 = ux*uy*(1-ct)-uz*st;
	RG1.m13 = ux*uz*(1-ct)+uy*st;
	RG1.m21 = ux*uy*(1-ct)+uz*st;
	RG1.m22 = uy2+(1-uy2)*ct;
	RG1.m23 = uy*uz*(1-ct)-ux*st;
	RG1.m31 = ux*uz*(1-ct)-uy*st;
	RG1.m32 = uy*uz*(1-ct)+ux*st;
	RG1.m33 = uz2+(1-uz2)*ct;

	VECTOR3 new_att_dir = mul(RG1,att_dir);
	VECTOR3 new_att_rot = mul(RG1,att_rot);



				v->SetAttachmentParams(hParentAttach,att_pos,new_att_dir,new_att_rot);
				return;
				}
			}

	}

}

void hst_ex_ev1::DoAttDirZ()
{
OBJHANDLE hPVes = GetAttachmentStatus(att1);
if (hPVes)
	{
	VESSEL *v = oapiGetVesselInterface (hPVes);

			DWORD att_count = v->AttachmentCount (false);
			for (int i = 0; i < att_count; i++)
			{
			ATTACHMENTHANDLE hParentAttach = v->GetAttachmentHandle(false, i);
			OBJHANDLE hChildVessel = v->GetAttachmentStatus(hParentAttach);
			if (hChildVessel == GetHandle()) 
				{	
				v->GetAttachmentParams(hParentAttach,att_pos,att_dir,att_rot);
				
	double dir_z_angle = 2*PI*delta_angle;
	
	double ct = cos(dir_z_angle);
	double st = sin(dir_z_angle);

	double ux = att_dir.x;
	double uy = att_dir.y;
	double uz = att_dir.z;
	double ux2 = ux*ux;
	double uy2 = uy*uy;
	double uz2 = uz*uz;

	MATRIX3 RG1;

	RG1.m11 = ux2+(1-ux2)*ct;
	RG1.m12 = ux*uy*(1-ct)-uz*st;
	RG1.m13 = ux*uz*(1-ct)+uy*st;
	RG1.m21 = ux*uy*(1-ct)+uz*st;
	RG1.m22 = uy2+(1-uy2)*ct;
	RG1.m23 = uy*uz*(1-ct)-ux*st;
	RG1.m31 = ux*uz*(1-ct)-uy*st;
	RG1.m32 = uy*uz*(1-ct)+ux*st;
	RG1.m33 = uz2+(1-uz2)*ct;

	VECTOR3 new_att_dir = mul(RG1,att_dir);
	VECTOR3 new_att_rot = mul(RG1,att_rot);



				v->SetAttachmentParams(hParentAttach,att_pos,new_att_dir,new_att_rot);
				return;
				}
			}

	}

}





void hst_ex_ev1::DoDecAttDirX()
{
OBJHANDLE hPVes = GetAttachmentStatus(att1);
if (hPVes)
	{
	VESSEL *v = oapiGetVesselInterface (hPVes);

			DWORD att_count = v->AttachmentCount (false);
			for (int i = 0; i < att_count; i++)
			{
			ATTACHMENTHANDLE hParentAttach = v->GetAttachmentHandle(false, i);
			OBJHANDLE hChildVessel = v->GetAttachmentStatus(hParentAttach);
			if (hChildVessel == GetHandle()) 
				{	
				v->GetAttachmentParams(hParentAttach,att_pos,att_dir,att_rot);

	VECTOR3 x_right = crossp(att_dir, att_rot);
	normalise(x_right);
				
	double dir_x_angle = 2*PI*delta_angle;
	
	double ct = cos(dir_x_angle);
	double st = sin(dir_x_angle);

	double ux = x_right.x;
	double uy = x_right.y;
	double uz = x_right.z;
	double ux2 = ux*ux;
	double uy2 = uy*uy;
	double uz2 = uz*uz;

	MATRIX3 RG1;

	RG1.m11 = ux2+(1-ux2)*ct;
	RG1.m12 = ux*uy*(1-ct)-uz*st;
	RG1.m13 = ux*uz*(1-ct)+uy*st;
	RG1.m21 = ux*uy*(1-ct)+uz*st;
	RG1.m22 = uy2+(1-uy2)*ct;
	RG1.m23 = uy*uz*(1-ct)-ux*st;
	RG1.m31 = ux*uz*(1-ct)-uy*st;
	RG1.m32 = uy*uz*(1-ct)+ux*st;
	RG1.m33 = uz2+(1-uz2)*ct;

	VECTOR3 new_att_dir = mul(RG1,att_dir);
	VECTOR3 new_att_rot = mul(RG1,att_rot);



				v->SetAttachmentParams(hParentAttach,att_pos,new_att_dir,new_att_rot);
				return;
				}
			}

	}

}

void hst_ex_ev1::DoDecAttDirZ()
{
OBJHANDLE hPVes = GetAttachmentStatus(att1);
if (hPVes)
	{
	VESSEL *v = oapiGetVesselInterface (hPVes);

			DWORD att_count = v->AttachmentCount (false);
			for (int i = 0; i < att_count; i++)
			{
			ATTACHMENTHANDLE hParentAttach = v->GetAttachmentHandle(false, i);
			OBJHANDLE hChildVessel = v->GetAttachmentStatus(hParentAttach);
			if (hChildVessel == GetHandle()) 
				{	
				v->GetAttachmentParams(hParentAttach,att_pos,att_dir,att_rot);
				
	double dir_z_angle = -2*PI*delta_angle;
	
	double ct = cos(dir_z_angle);
	double st = sin(dir_z_angle);

	double ux = att_dir.x;
	double uy = att_dir.y;
	double uz = att_dir.z;
	double ux2 = ux*ux;
	double uy2 = uy*uy;
	double uz2 = uz*uz;

	MATRIX3 RG1;

	RG1.m11 = ux2+(1-ux2)*ct;
	RG1.m12 = ux*uy*(1-ct)-uz*st;
	RG1.m13 = ux*uz*(1-ct)+uy*st;
	RG1.m21 = ux*uy*(1-ct)+uz*st;
	RG1.m22 = uy2+(1-uy2)*ct;
	RG1.m23 = uy*uz*(1-ct)-ux*st;
	RG1.m31 = ux*uz*(1-ct)-uy*st;
	RG1.m32 = uy*uz*(1-ct)+ux*st;
	RG1.m33 = uz2+(1-uz2)*ct;

	VECTOR3 new_att_dir = mul(RG1,att_dir);
	VECTOR3 new_att_rot = mul(RG1,att_rot);



				v->SetAttachmentParams(hParentAttach,att_pos,new_att_dir,new_att_rot);
				return;
				}
			}

	}

}








void hst_ex_ev1::DoIncAttRot()
{
OBJHANDLE hPVes = GetAttachmentStatus(att1);
if (hPVes)
	{
	VESSEL *v = oapiGetVesselInterface (hPVes);

			DWORD att_count = v->AttachmentCount (false);
			for (int i = 0; i < att_count; i++)
			{
			ATTACHMENTHANDLE hParentAttach = v->GetAttachmentHandle(false, i);
			OBJHANDLE hChildVessel = v->GetAttachmentStatus(hParentAttach);
			if (hChildVessel == GetHandle()) 
				{	
				v->GetAttachmentParams(hParentAttach,att_pos,att_dir,att_rot);
				
	double rot_angle = 2*PI*delta_angle;
	
	double ct = cos(rot_angle);
	double st = sin(rot_angle);

	double ux = att_rot.x;
	double uy = att_rot.y;
	double uz = att_rot.z;
	double ux2 = ux*ux;
	double uy2 = uy*uy;
	double uz2 = uz*uz;

	MATRIX3 RG1;

	RG1.m11 = ux2+(1-ux2)*ct;
	RG1.m12 = ux*uy*(1-ct)-uz*st;
	RG1.m13 = ux*uz*(1-ct)+uy*st;
	RG1.m21 = ux*uy*(1-ct)+uz*st;
	RG1.m22 = uy2+(1-uy2)*ct;
	RG1.m23 = uy*uz*(1-ct)-ux*st;
	RG1.m31 = ux*uz*(1-ct)-uy*st;
	RG1.m32 = uy*uz*(1-ct)+ux*st;
	RG1.m33 = uz2+(1-uz2)*ct;

	VECTOR3 new_att_dir = mul(RG1,att_dir);
	VECTOR3 new_att_rot = mul(RG1,att_rot);



				v->SetAttachmentParams(hParentAttach,att_pos,new_att_dir,new_att_rot);
				return;
				}
			}

	}

}



void hst_ex_ev1::DoDecAttRot()
{
OBJHANDLE hPVes = GetAttachmentStatus(att1);
if (hPVes)
	{
	VESSEL *v = oapiGetVesselInterface (hPVes);

			DWORD att_count = v->AttachmentCount (false);
			for (int i = 0; i < att_count; i++)
			{
			ATTACHMENTHANDLE hParentAttach = v->GetAttachmentHandle(false, i);
			OBJHANDLE hChildVessel = v->GetAttachmentStatus(hParentAttach);
			if (hChildVessel == GetHandle()) 
				{	
				v->GetAttachmentParams(hParentAttach,att_pos,att_dir,att_rot);
				
	double rot_angle = -2*PI*delta_angle;
	
	double ct = cos(rot_angle);
	double st = sin(rot_angle);

	double ux = att_rot.x;
	double uy = att_rot.y;
	double uz = att_rot.z;
	double ux2 = ux*ux;
	double uy2 = uy*uy;
	double uz2 = uz*uz;

	MATRIX3 RG1;

	RG1.m11 = ux2+(1-ux2)*ct;
	RG1.m12 = ux*uy*(1-ct)-uz*st;
	RG1.m13 = ux*uz*(1-ct)+uy*st;
	RG1.m21 = ux*uy*(1-ct)+uz*st;
	RG1.m22 = uy2+(1-uy2)*ct;
	RG1.m23 = uy*uz*(1-ct)-ux*st;
	RG1.m31 = ux*uz*(1-ct)-uy*st;
	RG1.m32 = uy*uz*(1-ct)+ux*st;
	RG1.m33 = uz2+(1-uz2)*ct;

	VECTOR3 new_att_dir = mul(RG1,att_dir);
	VECTOR3 new_att_rot = mul(RG1,att_rot);



				v->SetAttachmentParams(hParentAttach,att_pos,new_att_dir,new_att_rot);
				return;
				}
			}

	}

}




void hst_ex_ev1::SetParentAtt()
{

OBJHANDLE hPVes = GetAttachmentStatus(att1);
if (hPVes)
	{
	VESSEL *v = oapiGetVesselInterface (hPVes);

			DWORD att_count = v->AttachmentCount (false);
			for (int i = 0; i < att_count; i++)
			{
			ATTACHMENTHANDLE hParentAttach = v->GetAttachmentHandle(false, i);
			OBJHANDLE hChildVessel = v->GetAttachmentStatus(hParentAttach);
			if (hChildVessel == GetHandle()) 
				{	
				v->SetAttachmentParams(hParentAttach,scn_att_pos,scn_att_dir,scn_att_rot);
				//sprintf(oapiDebugString(), "scn_att_pos %0.4f %0.4f %0.4f", scn_att_pos.x, scn_att_pos.y, scn_att_pos.z);
				return;
				}
			}
	}
}

void hst_ex_ev1::GetParentAtt()
{

OBJHANDLE hPVes = GetAttachmentStatus(att1);
if (hPVes)
	{
	VESSEL *v = oapiGetVesselInterface (hPVes);

			DWORD att_count = v->AttachmentCount (false);
			for (int i = 0; i < att_count; i++)
			{
			ATTACHMENTHANDLE hParentAttach = v->GetAttachmentHandle(false, i);
			OBJHANDLE hChildVessel = v->GetAttachmentStatus(hParentAttach);
			if (hChildVessel == GetHandle()) 
				{	
				v->GetAttachmentParams(hParentAttach,scn_att_pos,scn_att_dir,scn_att_rot);
				return;
				}
			}
	}
}


void hst_ex_ev1::Grapple()
{
	if (GetAttachmentStatus(att2) == NULL)
	{             
		VECTOR3 gpos, grms, pos, dir, rot;
		Local2Global (arm_tip[0], grms);  // global position of RMS tip
		
		// Search the complete vessel list for a grappling candidate.
		// Not very scalable ...
		for (DWORD i = 0; i < oapiGetVesselCount(); i++) 
		{
			OBJHANDLE hV = oapiGetVesselByIndex (i);
			if (hV == GetHandle()) continue; // we don't want to grapple ourselves ...
			oapiGetGlobalPos (hV, &gpos);
			if (dist (gpos, grms) < 5) 
				{ // in range
				VESSEL *v = oapiGetVesselInterface (hV);
				DWORD nAttach = v->AttachmentCount (true);
				for (DWORD j = 0; j < nAttach; j++) 
					{ // now scan all attachment points of the candidate
					
					ATTACHMENTHANDLE hAtt = v->GetAttachmentHandle (true, j);
					const char *id = v->GetAttachmentId (hAtt);
					if (strncmp (id, "GS", 2)) continue; // attachment point not compatible
					v->GetAttachmentParams (hAtt, pos, dir, rot);
					v->Local2Global (pos, gpos);
					if (dist (gpos, grms) < 0.3) 
						{
						if (strncmp (id, "GSHLD", 5))
							{
							AttachChild (hV, att2, hAtt);
							return;
							}
						else{
							oapiDeleteVessel(hV);
							hldr_sw = 1;
							SetMeshVisibilityMode (1, MESHVIS_ALWAYS);
							return;
							}
						}
						
					}
				}
			}
		}

	else
	{
	DetachChild(att2);
	}
	



}

void hst_ex_ev1::EndEVA()
{
	VECTOR3 gpos, mepos, pos, dir, rot;
	GetGlobalPos(mepos);

	for (DWORD i = 0; i < oapiGetVesselCount(); i++) 
	{
	OBJHANDLE hV = oapiGetVesselByIndex (i);
	if (hV == GetHandle()) continue; // we don't want to grapple ourselves ...
	oapiGetGlobalPos (hV, &gpos);
	if (dist (gpos, mepos) < 2*oapiGetSize(hV)) 
		{
		VESSEL *v = oapiGetVesselInterface (hV);
		DWORD nDock = v->DockCount();
		if (nDock == 0) continue;
		for (DWORD j = 0; j < nDock; j++)
			{
			DOCKHANDLE hDock = v->GetDockHandle(j);
			v->GetDockParams(hDock, pos, dir, rot);
			v->Local2Global (pos, gpos);
			if (dist (gpos, mepos) < 1)
				{
				oapiDeleteVessel(GetHandle());
				}
			}

	
	
	
		}
	}

}

void hst_ex_ev1::DoHolder()
{
	if (GetAttachmentStatus(att2) == NULL && hldr_sw == 1)
	{
	SetMeshVisibilityMode (1, MESHVIS_NEVER);
	hldr_sw = 0;
	SetAttachmentParams(att2, arm_tip[0], arm_tip[1]-arm_tip[0], arm_tip[2]-arm_tip[1]);

	VESSELSTATUS2 vs;
	memset (&vs, 0, sizeof(vs));
	vs.version = 2;
	GetStatusEx(&vs);

	VECTOR3 ves_pos;
	Local2Rel(arm_tip[3],ves_pos);
	vs.rpos = ves_pos;
	vs.status = 0;

	VECTOR3 lyup = arm_tip[2]-arm_tip[1];
	VECTOR3 lzforward = arm_tip[1]-arm_tip[0];
	VECTOR3 lxright = crossp (lyup, lzforward);

	VECTOR3 xright, yup, zforward;

	GlobalRot(lxright, xright);
	GlobalRot(lyup, yup);
	GlobalRot(lzforward,zforward);

	double gamma = atan2( xright.x, yup.x);
	double beta = -1*(asin(zforward.x));
	double alpha = atan2(zforward.y, zforward.z);
	//SetGlobalOrientation (_V(alpha, beta, (PI/2)-gamma));

	vs.arot = _V(alpha, beta, (PI/2)-gamma);


	OBJHANDLE hWFC = oapiCreateVesselEx ("TempHolder", "HST_EX/holder", &vs);

	}


}