// ==============================================================
//                 ORBITER MODULE: hst_ex_sa2p_ru
//                  
//
// hst_ex_sa2p_ru.cpp
// Control module for hst_ex_sa2p_ru vessel class
//
// 
// ==============================================================

#define STRICT
#define ORBITER_MODULE

#include "orbitersdk.h"
#include "gcAPI.h"

// ==============================================================
// Some vessel parameters
// ==============================================================


const double ARRAY_OPERATING_SPEED = 0.2;



// 


class hst_ex_sa2p_ru: public VESSEL3 {
public:
	hst_ex_sa2p_ru (OBJHANDLE hVessel, int flightmodel);
	~hst_ex_sa2p_ru ();
	void clbkSetClassCaps (FILEHANDLE cfg);
	void DefineAnimations (void);
	void clbkPreStep(double SimT, double SimDT, double MJD);
	//void clbkPostStep (double simt, double simdt, double mjd);
	void clbkPostCreation();
	void clbkSaveState (FILEHANDLE scn);
	void clbkLoadStateEx (FILEHANDLE scn, void *status);

	



private:
	ATTACHMENTHANDLE eva1, eva2, hst;
	UINT anim_arm;
	double arm_proc;
	int arm_status, att_status;
};
///////////////////

hst_ex_sa2p_ru::hst_ex_sa2p_ru (OBJHANDLE hVessel, int flightmodel)
: VESSEL3 (hVessel, flightmodel)
{

}

hst_ex_sa2p_ru::~hst_ex_sa2p_ru ()
{
	
}



// Define animation sequences for moving parts
// --------------------------------------------------------------
void hst_ex_sa2p_ru::DefineAnimations (void)
{
	// 1. arm
	anim_arm = CreateAnimation (0);
	static UINT HiGainAnt1Grp[1] = {0};
	static MGROUP_ROTATE HiGainAnt1 (0, HiGainAnt1Grp, 1, _V(0.711,0,0), _V(0,0,1), (float)(90*RAD));
	
	AddAnimationComponent (anim_arm, 0, 1, &HiGainAnt1);
}



// ==============================================================
// hst_ex_sa2p_ru class interface
// ==============================================================

// ==============================================================
// Overloaded callback functions
// ==============================================================

// --------------------------------------------------------------
// Set the capabilities of the vessel class
// --------------------------------------------------------------
void hst_ex_sa2p_ru::clbkSetClassCaps (FILEHANDLE cfg)
{
	arm_proc = 0.0;
	arm_status = 0;
	att_status = 0;
	DefineAnimations ();
	gcInitialize();

	// physical specs
	SetCW (0.2, 0.2, 1, 1);
	SetRotDrag (_V(1,1,0.2));
	SetPMI (_V(0.3,0.3,0.15));
	SetCrossSections (_V(1,1,10));
	SetSize (2.0);
	SetEmptyMass (90);
	if (gcEnabled() == true) SetMeshVisibilityMode ((AddMesh ("HST_EX/DX9/sa2_port_ru")), MESHVIS_EXTERNAL);
	else SetMeshVisibilityMode ((AddMesh ("HST_EX/DX7/sa2_port_ru")), MESHVIS_EXTERNAL);
	SetTouchdownPoints (_V(-1,0,-0.5), _V(1,1,-0.5), _V(1,-1,-0.5));

	
	eva1 = CreateAttachment (true, _V(0.574, 0.168, 0),_V(0, 1, 0),_V(0, 0, 1), "GS");
	eva2 = CreateAttachment (true, _V(-2.22, 0, 0), _V(-1, 0, 0), _V(0, 1, 0), "GS");

}

// ==============================================================
// API callback interface
// ==============================================================

// --------------------------------------------------------------
// Vessel initialisation
// --------------------------------------------------------------



DLLCLBK VESSEL *ovcInit (OBJHANDLE hvessel, int flightmodel)
{
	return new hst_ex_sa2p_ru (hvessel, flightmodel);
}


// --------------------------------------------------------------
// Vessel cleanup
// --------------------------------------------------------------
DLLCLBK void ovcExit (VESSEL *vessel)
{
	if (vessel) delete (hst_ex_sa2p_ru*)vessel;
}
// ==============================================================
// Overloaded clbkPostCreation callback functions
// ==============================================================

void hst_ex_sa2p_ru::clbkPostCreation ()
{
	if (att_status == 1)
	{
	hst = CreateAttachment (true, _V(1.967, 0, -0.24), _V(1, 0, 0), _V(0, -1, 0), "S2P");
	}
}
void hst_ex_sa2p_ru::clbkSaveState (FILEHANDLE scn)
{
	char cbuf[256];

	// default vessel parameters
	VESSEL2::clbkSaveState (scn);

	// custom parameters

	sprintf (cbuf, "%d %0.4f %d", arm_status, arm_proc, att_status);
	oapiWriteScenario_string (scn, "ARMATT", cbuf);

}
void hst_ex_sa2p_ru::clbkLoadStateEx (FILEHANDLE scn, void *vs)
{
    char *line;
	while (oapiReadScenario_nextline (scn, line)) {
		if (!strnicmp (line, "ARMATT", 6)) {
            sscanf (line+6, "%d%lf%d", &arm_status, &arm_proc, &att_status);		
		}else {
            ParseScenarioLineEx (line, vs);
			// unrecognised option - pass to Orbiter's generic parser
		}
	}
	SetAnimation (anim_arm, arm_proc);
}


void hst_ex_sa2p_ru::clbkPreStep(double SimT, double SimDT, double MJD) 
{

if (att_status == 0)
	{
	if (!GetAttachmentStatus(eva1) == NULL)
		{
		att_status = 1;
		arm_status = 1;
		hst = CreateAttachment (true, _V(1.967, 0, -0.24), _V(1, 0, 0), _V(0, -1, 0), "S2P");
		}

	}

if (arm_status > 0)
{
double da = SimDT * ARRAY_OPERATING_SPEED;
arm_proc = arm_proc+da;
if (arm_proc > 1) arm_proc = 1;
SetAnimation (anim_arm, arm_proc);
}


}
//void hst_ex_sa2p_ru::clbkPostStep (double simt, double simdt, double mjd)
//{




//}