// ==============================================================
//
//hst_ex_sm1.cpp
// Control module for hst_ex_sm1 vessel class
//
// ==============================================================

#define STRICT
#define ORBITER_MODULE

#include "orbitersdk.h"
#include "gcAPI.h"

// ==============================================================
// Some vessel parameters
// ==============================================================

static double door_dur = 0.2;
static double rot_dur = 0.005;

static VECTOR3 beaconcol1 = _V(1.0,0.3,0.0);
static VECTOR3 beaconcol2 = _V(0.1,1.0,0.1);
static VECTOR3 beacon_col = _V(1,1,1);
static VECTOR3 sts_lightpos = _V(0.06,3.4,12.3);

#define GREEN RGB(125, 255, 125)

// ==============================================================
// Shuttle-PB class interface
// ==============================================================

class hst_ex_sm1: public VESSEL3 {
public:
	hst_ex_sm1 (OBJHANDLE hVessel, int flightmodel);
	~hst_ex_sm1 ();
	void clbkSetClassCaps (FILEHANDLE cfg);
	void clbkPreStep(double SimT, double SimDT, double MJD);
	//////void clbkPostStep (double simt, double simdt, double mjd);
	void DefineAnimations (void);
	void clbkSaveState (FILEHANDLE scn);
	void clbkLoadStateEx (FILEHANDLE scn, void *status);
	int clbkConsumeBufferedKey(DWORD key, bool down, char *kstate);
	bool clbkDrawHUD (int mode, const HUDPAINTSPEC *hps, oapi::Sketchpad *skp);
	void clbkPostCreation();
	
	void DoBeacons(void);
	void CheckVessels(void);
	void CheckVessels2(void);
	void SpawnEVA(void);
	void SpawnEVA2(void);



	
	ANIMATIONCOMPONENT_HANDLE ach;

	ATTACHMENTHANDLE ax_att, rad_att, rsu1_att, rsu2_att, rsu3_att, sa_port_att, sa_stbd_att, sa_temp_att, ft_pad_att;
	ATTACHMENTHANDLE hst_base_att, ft_pad_att2, hldr_att;

private:
	UINT anim_out_ring, anim_in_ring, anim_asipe, anim_rsipe, anim_rsu;
	double out_ring_proc, in_ring_proc, asipe_proc, rsipe_proc, rsu_proc;
	
	

	int out_ring_status, in_ring_status, asipe_status, rsipe_status, rsu_status;
	int ax_att_status, rad_att_status, rsu1_att_status, rsu2_att_status, rsu3_att_status;
	int	sa_port_att_status, sa_stbd_att_status, sa_temp_att_status, ft_pad_att_status;
	//int dobeacon_st;

	double step_time;

	VECTOR3 beaconpos[6];
	VECTOR3 attp_pos[12];

	int att_pc_check;
	int cv1cv2_check;
	int light_sts;
	

	BEACONLIGHTSPEC beacon[6];
	SpotLight *spotlight1;
	BEACONLIGHTSPEC sts_beacon;

	NOTEHANDLE hNote;
	int anot_status;
	double anot_timer;
};

hst_ex_sm1::hst_ex_sm1 (OBJHANDLE hVessel, int flightmodel)
: VESSEL3 (hVessel, flightmodel)
{
    

}

hst_ex_sm1::~hst_ex_sm1 ()
{
}




// ==============================================================
// Overloaded callback functions
// ==============================================================

// --------------------------------------------------------------
// Set the capabilities of the vessel class
// --------------------------------------------------------------
void hst_ex_sm1::clbkSetClassCaps (FILEHANDLE cfg)
{
	out_ring_proc = 0.0;
	in_ring_proc = 0.0;
	asipe_proc = 0.0;
	rsipe_proc = 0.0;
	rsu_proc = 0.0;

	step_time = 0.0;

	out_ring_status = 0;
	in_ring_status = 0;
	asipe_status = 0;
	rsipe_status = 0;
	rsu_status = 0;
	//dobeacon_st = 0;

	att_pc_check = 0;
	cv1cv2_check = 0;
	light_sts = 0;

	anot_status = 0;
	anot_timer = 0.0;

	ax_att_status = 0;
	rad_att_status = 0;
	rsu1_att_status = 0;
	rsu2_att_status = 0;
	rsu3_att_status = 0;
	sa_port_att_status = 0;
	sa_stbd_att_status  = 0;
	sa_temp_att_status = 0;
	ft_pad_att_status = 0;

	DefineAnimations();
	gcInitialize();

	// physical vessel parameters
	SetSize (10);
	SetEmptyMass (5000);
	SetPMI(_V(13,13,3));
	SetCrossSections (_V(40,40,20));
	SetRotDrag (_V(1,1,1));
	SetTouchdownPoints ( _V(0,-2, 5), _V(-2,-2,-5), _V( 2,-2,-5));

	// camera parameters
	SetCameraOffset (_V(0,0,2));

	// att point pos for grap checks
	attp_pos[0] = _V(0, 0.6, -0.4); // axial sipe cos
	attp_pos[1] = _V(0, -0.4, 2.85); // radial sipe wfc
	attp_pos[2] = _V(1.7, 1.2, 1.8); // rsu 1
	attp_pos[3] = _V(1.7, 1.2, 1.3); // rsu2
	attp_pos[4] = _V(1.7, 1.2, 0.8);	// rsu3
	
	attp_pos[5] = _V(0.883, 1.276, 3.945);	// sa stbd
	attp_pos[6] = _V(-0.403, 1.276, 8.115); // sa port
	attp_pos[7] = _V(2.319, 1.216, 4.000); // sa temp

	attp_pos[8] = _V(0, 2.585, -2.992); // hst docking point deployed

	attp_pos[9] = _V(-0.638,0.7,8.8); // ft pad
	attp_pos[10] = _V(0.3,0.9,8.9); // ft pad2
	attp_pos[11] = _V(-1.75, 1.15, 0.95); // hldr att

	//// ////////////
	ax_att = CreateAttachment (false, attp_pos[0], _V(0,-1,0), _V(-1,0,0), "ASP");
	rad_att = CreateAttachment (false, attp_pos[1], _V(0,0,-1), _V(0,-1,0), "RSP");

	rsu1_att = CreateAttachment (false, attp_pos[2], _V(1,0,0), _V(0,0,1), "RSU1");
	rsu2_att = CreateAttachment (false, attp_pos[3], _V(1,0,0), _V(0,0,1), "RSU2");
	rsu3_att = CreateAttachment (false, attp_pos[4], _V(1,0,0), _V(0,0,1), "RSU3");

	sa_stbd_att = CreateAttachment (false, attp_pos[5], _V(0,0, 1), _V(0,1,0), "SAS");
	sa_port_att = CreateAttachment (false, attp_pos[6], _V(0,0,-1), _V(0,-1,0), "SAP");
	sa_temp_att = CreateAttachment (false, attp_pos[7], _V(0,0, 1), _V(0,1,0), "SAT");

	hst_base_att = CreateAttachment (false, _V(0,1.537,-3.40), _V(0,-1,0), _V(1,0,0), "HST");

	ft_pad_att = CreateAttachment (false, attp_pos[9], _V(0,1,0), _V(0,0,-1), "FPD");
	ft_pad_att2 = CreateAttachment (false, attp_pos[10], _V(0,-1,0), _V(0,0,-1), "FPD");

	hldr_att = CreateAttachment (false, attp_pos[11], _V(-1,0,0), _V(0,1,0), "HLD");

	

	if (gcEnabled() == true)SetMeshVisibilityMode(AddMesh("HST_EX/DX9/sm1_cargobay"), MESHVIS_EXTERNAL);
	else SetMeshVisibilityMode(AddMesh("HST_EX/DX7/sm1_cargobay"), MESHVIS_EXTERNAL);


	beaconpos[0] = _V(-0.057,1.454,3.372); // rsipe door closed
	beaconpos[1] = _V(-0.636,1.125,0.925); // asipe door closed
	beaconpos[2] = _V(1.403,1.509,1.220); //  rsu door closed

	beaconpos[3] = _V(-0.057, 2.622, 1.364); // rsipe door open
	beaconpos[4] = _V(0.774, 2.391, 0.925); // asipe door open
	beaconpos[5] = _V(2.102, 2.136, 1.220); //  rsu door open
	



	///// add beacons
	for (int i = 0; i < 6; i++) 
	{
		beacon[i].shape = BEACONSHAPE_COMPACT;
		beacon[i].pos = &beaconpos[i];
		beacon[i].col = &beaconcol1;
		beacon[i].size = 0.1;
		beacon[i].falloff = 0.1;
		beacon[i].period = 1;
		beacon[i].duration = 0.75;
		beacon[i].tofs = 0;
		beacon[i].active = false;
		AddBeacon (&beacon[i]);
	}

	for (int i = 3; i < 6; i++) 
	{
		beacon[i].col = &beaconcol2;
	}

	COLOUR4 col_d = {1,1,0.9,0};
	COLOUR4 col_s = {1,1,0.9,0};
	COLOUR4 col_a = {0,0,0,0};

	
	

	sts_beacon.shape = BEACONSHAPE_STAR;
		sts_beacon.pos = &sts_lightpos;
		sts_beacon.col = &beacon_col;
		sts_beacon.size = 0.75;
		sts_beacon.falloff = 0.3;
		sts_beacon.period = 0;
		sts_beacon.duration = 0;
		sts_beacon.tofs = 0;
		sts_beacon.active = false;
		AddBeacon (&sts_beacon);
	
	spotlight1 = (SpotLight*)AddSpotLight(sts_lightpos, _V(0,-0.259,-0.966),300, 1, 0, 5e-5, 70*RAD, (90*RAD), col_d, col_s, col_a);
	spotlight1->Activate(false);

		

}



// ==============================================================
// API callback interface
// ==============================================================

// --------------------------------------------------------------
// Vessel initialisation
// --------------------------------------------------------------
DLLCLBK VESSEL *ovcInit (OBJHANDLE hvessel, int flightmodel)
{
	return new hst_ex_sm1 (hvessel, flightmodel);
}

// --------------------------------------------------------------
// Vessel cleanup
// --------------------------------------------------------------
DLLCLBK void ovcExit (VESSEL *vessel)
{
	if (vessel) delete (hst_ex_sm1*)vessel;
}

void hst_ex_sm1::DefineAnimations (void)
{
// 1. outer ring

	anim_out_ring = CreateAnimation (0);

	static UINT orGrp[2] = {12,13};
	static MGROUP_ROTATE orAnim (0, orGrp, 2, _V(0, 2.315, -3.77), _V(-1,0,0), (float)(90*RAD));	
	ach = AddAnimationComponent (anim_out_ring, 0, 1, &orAnim);


// 2. inner ring

	anim_in_ring = CreateAnimation (0);

	static UINT irGrp[1] = {11};
	static MGROUP_ROTATE irAnim (0, irGrp, 1, _V(0, 1.539, 0), _V(0,0,1), (float)(360*RAD));
	AddAnimationComponent (anim_in_ring, 0, 1, &irAnim, ach);

// 3. asipe

	anim_asipe = CreateAnimation (0);
	static UINT aspGrp[4] = {5,6,7,8};
	static MGROUP_ROTATE aspAnim (0, aspGrp, 4, _V(0, 2.038, 2.368), _V(-1,0,0), (float)(180*RAD));
	AddAnimationComponent (anim_asipe, 0, 1, &aspAnim);

// 4. rsipe

	anim_rsipe = CreateAnimation (0);
	static UINT rspGrp[3] = {0,1,2};
	static MGROUP_ROTATE rspAnim (0, rspGrp, 3, _V(0.702, 1.053, 0), _V(0,0,-1), (float)(90*RAD));
	AddAnimationComponent (anim_rsipe, 0, 1, &rspAnim);

// 5. rsu

	anim_rsu = CreateAnimation (0);
	static UINT rsuGrp[2] = {4,10};
	static MGROUP_ROTATE rsuAnim (0, rsuGrp, 2, _V(2.066, 1.473, 0), _V(0,0,-1), (float)(90*RAD));
	AddAnimationComponent (anim_rsu, 0, 1, &rsuAnim);



}


void hst_ex_sm1::clbkSaveState (FILEHANDLE scn)
{
	char cbuf[256];

	// default vessel parameters
	VESSEL3::clbkSaveState(scn);

	// custom parameters
	sprintf (cbuf, "%d", light_sts);
	oapiWriteScenario_string (scn, "LUM", cbuf);
	sprintf (cbuf, "%d %0.4f", asipe_status, asipe_proc);
	oapiWriteScenario_string (scn, "ASI", cbuf);
	sprintf (cbuf, "%d %0.4f", rsipe_status, rsipe_proc);
	oapiWriteScenario_string (scn, "RSI", cbuf);
	sprintf (cbuf, "%d %0.4f", rsu_status, rsu_proc);
	oapiWriteScenario_string (scn, "RSU", cbuf);
	sprintf (cbuf, "%d %0.4f", in_ring_status, in_ring_proc);
	oapiWriteScenario_string (scn, "IRN", cbuf);
	sprintf (cbuf, "%d %0.4f", out_ring_status, out_ring_proc);
	oapiWriteScenario_string (scn, "ORN", cbuf);
	
	

}
void hst_ex_sm1::clbkLoadStateEx (FILEHANDLE scn, void *vs)
{
    char *line;
	while (oapiReadScenario_nextline (scn, line)) {
		if (!strnicmp (line, "LUM", 3)) {
			sscanf (line+3, "%d", &light_sts);
		}else if (!strnicmp (line, "ASI", 3)) {
			sscanf (line+3, "%d%lf", &asipe_status, &asipe_proc);
		} else if (!strnicmp (line, "RSI", 3)) {
			sscanf (line+3, "%d%lf", &rsipe_status, &rsipe_proc);
		} else if (!strnicmp (line, "RSU", 3)) {
			sscanf (line+3, "%d%lf", &rsu_status, &rsu_proc);
		} else if (!strnicmp (line, "IRN", 3)) {
			sscanf (line+3, "%d%lf", &in_ring_status, &in_ring_proc);
		} else if (!strnicmp (line, "ORN", 3)) {
			sscanf (line+3, "%d%lf", &out_ring_status, &out_ring_proc);
		}else {
            ParseScenarioLineEx (line, vs);
			// unrecognised option - pass to Orbiter's generic parser
        }
   }
	SetAnimation (anim_asipe, asipe_proc);
	SetAnimation (anim_rsipe, rsipe_proc);
	SetAnimation (anim_rsu, rsu_proc);
	SetAnimation (anim_in_ring, in_ring_proc);
	SetAnimation (anim_out_ring, out_ring_proc);

if (light_sts == 1)
	{
	sts_beacon.active = true;
	spotlight1->Activate(true);
	}
else
	{
	sts_beacon.active = false;
	spotlight1->Activate(false);
	}



}



int hst_ex_sm1::clbkConsumeBufferedKey(DWORD key, bool down, char *kstate) {



  if (!down) return 0; // only process keydown events

  if (KEYMOD_SHIFT (kstate)) 
  {	 
	switch (key)
	{

	} 
  }


  else if (KEYMOD_CONTROL (kstate)) 
  { 
	switch (key)
	{
	
	} 
  }

	else { // unmodified keys
    switch (key) 
			{
	case OAPI_KEY_J: // spawn eva1
		SpawnEVA();
		return 1;

	case OAPI_KEY_K: // spawn eva2
		SpawnEVA2();
		return 1;

	case OAPI_KEY_G: // dock ring deploy
		if (out_ring_status == 0)out_ring_status = 1;
		else if (out_ring_status == 2)out_ring_status = 3;
		return 1;



	case OAPI_KEY_N: // innner rinf
		if (in_ring_status == 0)in_ring_status = 1;
		else if (in_ring_status == 1)in_ring_status = 0;
		else if (in_ring_status == 2)in_ring_status = 1;
		return 1;

	case OAPI_KEY_M: // inner ring
		if (in_ring_status == 0)in_ring_status = 2;
		else if (in_ring_status == 2)in_ring_status = 0;
		else if (in_ring_status == 1)in_ring_status = 2;
		return 1;
	
	case OAPI_KEY_B: // light
		if (light_sts == 0)
		{
			light_sts = 1;
			sts_beacon.active = true;
			spotlight1->Activate(true);
		}
		else if (light_sts == 1)
		{
			light_sts = 0;
			sts_beacon.active = false;
			spotlight1->Activate(false);
		}
		return 1;


		case OAPI_KEY_F5: // show att points
		if (oapiGetShowGrapplePoints () == true) oapiSetShowGrapplePoints (false);
		else oapiSetShowGrapplePoints (true);
		return 1;


		}
	}
  return 0;
}




void hst_ex_sm1::clbkPostCreation()
{


}

void hst_ex_sm1::clbkPreStep(double SimT, double SimDT, double MJD)
{

	////// set docking ring att point
	double cos_theta = cos(PI*0.5*out_ring_proc);
	double sin_theta = sin(PI*0.5*out_ring_proc);

	double new_z = (0.27*cos_theta) - (-0.778*sin_theta) - 3.77;
	double new_y = (-0.778*cos_theta) + (0.27*sin_theta) + 2.315;

	VECTOR3 new_pos = _V(0, new_y, new_z);
	VECTOR3 new_dir = _V(0,sin_theta,cos_theta);

	double ct = cos(2*PI*in_ring_proc);
	double st = sin(2*PI*in_ring_proc);

	VECTOR3 rot_axis = new_dir;
	normalise(rot_axis);

	double ux = rot_axis.x;
	double uy = rot_axis.y;
	double uz = rot_axis.z;
	double ux2 = ux*ux;
	double uy2 = uy*uy;
	double uz2 = uz*uz;

	MATRIX3 RG1;

	RG1.m11 = ux2+(1-ux2)*ct;
	RG1.m12 = ux*uy*(1-ct)-uz*st;
	RG1.m13 = ux*uz*(1-ct)+uy*st;
	RG1.m21 = ux*uy*(1-ct)+uz*st;
	RG1.m22 = uy2+(1-uy2)*ct;
	RG1.m23 = uy*uz*(1-ct)-ux*st;
	RG1.m31 = ux*uz*(1-ct)-uy*st;
	RG1.m32 = uy*uz*(1-ct)+ux*st;
	RG1.m33 = uz2+(1-uz2)*ct;

	VECTOR3 new_rot = mul(RG1,_V(1,0,0));

	SetAttachmentParams(hst_base_att,new_pos,new_dir,new_rot);

	//////////////////////////////

	/////////// set beacons
if (oapiGetShowGrapplePoints () == true) DoBeacons();
else
{
for (int i = 0; i < 6; i++) 
	{
	beacon[i].active = false;
	}
}

	////////////


step_time = SimDT;///////


if (cv1cv2_check < 2)CheckVessels();
else if (cv1cv2_check > 1)CheckVessels2();


if(att_pc_check == 0)att_pc_check = 1;
else if(att_pc_check == 1)att_pc_check = 0;


//sprintf(oapiDebugString(), "%d %0.4f %0.4f", (sa1_stbd_elb_sts + sa1_stbd_scale_sts + sa1_port_elb_sts + sa1_port_scale_sts),sa1_port_rot_proc,sa1_stbd_rot_proc);

////////////////// ANIMATIONS ///////////////

	SetAnimation (anim_rsu, rsu_proc);
	SetAnimation (anim_in_ring, in_ring_proc);
	SetAnimation (anim_out_ring, out_ring_proc);

/// asipe
if (asipe_status > 0)
{
double da = SimDT * door_dur;
if (asipe_status == 1) asipe_proc = asipe_proc + da;
else if (asipe_status == 3) asipe_proc = asipe_proc - da;

if (asipe_proc > 1)
	{
	if (asipe_status == 1) asipe_status = 2;
	asipe_proc = 1;
	}

else if (asipe_proc < 0)
	{
	if (asipe_status == 3) asipe_status = 0;
	asipe_proc = 0;
	}
SetAnimation (anim_asipe, asipe_proc);
}

////// rsipe
if (rsipe_status > 0)
{
double da = SimDT * door_dur;
if (rsipe_status == 1) rsipe_proc = rsipe_proc + da;
else if (rsipe_status == 3) rsipe_proc = rsipe_proc - da;

if (rsipe_proc > 1)
	{
	if (rsipe_status == 1) rsipe_status = 2;
	rsipe_proc = 1;
	}

else if (rsipe_proc < 0)
	{
	if (rsipe_status == 3) rsipe_status = 0;
	rsipe_proc = 0;
	}
SetAnimation (anim_rsipe, rsipe_proc);
}

////// rsu
if (rsu_status > 0)
{
double da = SimDT * door_dur;
if (rsu_status == 1) rsu_proc = rsu_proc + da;
else if (rsu_status == 3) rsu_proc = rsu_proc - da;

if (rsu_proc > 1)
	{
	if (rsu_status == 1) rsu_status = 2;
	rsu_proc = 1;
	}

else if (rsu_proc < 0)
	{
	if (rsu_status == 3) rsu_status = 0;
	rsu_proc = 0;
	}
SetAnimation (anim_rsu, rsu_proc);
}


////// outer ring
if (out_ring_status > 0)
{
double da = SimDT * door_dur;
if (out_ring_status == 1) out_ring_proc = out_ring_proc + da;
else if (out_ring_status == 3) out_ring_proc = out_ring_proc - da;

if (out_ring_proc > 1)
	{
	if (out_ring_status == 1) out_ring_status = 2;
	out_ring_proc = 1;
	}

else if (out_ring_proc < 0)
	{
	if (out_ring_status == 3) out_ring_status = 0;
	out_ring_proc = 0;
	}
SetAnimation (anim_out_ring, out_ring_proc);
}

////// inner ring
if (in_ring_status > 0)
{
double da = SimDT * rot_dur;
if (in_ring_status == 1) in_ring_proc = in_ring_proc + da;
else if (in_ring_status == 2) in_ring_proc = in_ring_proc - da;

if (in_ring_proc > 1)in_ring_proc = in_ring_proc - ((int)(in_ring_proc));
else if (in_ring_proc < 0)in_ring_proc = 1 + in_ring_proc - ((int)(in_ring_proc));

SetAnimation (anim_in_ring, in_ring_proc);
}


/////////////////////// on screen note
if(anot_status > 0)
{

if (anot_status == 1)
	{
char imstr[256];
strcpy(imstr, "SECURED");
VECTOR3 col = _V(0.8,0.7,0.1);
hNote = oapiCreateAnnotation(true, 1, col);
oapiAnnotationSetPos(hNote, 0.468, 0.0, 0.66, 0.05);
oapiAnnotationSetText(hNote, imstr);
anot_status = 2;
	}

if(anot_status == 2)
{
anot_timer = anot_timer + oapiGetSysStep();
if (anot_timer > 2)
	{
	anot_status = 3;
	anot_timer = 0;
	}

}

if(anot_status == 3)
	{
	anot_status = 0;
	oapiDelAnnotation (hNote);
	}


}





}




bool hst_ex_sm1::clbkDrawHUD (int mode, const HUDPAINTSPEC *hps, oapi::Sketchpad *skp)
{
	// draw the default HUD
	VESSEL3::clbkDrawHUD (mode, hps, skp);

	oapiSetDefNavDisplay(0);
	oapiSetDefRCSDisplay(0);

	int s = hps->H;
	double d = (s*0.00130208);
	int sw = ((hps->W));
	int lw =  (int)(16*sw/1024);
	int lwoffset = sw - (18*lw);
	int hlw = (int) (lw/2);


	int roxl = 0;
	int royl = 0;

	double ds = s;
	double dsw = sw;
	double sc_ratio = ds/dsw;


	if (sc_ratio < 0.7284)
	{
	roxl = (lw*10);
	royl = (int) (-88*d);
	}



	int wd = (int)(152*d);
	int wc = (int)(168*d);
	int w0 = (int)(184*d);
	int w1 = (int) (200*d);
	int w2 = (int)(216*d);
	int w3 = (int)(232*d);
	int w4 = (int)(248*d);
	int w5 = (int)(264*d);
	int w6 = (int)(280*d);
	int w7 = (int)(296*d);
	int w8 = (int)(312*d);
	int w9 = (int)(328*d);
	int w10 = (int)(344*d);
	int w11 = (int)(360*d);
	int w12 = (int)(376*d);
	int w13 = (int)(392*d);
	int w14 = (int)(408*d);
	int w15 = (int)(424*d);

	if (out_ring_status == 0)
	{
	skp->SetTextColor(0x0066FF66);
	char abuf[256];
	sprintf(abuf,"[G] = Deploy HST Dock Ring");
	skp->Text ((10+roxl), (wc+royl), abuf,strlen(abuf));
	}

	if (out_ring_status == 2)
	{
	skp->SetTextColor(0x0066FF66);
	char abuf[256];
	sprintf(abuf,"[G] = Stow HST Dock Ring");
	skp->Text ((10+roxl), (wc+royl), abuf,strlen(abuf));
	}

	if (in_ring_status == 1)
	{
	skp->SetTextColor(0x0066FFFF);
	char abuf[256];
	sprintf(abuf,"[N] = HST Dock Rot.+ On/Off");
	skp->Text ((10+roxl), (w0+royl), abuf,strlen(abuf));
	}
	else
	{
	skp->SetTextColor(0x0066FF66);
	char abuf[256];
	sprintf(abuf,"[N] = HST Dock Rot.+ On/Off");
	skp->Text ((10+roxl), (w0+royl), abuf,strlen(abuf));
	}

	if (in_ring_status == 2)
	{
	skp->SetTextColor(0x0066FFFF);
	char abuf[256];
	sprintf(abuf,"[M] = HST Dock Rot.- On/Off");
	skp->Text ((10+roxl), (w1+royl), abuf,strlen(abuf));
	}
	else
	{
	skp->SetTextColor(0x0066FF66);
	char abuf[256];
	sprintf(abuf,"[M] = HST Dock Rot.- On/Off");
	skp->Text ((10+roxl), (w1+royl), abuf,strlen(abuf));
	}

	if (light_sts == 1)
	{
	skp->SetTextColor(0x0066FFFF);
	char abuf[256];
	sprintf(abuf,"[B] = Cargobay Light On/Off");
	skp->Text ((10+roxl), (w2+royl), abuf,strlen(abuf));
	}
	else
	{
	skp->SetTextColor(0x0066FF66);
	char abuf[256];
	sprintf(abuf,"[B] = Cargobay Light On/Off");
	skp->Text ((10+roxl), (w2+royl), abuf,strlen(abuf));
	}

	if (GetAttachmentStatus(ft_pad_att) == NULL)
	{
	skp->SetTextColor(0x0066FF66);
	char abuf[256];
	sprintf(abuf,"[J] = Spawn EVA1 Astronaut");
	skp->Text ((10+roxl), (w3+royl), abuf,strlen(abuf));
	}

	if (GetAttachmentStatus(ft_pad_att2) == NULL)
	{
	skp->SetTextColor(0x0066FF66);
	char abuf[256];
	sprintf(abuf,"[K] = Spawn EVA2 Astronaut");
	skp->Text ((10+roxl), (w4+royl), abuf,strlen(abuf));
	}

	{
	skp->SetTextColor(0x0066FF66);
	char abuf[256];
	sprintf(abuf,"[F5] = Show/Hide Attachment Points");
	skp->Text ((10+roxl), (w5+royl), abuf,strlen(abuf));
	}

	


	return true;
}



void hst_ex_sm1::DoBeacons(void)
{

if (asipe_status == 0)beacon[0].active = true;
else beacon[0].active = false;
if (asipe_status == 2)beacon[3].active = true;
else beacon[3].active = false;

if (rsipe_status == 0)beacon[1].active = true;
else beacon[1].active = false;
if (rsipe_status == 2)beacon[4].active = true;
else beacon[4].active = false;

if (rsu_status == 0)beacon[2].active = true;
else beacon[2].active = false;
if (rsu_status == 2)beacon[5].active = true;
else beacon[5].active = false;


}
void hst_ex_sm1::CheckVessels(void)
{
if (cv1cv2_check == 0)cv1cv2_check = 1;
else if (cv1cv2_check == 1)cv1cv2_check = 2;


VECTOR3 pos = _V(0,0,0);
VECTOR3 dir = _V(0,0,0);
VECTOR3 rot = _V(0,0,0);
VECTOR3 att_glob = _V(0,0,0);

//// doors
VECTOR3 sm_pnt_glob[15];
for (int i = 0; i < 6; i++)
{
Local2Global(beaconpos[i], sm_pnt_glob[i]);
}

///// grap points
for (int i = 0; i < 9; i++)
{
Local2Global(attp_pos[i], sm_pnt_glob[(i+6)]);
}


int num_ves = oapiGetVesselCount();
for (int i = 0; i < num_ves; i++) 
{
VECTOR3 hst_pos_glob = _V(0,0,0);
GetGlobalPos(hst_pos_glob);
OBJHANDLE hVessel = oapiGetVesselByIndex(i);
if (hVessel == GetHandle()) continue;

VECTOR3 ves_pos_glob = _V(0,0,0);
oapiGetGlobalPos(hVessel, &ves_pos_glob);
if (dist(ves_pos_glob, hst_pos_glob)> 20)continue;

VESSEL *v = oapiGetVesselInterface (hVessel);
DWORD nAttach;
if (att_pc_check == 0)nAttach = v->AttachmentCount (false);
else nAttach = v->AttachmentCount (true);

if (nAttach < 1)continue;


for (DWORD j = 0; j < nAttach; j++)
{
ATTACHMENTHANDLE hAtt;
if(att_pc_check == 0) hAtt = v->GetAttachmentHandle (false, j);
else hAtt = v->GetAttachmentHandle (true, j);
v->GetAttachmentParams (hAtt, pos, dir, rot);
v->Local2Global (pos, att_glob);


//////// doors /////////
if(asipe_status == 0)
{
	if (dist(sm_pnt_glob[0], att_glob) < 0.1)
		{
			asipe_status = 1;
			return;
		}

}
if(asipe_status == 2)
{
	if (dist(sm_pnt_glob[3], att_glob) < 0.1)
		{
			asipe_status = 3;
			return;
		}

}


if(rsipe_status == 0)
{
	if (dist(sm_pnt_glob[1], att_glob) < 0.1)
		{
			rsipe_status = 1;
			return;
		}

}
if(rsipe_status == 2)
{
	if (dist(sm_pnt_glob[4], att_glob) < 0.1)
		{
			rsipe_status = 3;
			return;
		}

}

if(rsu_status == 0)
{
	if (dist(sm_pnt_glob[2], att_glob) < 0.1)
		{
			rsu_status = 1;
			return;
		}

}
if(rsu_status == 2)
{
	if (dist(sm_pnt_glob[5], att_glob) < 0.1)
		{
			rsu_status = 3;
			return;
		}

}
	



	

/////// axial cos
if (GetAttachmentStatus(ax_att) == NULL)
{
	if (dist(sm_pnt_glob[6], att_glob) < 0.2)	
	//sprintf(oapiDebugString(), "length %0.2f", dist(hst_attp_glob[0], att_glob));
	{
		const char *id = v->GetAttachmentId (hAtt);
		if (strncmp (id, "IN", 2)); // attachment point not compatible
		else 
			{
			int atchk = 0;
			DWORD natt = v->AttachmentCount (true);
			for (DWORD k = 0; k < natt; k++)
				{
				ATTACHMENTHANDLE atth = v->GetAttachmentHandle (true, k);
				if (!v->GetAttachmentStatus(atth) == NULL) atchk = 1;
				}

				if (atchk == 0)
				{
				AttachChild(hVessel,ax_att,hAtt);
				anot_status = 1;
				return;
				}

			}
	}
}


		}
	}
}


void hst_ex_sm1::CheckVessels2(void)
{
if (cv1cv2_check == 2)cv1cv2_check = 3;
else if (cv1cv2_check == 3)cv1cv2_check = 0;


VECTOR3 pos = _V(0,0,0);
VECTOR3 dir = _V(0,0,0);
VECTOR3 rot = _V(0,0,0);
VECTOR3 att_glob = _V(0,0,0);

//// doors
VECTOR3 sm_pnt_glob[9];
for (int i = 0; i < 8; i++)
{
Local2Global(attp_pos[(i+1)], sm_pnt_glob[i]);
}
Local2Global(attp_pos[(11)], sm_pnt_glob[8]);

int num_ves = oapiGetVesselCount();
for (int i = 0; i < num_ves; i++) 
{
VECTOR3 hst_pos_glob = _V(0,0,0);
GetGlobalPos(hst_pos_glob);
OBJHANDLE hVessel = oapiGetVesselByIndex(i);
if (hVessel == GetHandle()) continue;

VECTOR3 ves_pos_glob = _V(0,0,0);
oapiGetGlobalPos(hVessel, &ves_pos_glob);
if (dist(ves_pos_glob, hst_pos_glob)> 20)continue;

VESSEL *v = oapiGetVesselInterface (hVessel);
DWORD nAttach;
if (att_pc_check == 0)nAttach = v->AttachmentCount (false);
else nAttach = v->AttachmentCount (true);

if (nAttach < 1)continue;


for (DWORD j = 0; j < nAttach; j++)
{
ATTACHMENTHANDLE hAtt;
if(att_pc_check == 0) hAtt = v->GetAttachmentHandle (false, j);
else hAtt = v->GetAttachmentHandle (true, j);
v->GetAttachmentParams (hAtt, pos, dir, rot);
v->Local2Global (pos, att_glob);


/////// radial wfc
if (GetAttachmentStatus(rad_att) == NULL)
{
	if (dist(sm_pnt_glob[0], att_glob) < 0.2)	
	
	{
		const char *id = v->GetAttachmentId (hAtt);
		if (strncmp (id, "WF", 2)); // attachment point not compatible
		else 
			{
			int atchk = 0;
			DWORD natt = v->AttachmentCount (true);
			for (DWORD k = 0; k < natt; k++)
				{
				ATTACHMENTHANDLE atth = v->GetAttachmentHandle (true, k);
				if (!v->GetAttachmentStatus(atth) == NULL) atchk = 1;
				}

				if (atchk == 0)
				{
				AttachChild(hVessel,rad_att,hAtt);
				anot_status = 1;
				return;
				}

			}
	}
}


/////// rsu1
if (GetAttachmentStatus(rsu1_att) == NULL)
{
	if (dist(sm_pnt_glob[1], att_glob) < 0.2)	
	
	{
		const char *id = v->GetAttachmentId (hAtt);
		if (strncmp (id, "RS", 2)); // attachment point not compatible
		else 
			{
			int atchk = 0;
			DWORD natt = v->AttachmentCount (true);
			for (DWORD k = 0; k < natt; k++)
				{
				ATTACHMENTHANDLE atth = v->GetAttachmentHandle (true, k);
				if (!v->GetAttachmentStatus(atth) == NULL) atchk = 1;
				}

				if (atchk == 0)
				{
				AttachChild(hVessel,rsu1_att,hAtt);
				anot_status = 1;
				return;
				}

			}
	}
}


/////// rsu2
if (GetAttachmentStatus(rsu1_att) == NULL)
{
	if (dist(sm_pnt_glob[2], att_glob) < 0.2)	
	
	{
		const char *id = v->GetAttachmentId (hAtt);
		if (strncmp (id, "RS", 2)); // attachment point not compatible
		else 
			{
			int atchk = 0;
			DWORD natt = v->AttachmentCount (true);
			for (DWORD k = 0; k < natt; k++)
				{
				ATTACHMENTHANDLE atth = v->GetAttachmentHandle (true, k);
				if (!v->GetAttachmentStatus(atth) == NULL) atchk = 1;
				}

				if (atchk == 0)
				{
				AttachChild(hVessel,rsu2_att,hAtt);
				anot_status = 1;
				return;
				}

			}
	}
}


/////// rsu3
if (GetAttachmentStatus(rsu1_att) == NULL)
{
	if (dist(sm_pnt_glob[3], att_glob) < 0.2)	
	
	{
		const char *id = v->GetAttachmentId (hAtt);
		if (strncmp (id, "RS", 2)); // attachment point not compatible
		else 
			{
			int atchk = 0;
			DWORD natt = v->AttachmentCount (true);
			for (DWORD k = 0; k < natt; k++)
				{
				ATTACHMENTHANDLE atth = v->GetAttachmentHandle (true, k);
				if (!v->GetAttachmentStatus(atth) == NULL) atchk = 1;
				}

				if (atchk == 0)
				{
				AttachChild(hVessel,rsu3_att,hAtt);
				anot_status = 1;
				return;
				}

			}
	}
}


/////// sas
if (GetAttachmentStatus(sa_stbd_att) == NULL)
{
	if (dist(sm_pnt_glob[4], att_glob) < 0.2)	
	
	{
		const char *id = v->GetAttachmentId (hAtt);
		if (strncmp (id, "S1", 2)); // attachment point not compatible
		else 
			{
			int atchk = 0;
			DWORD natt = v->AttachmentCount (true);
			for (DWORD k = 0; k < natt; k++)
				{
				ATTACHMENTHANDLE atth = v->GetAttachmentHandle (true, k);
				if (!v->GetAttachmentStatus(atth) == NULL) atchk = 1;
				}

				if (atchk == 0)
				{
				AttachChild(hVessel,sa_stbd_att,hAtt);
				anot_status = 1;
				return;
				}

			}
	}
}


/////// sap
if (GetAttachmentStatus(sa_port_att) == NULL)
{
	if (dist(sm_pnt_glob[5], att_glob) < 0.2)	
	
	{
		const char *id = v->GetAttachmentId (hAtt);
		if (strncmp (id, "S1", 2)); // attachment point not compatible
		else 
			{
			int atchk = 0;
			DWORD natt = v->AttachmentCount (true);
			for (DWORD k = 0; k < natt; k++)
				{
				ATTACHMENTHANDLE atth = v->GetAttachmentHandle (true, k);
				if (!v->GetAttachmentStatus(atth) == NULL) atchk = 1;
				}

				if (atchk == 0)
				{
				AttachChild(hVessel,sa_port_att,hAtt);
				anot_status = 1;
				return;
				}

			}
	}
}

/////// sat
if (GetAttachmentStatus(sa_temp_att) == NULL)
{
	if (dist(sm_pnt_glob[6], att_glob) < 0.2)	
	
	{
		const char *id = v->GetAttachmentId (hAtt);
		if (strncmp (id, "S1", 2)); // attachment point not compatible
		else 
			{
			int atchk = 0;
			DWORD natt = v->AttachmentCount (true);
			for (DWORD k = 0; k < natt; k++)
				{
				ATTACHMENTHANDLE atth = v->GetAttachmentHandle (true, k);
				if (!v->GetAttachmentStatus(atth) == NULL) atchk = 1;
				}

				if (atchk == 0)
				{
				AttachChild(hVessel,sa_temp_att,hAtt);
				anot_status = 1;
				return;
				}

			}
	}
}

/////// hst
if (out_ring_status == 2)
{
if (GetAttachmentStatus(hst_base_att) == NULL)
{
	if (dist(sm_pnt_glob[7], att_glob) < 0.2)	
	{
		const char *id = v->GetAttachmentId (hAtt);
		if (strncmp (id, "SCM", 3)); // attachment point not compatible
		else 
			{
			int atchk = 0;
			DWORD natt = v->AttachmentCount (true);
			for (DWORD k = 0; k < natt; k++)
				{
				ATTACHMENTHANDLE atth = v->GetAttachmentHandle (true, k);
				if (!v->GetAttachmentStatus(atth) == NULL) atchk = 1;
				}

				if (atchk == 0)
				{
				AttachChild(hVessel,hst_base_att,hAtt);
				anot_status = 1;
				return;
				}

			}
	}
}
}

/////// hldr
if (GetAttachmentStatus(hldr_att) == NULL)
{
	if (dist(sm_pnt_glob[8], att_glob) < 0.2)	
	
	{
		const char *id = v->GetAttachmentId (hAtt);
		if (strncmp (id, "GSHLD", 5)); // attachment point not compatible
		else 
			{
			int atchk = 0;
			DWORD natt = v->AttachmentCount (true);
			for (DWORD k = 0; k < natt; k++)
				{
				ATTACHMENTHANDLE atth = v->GetAttachmentHandle (true, k);
				if (!v->GetAttachmentStatus(atth) == NULL) atchk = 1;
				}

				if (atchk == 0)
				{
				AttachChild(hVessel,hldr_att,hAtt);
				anot_status = 1;
				return;
				}

			}
	}
}


/////
		}
	}
}

void hst_ex_sm1::SpawnEVA(void)
{
	if (GetAttachmentStatus(ft_pad_att) == NULL)
	{
	SetAttachmentParams(ft_pad_att, attp_pos[9], _V(0,1,0), _V(0,0,-1));
	VESSELSTATUS2 vs;
	memset (&vs, 0, sizeof(vs));
	vs.version = 2;
	GetStatusEx(&vs);


	OBJHANDLE hWFC = oapiCreateVesselEx ("EVA1", "HST_EX/ev1", &vs);

	
	VESSEL *v = oapiGetVesselInterface (hWFC);
	ATTACHMENTHANDLE hAttWFC = v->GetAttachmentHandle(true,0);
	AttachChild(hWFC, ft_pad_att, hAttWFC);
	}

}

void hst_ex_sm1::SpawnEVA2(void)
{
	if (GetAttachmentStatus(ft_pad_att2) == NULL)
	{
	SetAttachmentParams(ft_pad_att2, attp_pos[10], _V(0,-1,0), _V(0,0,-1));
	VESSELSTATUS2 vs;
	memset (&vs, 0, sizeof(vs));
	vs.version = 2;
	GetStatusEx(&vs);


	OBJHANDLE hWFC = oapiCreateVesselEx ("EVA2", "HST_EX/ev2", &vs);

	
	VESSEL *v = oapiGetVesselInterface (hWFC);
	ATTACHMENTHANDLE hAttWFC = v->GetAttachmentHandle(true,0);
	AttachChild(hWFC, ft_pad_att2, hAttWFC);
	}

}
